#!/usr/local/bin/perl
#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************
# =========================================================
# dbamon_ora_sum.pl
#
# Changes:
# 
#   bb 12Jun00 - Initial
# =========================================================

$debug = 1;

$| = 1;
$P = "dbamon_www_email.pl";
$ENV{'PATH'} = "/opt/dbamon/bin:$ENV{'PATH'}";
$os = $ENV{"ORACLE_SID"};

#------------------------------------------------------------------------------
sub trim
#------------------------------------------------------------------------------
{
  my @out = @_;
  for (@out)
  {
    s/^\s+//;
    s/\s+$//;
  }
  return wantarray ? @out : $out[0];
}

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  $msgmsg = $_[0];
  chop($date = `date +%Y/%m/%d-%H:%M:%S`);
  if ($debug) { print "$date ${P} | $msgmsg\n"; }
}

#------------------------------------------------------------------------------
sub bg
#------------------------------------------------------------------------------
{
  if ($lastbg == 1)
  {
    $lastbg = 0;
    return "bgcolor=ffffff"; 
  }
  else
  {
    $lastbg = 1;
    return "bgcolor=lightgrey"; 
  }
}

#------------------------------------------------------------------------------
sub error
#------------------------------------------------------------------------------
{
  $msg = $_[0];
  &msg("=============================================================");
  &msg("Fatal Error: $msg");
  &msg("=============================================================");
  &msg("Exiting rc=1");
  exit 1;
}

#------------------------------------------------------------------------------
sub connectDB
#------------------------------------------------------------------------------
{
  &msg("Connecting to DBAmon Oracle Repository DB to read DBC data");
  use DBI;
  $user   = "dbatools";
  $passwd = "br0nc0s";
  $data_source = "DBI:Oracle";
  $database = $os;

  $dbh = DBI->connect("$data_source:$database", $user, $passwd) ||
    &error('Failed_on_DB_connect');
  $dbh->{LongReadLen} = 4000;
}

#-----------------------------------------------------------------------------
sub DDReadFile
#-----------------------------------------------------------------------------
{
  $ddfile = "/home/dbamon/dd.txt";
  
  #---- Get high version number from file
  chop($dd_highver = `cat $ddfile | awk '{ print \$1 }' | sort -u | tail -1`); 
  &msg("High version from ddfile=$ddfile - dd_highver=$dd_highver"); 

  $dd_rowcount = &DDGetRowCount($dd_highver);

  $dd_cnt = 0;
  $dd_sql = ""; 

  foreach (`cat $ddfile`)
  {
    @words = split;
    $dd_ver  = $words[0]; 

    next if $dd_ver ne $dd_highver;

    $dd_word = $words[1]; 

    $dd_word =~ s/'/"/g;

    push(@dd_sql, "insert into DBAMON.PWDICT (pw_ver, pw_word) values ('$dd_ver','$dd_word')\n"); 
    $dd_cnt++; 
  }

  &msg("Found $dd_cnt eligible words - dd_rowcount=$dd_rowcount"); 

  if ($dd_cnt != $dd_rowcount)
  {
    &msg("Since the number of rows in DBAMON.PWDICT ($dd_rowcount) != Words in file ($dd_cnt) Rows will be inserted"); 
    &DDInsert; 
  }
  else
  {
    &msg("Since the number of rows in DBAMON.PWDICT ($dd_rowcount) = Words in file ($dd_cnt) Rows will NOT be inserted"); 
  }
}

#-----------------------------------------------------------------------------
sub DDInsert
#-----------------------------------------------------------------------------
{
  &msg("Deleting from DBAMON.PWDICT where PW_VER=$dd_ver");
  $sql = "delete from DBAMON.PWDICT where PW_VER = '$dd_ver'"; 
  $sth = $dbh->prepare($sql) || &error('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &error('Failed_on_DB_execute_in_chgEnt');

  $dbh->{AutoCommit} = 0;

  &msg("Inserting SQL from dd_sql into DBAMON.PWDICT (1 dot per 1000 rows)");
  foreach $sql (@dd_sql)
  {
    $sth = $dbh->prepare($sql) || &error('Failed_on_DB_prepare_in_chgEnt');
    $sth->execute || &error('Failed_on_DB_execute_in_chgEnt');

    $insertcnt++; 

    print "." if $insertcnt / 1000 == int($insertcnt / 1000); 
  }

  $dbh->commit();
  $dbh->{AutoCommit} = 1;

  print "\n"; 
  &msg("Inserted $insertcnt rows"); 

  &msg("Running analyze on DBAMON.PWDICT");
  $sql = "analyze table DBAMON.PWDICT compute statistics"; 
  $sth = $dbh->prepare($sql) || &error('prepare_analyze');
  $sth->execute || &error('execute_analyze');
}

#-----------------------------------------------------------------------------
sub DDCreateTable
#-----------------------------------------------------------------------------
{
  &msg("Creating DBAMON.PWDICT table");
  $sql = "
    create table dbamon.pwdict
      (
        pw_ver   varchar2(8)  not null,
        pw_word  varchar2(24) not null,
        pw_ts    date         default sysdate
      ) 
  "; 
  $sth = $dbh->prepare($sql) || &error('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &error('Failed_on_DB_execute_in_chgEnt');

  $sql = "create index dbamon.pwdict_x1 on dbamon.pwdict (pw_word)"; 
  $sth = $dbh->prepare($sql) || &error('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &error('Failed_on_DB_execute_in_chgEnt');

  $sql = "create index dbamon.pwdict_x2 on dbamon.pwdict (pw_ver)"; 
  $sth = $dbh->prepare($sql) || &error('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &error('Failed_on_DB_execute_in_chgEnt');
}

#-----------------------------------------------------------------------------
sub DDGetRowCount
#-----------------------------------------------------------------------------
{
  $dd_thisver = $_[0];
  &msg("Selecting data for dd_thisver=$dd_thisver");

  $sql  = "
    select 
      count(*)
    from 
      dbamon.pwdict
    where
      pw_ver = '$dd_thisver' 
  ";
  $rc = $sth = $dbh->prepare($sql); 
  if ($rc != 1)
  {
    $sqlerror = $DBI::errstr;
    ##&msg("Encountered sqlerror=($sqlerror)"); 
    if ($sqlerror =~ /ORA-00942/)
    {
      &DDCreateTable; 
      return 0; 
    }
    elsif ($sqlerror ne "") 
    {
      &error("GetVerError");
    }
  }

  $sth->execute || &error('Failed_on_DB_execute_in_chgEnt');
 
  $this_rowcount = 0; 

  while (@Data = $sth->fetchrow_array)
  {
    $this_rowcount = $Data[0];
  }

  return $this_rowcount; 
}

#-----------------------------------------------------------------------------
sub init
#-----------------------------------------------------------------------------
{
  $rcfile = "/opt/dbamon/adm/dbamonrc";
  
  chop($os = `grep ORACLE_SID:  $rcfile | awk '{ print \$2}'`);
  chop($oh = `grep ORACLE_HOME: $rcfile | awk '{ print \$2}'`);

  ##&msg("Setting Oracle Env.");
  $ENV{"ORACLE_SID"} = $os;
  $ENV{"ORACLE_HOME"} = $oh;
  ##system("env | grep ORACLE");

  #chop($hostname = `hostname`);
  #chop($date = `date`);
  #chop($wsname = `hostname`);
  #chop($wsnamef = `nslookup $wsname | grep Name: | awk '{ print \$2 }'`);

}
#------------------------------------------------------------------------------
#
# Main
#
#------------------------------------------------------------------------------

&msg("Starting"); 

&init;
&connectDB;

&DDReadFile; 

&msg("Done"); 
exit;
