#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************
#######################################################################
#
#
#   "onflags" is used to replicate logical log file information,
#   specifically flag info, in a format more consistent with the output
#   of "onstat -l" prior to INFORMIX-OnLine Dynamic Server 7.10.UD1.
#
#
#   Usage:   onflags
#
#######################################################################

#
#  This function retrieves the logical log file numbers from 
#  sysmaster:syslogs.
#

get_filenums ()
{
X=`dbaccess sysmaster <<EOF 2> /dev/null
select number from syslogs;
EOF`
set `echo $X`
shift
echo $*
}

#
#  This function retrieves the loguniq of the logical log that contains
#  the "begin work;" of the oldest transaction via SMI.
#

get_min_loguniq ()
{
X=`dbaccess sysmaster <<EOF 2> /dev/null
select min(tx_loguniq) from systrans where tx_loguniq <> 0;
EOF`
set `echo $X`
shift
echo $*
}

#
#  This function retrieves information about a specific logical log file
#  from sysmaster:syslogs indexed by the logical log number.
#
#  NOTE: Function takes a logical log file number as an argument.
#

get_fileinfo ()
{
dbaccess sysmaster <<EOF 2> /dev/null
select uniqid, size, used, is_used, is_current, is_backed_up, is_new
   from syslogs
   where number = $1;
EOF
}

#
#  Main
#

#
#  Display header info...
#

onstat -
echo "Logical Logging"
echo
echo "number	flags	uniqid	    size	used	    %used"

#
#  Get oldest unique id with active transaction.
#

MIN_LOGUNIQ=`get_min_loguniq`

#
#  Get number of logical log files in system
#

FILENUMS=`get_filenums`

#
#  Get the log containing the last check point record.
#  Couldn't find a better way of getting the following..."
#

LASTCHKPT=`onstat -l | grep -- "-L " | awk ' { print $4 } '`

#
#  This is the payoff.  Generate a flags field similar to that shown
#  prior to ODS 7.10.UD1 based on info retrieved from sysmaster:syslogs.
#

for i in $FILENUMS
do

   FILEINFO=`get_fileinfo $i`

   set `echo $FILEINFO`
   shift

   UNIQID=$1
   shift; shift

   SIZE=$1
   shift; shift

   USED=$1
   shift; shift

   IS_USED=$1
   shift; shift

   IS_CURRENT=$1
   shift; shift

   IS_BACKED_UP=$1
   shift; shift

   IS_NEW=$1

   FLAG2=-
   FLAG4=-
   FLAG6=-
   if [ $IS_USED -eq 1 ]
   then
      FLAG0=U
      if [ $IS_BACKED_UP -eq 1 ]
      then
	  FLAG2=B
          if [ $UNIQID -lt $MIN_LOGUNIQ ]
          then
	     FLAG0=F
	     FLAG2=-
          fi
      fi
      if [ $IS_CURRENT -eq 1 ]
      then
         FLAG4=C
      fi
      if [ $UNIQID -eq $LASTCHKPT ]
      then
         FLAG6=L
      fi
   else
      if [ $IS_NEW -eq 1 ]
      then
         FLAG0=A
      else
	 FLAG0=F
      fi
   fi

   FLAGS="${FLAG0}-${FLAG2}-${FLAG4}-${FLAG6}"

#
#  Print it out!
#

   PCTUSED="`echo $USED $SIZE | awk ' { printf "%6.2f\n", $1 * 100.00 / $2
} '`"

   echo "$i	$FLAGS	$UNIQID	    $SIZE		$USED	    $PCTUSED"

done
echo

