#!/usr/bin/perl
#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************
#------------------------------------------------------------------------------
# dbamon_make_ticket.pl
#
# Changes:
# 
#   bb 02Feb97 -
#   nb 21Jun99 - Added code to create BLT tickets.
#   vd 19Sep02 - Added code to generate ITO tickets 
# 2.02 bb 03Oct02 - Added code to suppress blt tickets for the events
#   where ITO is going to combine events for 1 server. However it is 
#   commented now until John Bathelt gets ito release problems resolved
#   on all servers.
# 2.03 bb 07Oct02 - Added ID3 code and removed ID2 code
#------------------------------------------------------------------------------
$version = "2.03";

$P = "dbamon_make_ticket.pl";
$tmp1 = "/tmp/${P}_1_$$.txt";

# =========================================================
# Arguments
# =========================================================

# =========================================================
# Global vars
# =========================================================

chop($dbamonhost = `hostname`);
$| = 1;
#$rand = substr(time/$$,8,4);

# =========================================================
# Subs
# =========================================================

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  $msgmsg = $_[0];
  chop($date = `date +%Y/%m/%d-%H:%M:%S`);
  print "$date ${P} $version | $msgmsg\n";
}

#------------------------------------------------------------------------------
sub blt_ticket 
#------------------------------------------------------------------------------
{
  &msg("Creating BLT Ticket");

  $msg = $ENV{"ERRMSG"};

  ##print "msg b4=$msg\n";
  $msg =~ s/:/-/;
  $_ = $msg;
  s/!!!//g;
  s/   //g;
  s/  //g;
  s/  //g;
  #s/ \| /\|/g;
  s/\| /\|/g;
  s/ \|/\|/g;
  s/ \|/\|/g;
  s/DBAmon\|//g;
  $msg = $_;
  ##print "msg af=$msg\n";

  ##system("env > /tmp/env");
  $efile = $ENV{"ERRFILE"};
  $msgtext = `cat $efile`;

  $email="dbagap\@bigdog.cs.itc.hp.com";
  $teamname = "team_database informix";
  $reqtype  = "database informix";
  $phone="590-3402";
  chop($bltserver = `/usr/local/dba/tools/bltserver`);

  &msg("Creating Ticket Host=$ENV{'HOST'} bltserver=$bltserver Subject=$msg");

  $login="csaitsystemsmail";
  $site="cs ait-systems";
  $subrequest="exception monitoring";
  $action="troubleshoot/repair";
  $priority="4";

  if ($ENV{"DBMS"} eq "Oracle")
  {
    $teamname = "team_database oracle";
    $reqtype  = "database oracle";
  }
  if ($ENV{"DBMS"} eq "OracleNT")
  {
    $teamname = "team_database oracle";
    $reqtype  = "database oracle";
  }
  if ($ENV{"DBMS"} eq "MSSQL")
  {
    $teamname = "team_database mssql";
    $reqtype  = "database sql server";
  }
  if ($ENV{"DBMS"} eq "SAP")
  {
    $teamname = "team_sap basis";
    $reqtype  = "sap";
    $subrequest="problem";
    $action="status";
  }

  ##if ($ENV{"DBMS"} eq "OracleApps")
  ##{
  ##  $teamname = "team_oracle apps erp";
  ##  $reqtype  = "database oracle";
  ##}

  # Override
  ##if ($ENV{"ID2"} eq "ERP" || $ENV{"ID1"} eq "ERP")
  ##{
  ##  $teamname = "team_oracle apps erp";
  ##  $reqtype  = "database oracle";
  ##}

  # Override
  if (
      $ENV{"MSGID"} eq "zDBA665C" ||
      $ENV{"MSGID"} eq "zDBA665C"
     )
  {
    $teamname = "team_database oracle core";
  }

  &msg("teamname=$teamname");

  ##print "$P | Sending EMail\n";
  open(MAIL, "| dbamon_mailer -m -s '$teamname' ARSystem\@$bltserver frontline_bltoutage\@agilent.com") ||
    die "Can't do dbamon_mailer!\n";

  if (length($msg) > 120)
  {
    $msg = substr($msg,0,120);
  }

  # Random number for "data_4"
  $unique = time . "_" . $$ . "_" . $dbamonhost;

  # Set geography from ID1: dbc parameter
  #$z = $ENV{"ID1"};
  #if ($z eq "") { $geo = "COS"; }
  #else          { $geo = $z;    }
  #$msg = "[$geo]$msg";

  # Set gap-x from ID3: dbc parameter
  $z = $ENV{"ID3"};
  if ($z eq "") { $geo = "";    }
  else          { $geo = "[$z]";}
  $msg = "$geo$msg";

  # Calc cksum
  chop($cksum = `echo '$msg' | cksum | awk '{ print \$1}'`);

  if (
      "$ENV{'DBMS'}-$ENV{'SHORT'}" eq "MSSQL-MSSQLAlert" ||
      "$ENV{'DBMS'}-$ENV{'SHORT'}" eq "Oracle-FileOld" 
     )
  {
    &msg("This ticket, with a short message value of $ENV{'DBMS'}-$ENV{'SHORT'} will be");
    &msg("  handled by ITO, so a BLT ticket will not be created");
    $blt_skipped = 1;
    return;
  }

  # Check for pre-existing duplicate NEW ticket
  &msg("Running /usr/local/dba/tools/blt_query_dup DBAmon\@$dbamonhost $ENV{'HOST'} $cksum");
  $rc = system("/usr/local/dba/tools/blt_query_dup DBAmon\@$dbamonhost $ENV{'HOST'} $cksum");
  $rc /= 256;
  &msg("blt_query_dup rc=$rc ");
  # If rc=0, then there is already a dup ticket
  if ($rc == 0)    # If this is a duplicate
  {
    &msg("This BLT is duplicate with an existing status=new ticket with the same information - exiting");
    ##system("echo '' | mailx -m -s 'DBAmon - Dup $dbamonhost - $msg' bill_border\@agilent.com");
    if ($ENV{'RESULTS_FILE'} ne "")
    {
      system("echo 'N/A Dup-2' > $ENV{'RESULTS_FILE'}");
    }
    exit 1;
  }

  # Create mail message
  print MAIL <<END;
#AR-Message-Begin             Do Not Delete This Line
Schema: BLT4.0
Server: $bltserver
Login: $login
Password:
Action: Submit
# Values: Submit, Query
Format: Short
# Values: Short, Full 
!600000100!: $site
!        2!: DBAmon[$ENV{"MSGID"}]
!536870915!: DBAmon[$ENV{"MSGID"}]
!536870916!: $phone
!536870917!: $email
!700000000!: $reqtype
!700000100!: $subrequest
!700000200!: $ENV{"DBMS"}-$ENV{"SHORT"}
!720000001!: $cksum
!720000002!: $ENV{"HOST"}
!720000003!: $ENV{"MSGID"}
!720000004!: $unique
!720000008!: $ENV{"ID3"}
!610000100!: $action
!610000200!: $priority
!610000000!: new
!        4!: $teamname
!        8!: $msg
!536870922!: $msg
!536870987!: DBAmon
!536870921!: 0
!536870960!: DBAmon Critical Event
Event Description:
-------------------------------------
Host:     $ENV{"HOST"}
Instance: $ENV{"INSTANCE"}
Event:    $msg
Text:
$msgtext
-------------------------------------
#AR-Message-End             Do Not Delet This Line
END

  close(MAIL);
  ##print "$P | Created BLT ticket\n"
}

sub error
{
  $msg = $_[0];
  &msg("Fatal Error: $msg");
  exit 1;
}

#-----------------------------------------------------------------------------
sub ito_ticket 
#-----------------------------------------------------------------------------
{
&msg("Creating ITO ticket") ;
&msg("Checking if ITO is installed on this server") ;
if (!(-f "/opt/OV/bin/OpC/opcmsg"))
 {
  &msg("ITO not installed on this server, can't send ITO message") ;
  return ; 
 }
 $priority = 'minor' ; 
 $object =  $ENV{"DBMS"} . '_' . $ENV{"SHORT"} ; 
 #using blt_ticket functions variable $msg , may need change ..
 $msg_text = "Node: " . $ENV{"HOST"} . " Event: " .  $msg ;
 $msg_grp = "Job" ; 
 $appl = "DBA_BLT_Information" ;
 &msg("Setting severity to $priority") ; 
 &msg("Setting application to $appl") ; 
 &msg("Setting msg_grp to $msg_grp") ; 
 &msg("Setting object to $object") ; 
 &msg("Setting msg_text to $msg_text") ; 
 
 &msg("Running /opt/OV/bin/OpC/opcmsg severity=$priority application=$appl msg_grp=$msg_grp object=$object msg_text=$msg_text") ; 
  system("/opt/OV/bin/OpC/opcmsg severity=$priority application=$appl msg_grp=$msg_grp object=$object msg_text=\"$msg_text\"") ; 
  if ($? != 0) 
  {
   &msg("Something went wrong, opc_msg returned : $?") ; 
   system("mailx -s \"ITO msg failed $?\" volker_demand\@agilent.com") ; 
  }
 else 
  {
   &msg("opc_msg completed successfully , returned $? ") ; 
  } 
 return ; 
}

#-----------------------------------------------------------------------------
sub blt_query
#-----------------------------------------------------------------------------
{
  &msg("Querying BLT ticket number - data_4=$unique");

  &msg("temporarily turn off due to hang");
  return;

  for ($i=0; $i<6; $i++)
  {
    sleep 5;
    $rc = system("/usr/local/dba/tools/blt_query data_4 $unique > $tmp1 2>&1");
    $rc /= 256;

    #print "rc=$rc tmp1=$tmp1 \n";
    #system("ll $tmp1 2>&1");
    #system("cat $tmp1 2>&1");

    if ($rc == 0)
    {
      @out = `cat $tmp1`;
      foreach (@out)
      {
	##print;
	@words = split;
	if ($words[0] eq "call_id:") { $blt_ticket = $words[1]; }
	if ($words[0] eq "status:")  { $blt_status = $words[1]; }
      }
    }
    last if $blt_ticket ne "" && $blt_ticket !~ /temp/; 
  }

  &msg("Resulting blt_ticket=$blt_ticket blt_status=$blt_status i=$i");
  if ($ENV{'RESULTS_FILE'} ne "")
  {
    system("echo '$blt_ticket' > $ENV{'RESULTS_FILE'}");
  }
}

# =========================================================
# Main 
# =========================================================

&msg("Starting");

&blt_ticket();
&ito_ticket(); 
&blt_query() if $blt_skipped != 1;

&msg("Done");

system("rm $tmp1 2>/dev/null");
exit 0;
