#!/usr/local/bin/perl
# --------------------------------------------------------------
# Program:   dbamon_onarc.pl
# Date:      1998
# Changes:
#   
#   bb 17Mar99 - Logic to count hours since last backup was
#      flawed in that it did not consider the number of days
#      (integer before timestamp on diff sql output). Changed
#      code to count number of days. This number is multiplied
#      by 24 and added to hours.
# --------------------------------------------------------------

# 
# Variables
# 

$| = 1;
$P = "dbamon_onarc.pl";

#print "$P | Starting\n";

#
# Subroutines
#

sub error
{
  $msg = @_[0];
  print "\n";
  print "$P | Fatal Error: $msg\n";
  $date = `date`;
  print "$P | Exiting\n";
  exit 1;
}

sub report
{
  open(INDB, "echo '\
  set isolation to dirty read;
  select
    \"FLAG\",
    b.req_execution_date,
    a.ae_level,
    b.req_rid,
    c.ds_name
  from
    arc_archive_event a,
    arc_request b,
    arc_dbspace_set c
  where
    a.ae_rid = b.req_rid
    and b.req_status = \"SU\" 
    and b.req_type = \"AR\"
    and a.ae_dsid = c.ds_dsid
  order by
    2 desc ;
  ' | dbaccess -e sysmaster 2>&1 |") || die "Error running dbaccess";

  printf "\n\nReport of all ONArchive Completed Archive Requests\n\n";
  print "Backup Date              Level   Request ID   DBSpace Set\n";
  print "----------------------   -----   ----------   -----------\n";

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = @words[0];
    if ($verb eq "FLAG")
    {
      printf "%-12s %-5s %8s %12s   %-12s\n",@words[1],@words[2],@words[3],@words[4],@words[5];
    }

  }
  close(INDB);
  printf "\nNote: dbspaceset=* backups will not appear on this report\n\n";
}


sub listlast
{
  open(INDB, "echo '\
  select
    *
  from
    arc_request a,
    arc_archive_event b
  where
    a.req_rid = b.ae_rid and
    b.ae_level = 0 and
    req_type = \"AR\"
  into temp t1
  ;
  select
    \"MRL0DSID\",
    ae_dsid
  from
    t1
  where
  req_execution_date =
    (select max(req_execution_date) from t1)
    ;
  select 
    *,
    current - req_execution_date DIFF
    from 
    t1;
  ' | dbaccess -e sysmaster 2>&1 |") || die "Error running dbaccess";

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = @words[0];

    if ($verb eq "MRL0DSID")
    {
      $mrl0dsid = @words[1];
    }
    if ($verb eq "diff")
    {
      $mrl0daysago = @words[1];
      last;
    }
  }
  close(INDB);
}


sub listarc
{
  open(INDB, "echo '\
  set isolation to dirty read;
  select
    a.ae_level,
    a.ae_dsid,
    a.ae_timestamp,
    b.req_execution_date,
    current now,
    current - b.req_execution_date DIFF
  from
    arc_archive_event a,
    arc_request b
  where
    a.ae_rid = b.req_rid
    and b.req_execution_date =
    (select max(req_execution_date) from arc_request
     where req_status = \"SU\" and req_type = \"AR\")
    ;
  ' | dbaccess -e sysmaster 2>&1 |") || die "Error running dbaccess";

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = @words[0];

    if ($verb eq "diff")
    {
      $ts      = @words[2];
      @words2  = split(/:/,$ts);
      $in_days = int(@words[1]);
      $in_hrs  = int(@words2[0]);

      $hour    = $in_hrs + ($in_days * 24);

      ##print "ts=$ts hour=$hour\n";
    }

  }
  close(INDB);
}

sub running
{
  open(INDB, "echo '\
  set isolation to dirty read;
  select
    \"THIS\",
    b.req_execution_date
  from
    arc_request b
  where
    req_status = \"EX\" and req_type = \"AR\"
    ;
  ' | dbaccess -e sysmaster 2>&1 |") || die "Error running dbaccess";

  $running = "N";

  while(<INDB>)
  {
    ##print;
    @words = split;
    $verb    = @words[0];
    if ($verb eq "THIS")
    {
      $running = "Y";
    }
  }
  close(INDB);
}


# --------------------------------------------------------------
# Main

#
# Find DSID of most recent backup
#
&listlast();
print "MRL0DSID: $mrl0dsid\n";
print "MRL0DAYSAGO: $mrl0daysago\n";

#
# Find number of hours since any backup
#
&listarc();
print "HOURS: $hour\n";

&running();
print "RUNNING: $running\n";

&report();

exit 0;
