#!/usr/bin/perl

$| = 1;
$P = "dbamon_tx.pl";

$arg = $ARGV[0];

$debug = 0;

#
# Find ltxhwm
#

$ltxhwm = `onstat -c | grep ^LTXHWM | awk '{ print \$2 }'`;
chop $ltxhwm;
if ($debug) { print "ltxhwm=$ltxhwm\n"; }

#
# Find current log #
#

$log_cur = `onstat -l | grep 'C' | grep -vi INFORMIX | awk '{ print \$4 }'`;
chop $log_cur;
if ($debug) { print "log_cur=$log_cur\n"; }

#
# Find number of logs
#

$log_cnt = `echo "select 'THIS', count(*) from syslogs" | dbaccess sysmaster 2>&1 | grep THIS | awk '{ print \$2 }'`;
chop $log_cnt;
if ($debug) { print "log_cnt=$log_cnt\n"; }

#
# Find tx's with non-zero logbegin  
#

open(IN, "echo '\
set isolation to dirty read;
select 
  tx_id, 
  us_name, 
  us_sid, 
  username,
  pid,
  hostname,
  connected,
  tx_logbeg 
from 
  systrans,
  sysuserthreads,
  syssessions
where 
  tx_logbeg > 0
  and us_address = tx_owner
  and us_sid = sid
;' | dbaccess sysmaster 2>&1 |") || die "Error running dbaccess";

print "Possible Long Transactions\n";
print "Current Log: $log_cur  LTXHWM: $ltxhwm Log Count: $log_cnt\n\n";
print "   TX-Id User Name SID     Hostname           PID   Connected          LogBeg LogDlta  LogPct  HWMPct \n";

while(<IN>)
{
  if ($debug) { print };
  @words = split;
  $verb    = @words[0];

  if ($verb eq "tx_id")     { $tx_id     = @words[1]; }
  if ($verb eq "us_name")   { $us_name   = @words[1]; }
  if ($verb eq "us_sid")    { $us_sid    = @words[1]; }
  if ($verb eq "hostname")  { $hostname  = @words[1]; }
  if ($verb eq "pid")       { $pid       = @words[1]; }
  if ($verb eq "connected") 
  { 
    $connected = @words[1]; 
    ($sec,$min,$hour,$mday,$mon,$year)=localtime($connected);
    ++$mon;
    $cts = "$year/$mon/$mday-$hour:$min:$sec";
  }
  

  if ($verb eq "tx_logbeg") 
  { 
    $tx_logbeg = @words[1]; 

    $log_delta = $log_cur - $tx_logbeg;
    $log_pct   = int($log_delta / $log_cnt * 100);
    $log_hwm   = int($log_pct / $ltxhwm * 100);
    if ($debug) { print "log_delta=$log_delta log_pct=$log_pct\n"; }

    printf("-- %-4s  %-8s  %-6s  %12s  %6s %-17s  %6s  %6s  %6s  %6s\n",$tx_id, $us_name, $us_sid, $hostname, $pid, $cts, $tx_logbeg, $log_delta, $log_pct, $log_hwm);

    #
    # If the option was specified, get the SQL too
    #
    if ($arg eq "-sql")
    {

      open(INSQL, "echo '\
      set isolation to dirty read;
      select
	sqs_dbname,
	sqs_statement
      from
	syssqlstat
      where
	sqs_sessionid + 0 = \"$us_sid\"
      ;' | dbaccess sysmaster 2>&1 |") || die "Error running dbaccess #2";
      while (<INSQL>)
      {
	if ($debug) { print };
	@words = split;
	$verb    = @words[0];

        if ($verb eq "")
        {
          $flag = 0;
        }
        if ($flag) 
        {
          print;
        }
        if ($verb eq "sqs_dbname")
        {
          print "    DBName: @words[1]\n";
        }
        if ($verb eq "sqs_statement")
        {
          $s = substr($_,15);
          print "    Statement: $s";
          $flag = 1;
        }
      } # End of while
    print "\n";
    } # End of if sql
  }
}
exit 0;
