#!/usr/bin/perl
#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************
# =========================================================
# dbamon_ora_sum.pl
#
# Desc:
#   Run every minute (cron) and recreate the instance    
#     summary pages (test and prod).
#
# Changes:
#   bb 01Dec00 - Initial
# =========================================================

$debug = 1;

$| = 1;
$P = "dbamon_www.pl";
$ENV{'PATH'} = "/opt/dbamon/bin:$ENV{'PATH'}";

#------------------------------------------------------------------------------
sub trim
#------------------------------------------------------------------------------
{
  my @out = @_;
  for (@out)
  {
    s/^\s+//;
    s/\s+$//;
  }
  return wantarray ? @out : $out[0];
}

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  $msgmsg = @_[0];
  chop($date = `date +%Y/%m/%d-%H:%M:%S`);
  if ($debug) { print "$date ${P} | $msgmsg\n"; }
}

#------------------------------------------------------------------------------
sub error
#------------------------------------------------------------------------------
{
  $msg = @_[0];
  &msg("=============================================================");
  &msg("Fatal Error: $msg");
  &msg("=============================================================");
  &msg("Exiting rc=1");
  exit 1;
}

#------------------------------------------------------------------------------
sub connectDB
#------------------------------------------------------------------------------
{
  &msg("Connecting to DB");
  use DBI;
  $user = "dbamon";
  $passwd = "dbamon";
  $data_source = "DBI:Oracle";
  $database = "DBA";

  $dbh = DBI->connect("$data_source:$database", $user, $passwd) ||
    &error('Failed_on_DB_connect');
  $dbh->{LongReadLen} = 4000;
}

#-----------------------------------------------------------------------------
sub makeTitle
#-----------------------------------------------------------------------------
{
  &msg("Creating title section");

  print HTML "<title>DBAmon Instance Summary</title>\n" ;
  print HTML "<meta http-equiv=Refresh content=3600>\n" ;
  print HTML "<body bgcolor=white>\n" ;
  print HTML "<table><tr><td><img src='$images_path/dbamonlogo.jpg'>\n" ;
  print HTML "<tr bgcolor=wheat><td><font size=5><center>Instance Summary</center>\n";
  print HTML "</table>\n" ;
  print HTML "<font size=-2><i>This page was generated by DBAmon <b>$verrel</b> running on system: $hostname on $date - This execution of DBAMon was started on $startdate</i></font>\n" ;
  print HTML "<hr><ul><font size=-1>\n";
  print HTML "<li> <a HREF='$http_rpt_path'><b>DBAmon Disk Usage Report</a> </b>\n" ;
  print HTML "<li> <a HREF='$http_path'>    <b>DBAmon Event Log</a>         </b>\n" ;
  print HTML "<li> <a HREF='$http_rpt_path'><b>DBAmon Disk Usage Report</a> </b>\n" ;
  print HTML "<li> <a HREF='http://dbamon.com'><b>DBAmon Home Page</a> </b>\n" ;
  print HTML "</ul></font>\n" ;
  print HTML "<hr>\n" ;
}

#-----------------------------------------------------------------------------
sub makeScoreboard
#-----------------------------------------------------------------------------
{
  &msg("Creating scoreboard section");

  $statement  = "
    select 
      count(*), 
      sum(spacealloc) 
    from 
      dbamon.summary_detail
  ";
  ##print "$statement\n";
  $sth = $dbh->prepare($statement) ||
          &displayError('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &displayError('Failed_on_DB_execute_in_chgEnt');
  $selectcount = 0;
  while (@Data = $sth->fetchrow_array)
  {
    $health_tot = $Data[0];
    $total_space = $Data[1];
  }

  $statement  = "select sum(count) from dbamon.summary_tot where text = 'No Events'";
  ##print "$statement\n";
  $sth = $dbh->prepare($statement) ||
          &displayError('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &displayError('Failed_on_DB_execute_in_chgEnt');
  $selectcount = 0;
  while (@Data = $sth->fetchrow_array)
  {
    $health_good = $Data[0];
  }

  $health = int(($health_good / $health_tot) * 100);

  if    ($health >= 90) { $healthcolor = "lime"; }
  elsif ($health >= 70) { $healthcolor = "yellow"; }
  else                  { $healthcolor = "red"; }

  #print HTML "<h4>Scoreboard - Iteration: #" iter_count " - Duration: " etime " Minute(s)</h4>\n";
  print HTML "<p><font size=3><b>Scoreboard:</font></b>\n";

  print HTML "<table border=1 width=33% bgcolor=lightgrey>\n";

  print HTML "<tr><th><h5>All Instance Health Index (1-100):<td bgcolor=$healthcolor><font size=6><b><center>$health</b></center></font>\n";
  print HTML "</table>\n";

  print HTML "<table border=1 width=33% bgcolor=lightgrey>\n";
  print HTML "<tr bgcolor=slate><th><h5><i>Instance Status<th><h5><i>Instance Count\n";

  $statement  = "select text, color, sum(count) from dbamon.summary_tot where dbamonmode like '${mode}%' group by text, color order by 3 desc";
  ##print "$statement\n";
  $sth = $dbh->prepare($statement) ||
          &displayError('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &displayError('Failed_on_DB_execute_in_chgEnt');
  $selectcount = 0;
  while (@Data = $sth->fetchrow_array)
  {
    $text   = $Data[0];
    $color  = $Data[1];
    $count  = $Data[2];
    if (substr($color,0,5) eq "black")
    {
      print HTML "<tr><td bgcolor=black><h5><font color=white>$text</font><td align=right><h5>$count \n";
    }
    else
    {
      print HTML "<tr><td bgcolor=$color><h5>$text<td align=right><h5>$count \n";
    }
    $selectcount++;
  }
  &msg("Selected $selectcount summary_tot rows");

  print HTML "</table><p><table border=1 width=33% bgcolor=lightgrey>\n";
  print HTML "<tr bgcolor=slate><th><h5><i>DBMS/Application<th><h5><i>Instance Count<th><h5><i>Percent of Total Instances<th><h5><i>Low Version<th><h5><i>High Version\n";

  $statement  = "select dbms, v_low, v_high, sum(count) from dbamon.summary_dbms where dbamonmode like '${mode}%' group by dbms, v_low, v_high order by 4 desc";
  ##print "$statement\n";
  $sth = $dbh->prepare($statement) ||
          &displayError('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &displayError('Failed_on_DB_execute_in_chgEnt');
  $selectcount = 0;
  while (@Data = $sth->fetchrow_array)
  {
    $dbms   = $Data[0];
    $v_low  = $Data[1];
    $v_high = $Data[2];
    $count  = $Data[3];

    $pct = int(($count/$health_tot) * 100);

    if ($dbms eq "Informix        ") { $color = "darkblue" ; }
    if ($dbms eq "OracleDB-UX     ") { $color = "red"      ; }
    if ($dbms eq "OracleApps      ") { $color = "darkred"  ; }
    if ($dbms eq "OracleDB-NT     ") { $color = "ff0090"   ; }
    if ($dbms eq "MSSQL           ") { $color = "green"    ; }

    print HTML "<tr><td><h5><font color=$color>$dbms</font><td align=right><h5>$count<td align=right><h5>${pct}%<td><h5>$v_low<td><h5>$v_high \n";
  }
  print HTML "</table><hr>\n" ;
}

#-----------------------------------------------------------------------------
sub makeDetail
#-----------------------------------------------------------------------------
{
  &msg("Creating detail section");

  print HTML "<p><font size=3><b>Instance Detail ($health_tot Instance(s) - $total_space GB Total Size):</font></b>\n" ;
  print HTML "<table border=1 bgcolor=lightgrey>\n" ;

  print HTML "<tr bgcolor=slate>\n" ;
  print HTML "<th colspan=4><h5><i>Status/<br>Instance\n" ;
  print HTML "<th><h5><i>DBMS/<br>Application\n" ;
  print HTML "<th><h5><i>Actions\n" ;
  print HTML "<th><h5><i>Events\n" ;
  print HTML "<th><h5><i>Status<br>/CONFIG<br>Flags\n" ;
  print HTML "<th><h5><i>DBMS/<br>Appl.<br>Version\n" ;

  print HTML "<th><h5><i>Bkup Method: Hours Since<br>Last Bkup/ Threshold<br>(R)erun Cmd\n" ;
  print HTML "<th><h5><i>CPU<br>Model-CPUs<br>(MB Mem)\n" ;
  print HTML "<th><h5><i>O/S<br>Level<br>(Mode)\n" ;
  print HTML "<th><h5><i>Alloc/<br>Used<br>GB\n" ;
  print HTML "<th><h5><i>Mon.<br>Secs.<br>(% Tot.)\n" ;
  print HTML "<th><h5><i>Misc.\n" ;
  print HTML "<th><h5><i>Task/Entry<br>Age<br>Mins.\n" ;

  print HTML "</tr>\n" ;

  $statement  = "
    select 
      system, 
      title, 
      dbms, 
      ts, 
      sysdate-ts,
      dbamonmode,
      htmlstring 
    from 
      dbamon.summary_detail 
    where 
      dbamonmode like '${mode}%' 
    order by 2
  ";
  ##print "$statement\n";
  $sth = $dbh->prepare($statement) ||
          &displayError('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &displayError('Failed_on_DB_execute_in_chgEnt');
  $selectcount = 0;
  while (@Data = $sth->fetchrow_array)
  {
    $sytem  = $Data[0];
    $title  = $Data[1];
    $dbms   = $Data[2];
    $ts     = $Data[3];
    $age    = int($Data[4] * (60*24));    #Convert to minutes
    $task   = trim($Data[5]);
    $html   = $Data[6];

    print HTML "$html <td nowrap><h6>$task/$age\n";
    $selectcount++;
  }
  &msg("Selected $selectcount summary_detail rows");
  print HTML "</table><hr>\n" ;
}

#-----------------------------------------------------------------------------
sub makeFoot
#-----------------------------------------------------------------------------
{
  &msg("Creating foot section");

  print HTML "<p><font size=3><b>Monitor Task Status:</font></b>\n" ;
  print HTML "<table border=1 bgcolor=lightgrey>\n" ;

  print HTML "<tr bgcolor=slate>\n" ;
  print HTML "<th><h5><i>Monitor Process\n" ;
  print HTML "<th><h5><i>Number of Instances Monitored\n" ;
  print HTML "</tr>\n" ;

  $statement  = "select dbamonmode, count(*) from dbamon.summary_detail where dbamonmode like '${mode}%' group by dbamonmode order by 1";
  ##print "$statement\n";
  $sth = $dbh->prepare($statement) ||
          &displayError('Failed_on_DB_prepare_in_chgEnt');
  $sth->execute || &displayError('Failed_on_DB_execute_in_chgEnt');
  $selectcount = 0;
  while (@Data = $sth->fetchrow_array)
  {
    $mode  = $Data[0];
    $count = $Data[1];

    print HTML "<tr><td><h5>$mode<td align=right><h5>$count \n";
    $selectcount++;
  }
  &msg("Selected $selectcount summary_detail mode rows");
  print HTML "</table><hr>\n" ;
}

#-----------------------------------------------------------------------------
sub init
#-----------------------------------------------------------------------------
{
  chop($ORACLE_SID  = `grep ORACLE_SID:  /opt/dbamon/adm/dbamonrc$modesuf | awk '{ print \$2 }'`);
  chop($ORACLE_HOME = `grep ORACLE_HOME: /opt/dbamon/adm/dbamonrc$modesuf | awk '{ print \$2 }'`);
  $ENV{'ORACLE_SID'}  = $ORACLE_SID;
  $ENV{'ORACLE_HOME'} = $ORACLE_HOME;
  msg("Setting ORACLE env - ORACLE_SID=$ORACLE_SID");

  msg("Seeing if this is already running");
  system("ps -ef | grep $P | grep -v grep");
  chop($num = `ps -ef | grep $P | grep -v grep | wc -l`);
  msg("Number of $P processes: $num");
  if ($num > 2)
  {
    &msg("$num $P processes already running - killing myself");
    exit 1;
  }
}

#------------------------------------------------------------------------------
#
# Main
#
#------------------------------------------------------------------------------

&msg("Starting");

&init;

&msg("Invoking dbamon_ora_sumread.pl prod");
system("dbamon_ora_sumread.pl prod");
&msg("Invoking dbamon_ora_sumread.pl test");
system("dbamon_ora_sumread.pl test");

&msg("Done");
exit 0;
