#!/usr/local/bin/perl
#***************************************************************************
#*          Copyright 2000 DBAmon Software, All Rights Reserved            *
#***************************************************************************
# --------------------------------------------------------------
# Changes:
#   bb 19Oct01 - Added arcDestFix to fix broken destinations.
#   bb 29Oct01 - Changed arcDestFix to sleep for 60 seconds (was 5)
#     before rechecking a fixed arcdest.
#   bb 05Nov01 - Added code to detect 64 bit 8.1.7.
#   bb 13Dec01 - Added inSync code.
#   bb 17May02 - Added code to InSyncPri to send a message as to
#     whether the DB is up on the primary. 
#   bb 03Jun02 - Added code to read backup age from a file (standby)
#   bb 04Sep02 - Added code to process multiple archivelog destinations
# 1.22 bb 19Dec02 - New code to not consider UNDO ts's for space checking. 
# 1.23 bb 17Jan03 - New aliasInfo sub.
# 1.24 bb 27Jan03 - New rmanHung sub.
# 1.25 bb 18Mar03 - Added code to query lib and dict get count.
# 1.26 bb 03Aug03 - Added code to diagnose tnsping problem.
# 1.27 bb 06Oct03 - Changed 'chop trailing period' logic for nslookup linux.
# 1.28 bb 06Oct03 - Changed bklvlts logic to account for OFFLINE backups. 
# 1.29 bb 29Apr04 - New code to not report TEMP Df's on standby DB's as orphan.
# 1.30 bb 18Jun04 - Set SHLIB_PATH if needed
# 1.31 bb 15Jul04 - New patches subroutine 
# 1.32 bb 27Aug04 - Fix to rmanHung - added ruser column
# 1.33 bb 16Nov04 - Change save_config DF list to query v$ views 
# 1.34 bb 18Nov04 - New logic to automatically alter users with bad dflt/temp TS
# 1.35 bb 27Nov04 - Added DBOpenheck logic - for standbys that drop open 
# 1.36 bb 23Dec04 - Changed perf to check for sg_size specified in "M"
# 1.37 bb 07Jan05 - Added security subroutine and first few checks 
# 1.38 bb 30Jan05 - Added checkRollstat for WWW invocation - Good db hang check 
# 1.39 bb 08Feb05 - Added undo and mts checks in perf
# 1.40 bb 07Mar05 - New logic to check for standby inhibit
# 1.41 bb 11Mar05 - Added code to count analyzed SYS objects (sys_analyzed) 
# 1.42 bb 06Apr05 - New oddJob1 logic 
# 1.43 bb 14Apr05 - New serverInfoHPUX module, new memory % used logic 
# 1.44 bb 21Apr05 - Added comm_method logic to make insync work on ssh servers 
# 1.45 bb 03May05 - New debug log in select from dual
# 1.46 bb 04May05 - Security fix - New logic in saving and reading files from /tmp 
# 1.47 bb 16May05 - Security fix - turned on listener_security
# 1.48 bb 20May05 - New REQUIRE file - this will simplify much 
# --------------------------------------------------------------
$version = "1.48";
$| = 1;
#$# = 2;

use Time::Local;
use File::Basename;

##$listener_security = 0;
##$listener_security = 1 if $host eq "acomr20";
$listener_security = 1;

##$debug = 1;

$P       = basename($0); 
$progdir = dirname($0);
##$tmpdir= "$progdir";      #---- Old setting 
$tmpdir  = "$progdir/tmp";  #---- FUTURE - after new dirs are implemented
system("mkdir  $tmpdir 2>&1") if ! -e $tmpdir;
chdir("$progdir") || print "chdir to $progdir failed\n";
chop($pwd = `pwd`);

#---- START Temporary code to cleanup the old software/tmp files
#system("rm -f /tmp/dbamon_* 2>/dev/null");
#---- END   Temporary code to cleanup the old software/tmp files

$ENV{"PATH"} = "$ENV{'PATH'}:/usr/sbin";

chop($thisos = `uname`);
if    ($thisos eq "HP-UX") { $bdfcmd = "bdf";    }
elsif ($thisos eq "SunOS") { $bdfcmd = "df -k"; }
elsif ($thisos eq "Linux") { $bdfcmd = "df -k"; }
else                       { $bdfcmd = "df -k";    }
##print "$P | Setting bdfcmd=$bdfcmd for OS=$thisos version=$version \n";

#---- Set config file path
$cfg = "$progdir/dbamon_ora.cfg";

chop($os_version = `uname -r`);

$verb = $ARGV[0];
$arg1 = $ARGV[1];
$arg2 = $ARGV[2];
$arg3 = $ARGV[3];
$arg4 = $ARGV[4];
$dashes = '-' x 80;

# Get hostname
chop($host = `hostname`);
if (index($host,'.') > -1)
{
  $host = (split(/\./,$host)) [0];
  &msg("This host had a fully qualified hostname - trimmed to first level (host=$host)");
}

#TEST
if ($host eq "zfdprd01")
{
  system("sleep 120");
}


# Get userid
chop($userid = `whoami`);

# Save Company name
if ($thisos eq "Linux")
{
  chop($compwork = `host $host | awk '{ print \$1 }'`);
  chop $compwork if $compwork =~ /\.$/;  # remove trailing period
}
else
{
  chop($compwork = `nslookup $host | grep 'Name:' | awk '{ print \$2 }'`);
}
$fqhost = $compwork;

@words = split(/\./,$compwork);
$nexttolast = $#words - 1;
$company = $words[$nexttolast];

chop($date = `date`);
#chop($datefmt = `date +%Y/%m/%d`);

$os = $ENV{"ORACLE_SID"};
$oh = $ENV{"ORACLE_HOME"};

$tmp1 = "$tmpdir/dbamon_ora_${os}_tmp1_$$.txt";
$tmp2 = "$tmpdir/dbamon_ora_${os}_tmp2.txt";

#---- Require the common parameter file
$reqfile = "$progdir/dbamon_common_req_$os.pl"; 
if (-e "$reqfile") 
{
  &msg("DEBUG - Requiring reqfile=$reqfile"); 
  #---- Only require the file if it passes a perl syntax check
  if (system("perl -wc $reqfile > $tmp2 2>&1") != 0)
  {
    &msg("Require file $reqfile Perl compile error - Not requiring"); 
    system("cat $tmp2"); 
    system("cat $tmp2 | mailx -s 'DBAmon | $host/$os $reqfile Compile Error' border\@dba.it.agilent.com") if $company eq "agilent";
  }
  else
  {
    require "$reqfile"; 
  }
  &msg("DEBUG - rq_listener_check=$rq_listener_check rq_runmon=$rq_runmon rq_backup_age=$rq_backup_age"); 
}

#---- See if SHLIB_PATH is needed - set it if it is
if (`sqlplus -? 2>&1` =~ /dld.sl/) 
{
  &msg("SHLIB_PATH is needed - Setting - Ignore previous Coredump message");
  $ENV{'SHLIB_PATH'} = "$oh/lib";
  system("env | grep 'SHLIB'");
}

# This should only happen for the lightbulb
if ($oh eq "*" || $oh eq "")
{
  &msg("Resolving ORACLE_HOME of */blank");
  chop($ENV{"ORACLE_HOME"} = `grep '^$ENV{"ORACLE_SID"}:' /etc/oratab | awk -F: '{ print \$2 }'`);
  $oh = $ENV{"ORACLE_HOME"};
  &msg("Resolved to: $oh");
  $ENV{"PATH"} = "$oh/bin:$progdir:$ENV{'PATH'}";
}

# Set the temp tnsnames dir name
$tmp_tnsnames_dir = "dbamon_tnsnames_dir_$os";

#---- Try to determine the Oracle version that we are running

#---- Old way
#if (-f "$oh/install/unix.rgs")
#{
#  chop($oraver = `cat \$ORACLE_HOME/install/unix.rgs | grep 'rdbms' | head -1 |awk '{ print \$2 }'`);
#}
#else
#{
#  chop($oraver = `sqlplus -v | grep 'SQL' | awk '{ print \$3 }'`);
#}
#$oraver3 = substr($oraver,0,3);
#if    ($oraver3 eq "10.") { $sqlcmd = "sp"; } # sqlplus
#elsif ($oraver3 lt "9.0") { $sqlcmd = "sv"; } # svrmgrl
#else                      { $sqlcmd = "sp"; } # sqlplus

#---- New way
@temp_banner = `$oh/bin/sqlplus -?`;
foreach $line (@temp_banner)
{
  if ($line =~ /SQL/)
  {
    @words        = split(' ',$line);
    $oraver       = substr(${words[2]},0,5);
    $oraver_short = $words[2];
    #---- Chop the last 4 chars off (8.1.7.4.0 -> 8.1.7)
    $oraver_short =~ s/......$//g;
    $oraver_num   = $oraver_short + 0;
    last;
  }
}
if ($oraver_num >= 8.1) { $sqlcmd = "sp"; }
else                    { $sqlcmd = "sv"; }

&oraSQL("exit");

$WWW = $ENV{"WWW"}; 

#---- Figure out comm_method
##system("env 2>&1");
if ($ENV{"SSH_CLIENT"} ne "")
{
  $comm_method = "ssh";
}
else
{
  $comm_method = "remsh";
}

&msg("Starting - version=$version host=$host verb=$verb arg1=$arg1 arg2=$arg2 arg3=$arg3 arg4=$arg4 date=$date userid=$userid");
&msg("  ORACLE_HOME=$oh ORACLE_SID=$os fqhost=$fqhost company=$company");
&msg("  thisos=$thisos os_version=$os_version tmp_tnsnames_dir=$tmp_tnsnames_dir oraver=$oraver sqlcmd=$sqlcmd WWW=$WWW");
&msg("  progdir=$progdir tmpdir=$tmpdir pwd=$pwd os_scmd=$os_scmd os_conn=$os_conn comm_method=$comm_method");

#if ($debug)
#{
#  print "$P | Env:\n";
#  system("env | sort");
#}

$lockfile = "/tmp/DBAmon_Lock_$os.txt";
$lockfile_hours = 2;
&msg("Checking for Lockfile $lockfile"); 
system("ls -l $lockfile 2>&1");

# 30Jan02 - bb - New "Lockfile" - this allows an application to block monitoring
#  for the time during which file /tmp/DBAmon_Lock_<SID>.txt exists. If we find
#  the file we will sleep until it is gone.
#while(-e "$lockfile")
#{
#  &msg("Lockfile $lockfile exists - sleeping 60 before rechecking"); 
#  system("sleep 60");
#}

# 07Sep02 - bb - Lockfile will now act as a trigger to auto-inhibit. If
#  the lockfile is found, its create date will be examined. i will add
#  120 minutes to it and that will be the time during which auto-inhibit
#  is valid. I will pass the string AUTO-INHIBIT back to the caller.
if (-e "$lockfile")
{
  $lockfile_mtime = (stat "$lockfile") [9];
  $lockfile_age_min = int((time - $lockfile_mtime) / 60);
  &msg("Lockfile $lockfile does exist - it was created $lockfile_age_min minutes ago"); 
  $exp_mtime = $lockfile_mtime + ($lockfile_hours*60*60);  # Add $lockfile_hours hours
  if (time < $exp_mtime)
  {    
    &msg("Since it is < $lockfile_hours hours since Lockfile was created, sending AUTO-INHIBIT message back to caller - exiting"); 
    exit;
  }
  else
  {    
    &msg("Since it is >= $lockfile_hours hours since Lockfile was created, it is not valid - rm'ing $lockfile"); 
    system("rm $lockfile");
    print "$P | diag. email\n";
    system("echo ' ' | mailx -s 'DBAmon | Lockfile $lockfile deleted - lockfile_hours=$lockfile_hours border\@dba.it.agilent.com") if $company eq "agilent";
  }
}

#------------------------------------------------------------------------------
#
# Subroutines
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
sub msg
#------------------------------------------------------------------------------
{
  my $msgmsg = $_[0];
  chop($date = `date +%Y/%m/%d-%H:%M:%S`);
  print "$date ${P} | $msgmsg\n";
}

#------------------------------------------------------------------------------
sub oraSQL
#------------------------------------------------------------------------------
{
  my $sql = $_[0];

  if ($sqlcmd eq "sp")
  {
    $os_scmd = "sqlplus /NOLOG";
    $os_conn = "connect / as sysdba ";
    $os_set  = "set linesize 9999;";
    $os_end  = "/";
  }
  else
  {
    $os_scmd = "svrmgrl";
    $os_conn = "connect internal ";
    $os_set  = "";
    $os_end  = "";
  }

  ##&msg("os_scmd=$os_scmd os_conn=$os_conn");

  @oraSQLout = `$os_scmd <<! | tee -a $tmp2
    $os_conn;
    $os_set
    $sql
    ;
  !`;
}

# ----------------------------------------------------------------------
sub error
# ----------------------------------------------------------------------
{
  my $emsg = $_[0];

  if ($ENV{'WWW'})
  {
    print "\n<table bgcolor=red><tr><td><font size=2>$P | Fatal Error: <b>$emsg</b></table>\n\n";
  }
  else
  {
    &msg("================================================================");
    &msg("Fatal Error: $emsg");
    &msg("================================================================");
  }

  &msg("Exiting");
  &msg("END_OF_PROBE");
  exit 1;
}

# ----------------------------------------------------------------------
sub lan
# ----------------------------------------------------------------------
{
  if ($thisos eq "HP-UX" && $os_version lt "B.11.00")
  {
    &msg("LAN Half-Duplex checking only works on HP-UX >= 11.00");
    return;
  }

  &msg("Checking lan cards for half-duplex");

  @lans = `netstat -i | grep -v 'none' | grep '^lan' | grep -v ':' | sort | awk '{ print \$1 }'`;
  foreach $lancard (@lans)
  {
    chop $lancard;
    &msg("Checking lan interface: $lancard");
    chop($crd = `lanscan | grep '$lancard ' | awk '{ print \$3 }'`);
    chop($sta = `lanadmin -x $crd 2>&1`);
 
    $sta =~ s/  //g;

    &msg("  crd=$crd sta=$sta");
    if (
        $sta =~ /Half-Duplex/ &&
        $sta !~ /10 Half-Duplex/ 
       )
    {
      print "$P HALF_DUPLEX_LAN: $lancard $sta\n";
    }
    else
    {
      print "$P FULL_DUPLEX_LAN: $lancard $sta\n";
    }
  } 
 
}

# ----------------------------------------------------------------------
sub ts_full
# ----------------------------------------------------------------------
{
  $tmplog = "$tmpdir/dbamon_ora_ts_full_${$}.log";
  $t_warn = $arg1;
  $t_crit = $arg2;
  $t_free = $arg3;
  $t_cmd  = $arg4;

  print "\n$P | ts_full() - t_warn=$t_warn t_crit=$t_crit t_free=$t_free t_cmd=$t_cmd \n";

  $sqlstmt = "select \'DBAMON_TS_SIZE\',
          a.tablespace_name as tablespace_name,
          a.sum_bytes as bytes_avail,
          a.sum_bytes - nvl(c.sum_bytes,0) as bytes_used,
          nvl(c.sum_bytes,0) as bytes_free,
          round((a.sum_bytes-nvl(c.sum_bytes,0.00)) / a.sum_bytes * 100.0, 2) as pct_used,
          b.contents
     from 
          ( select 
              tablespace_name,
              sum(bytes) sum_bytes,
              count(1) max_id
            from 
              dba_data_files
            group by 
              tablespace_name
          ) a,
          ( select 
              tablespace_name,
              contents
            from 
              dba_tablespaces
          ) b,
          ( select 
              tablespace_name,
	      sum(bytes) sum_bytes
	    from 
              dba_free_space
	    group by 
              tablespace_name
          ) c
     where 
         a.tablespace_name = c.tablespace_name (+) and
         a.tablespace_name = b.tablespace_name 
       $ts_where ;
    select
      \'T\',
      tablespace_name,
      sum(bytes)
    from
      dba_temp_files
    group by
      tablespace_name ;
     ";

  $sqlgood = 0;

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    $diagout .= $_;
    @words = split;
    $verb    = $words[0];
    if ($verb =~ /ORA-/) 
    { 
      print "$P | ERROR: ORA- Oracle error message encountered while checking tablespaces \n";
    }

    if ($verb eq "T") 
    { 
      $ts   = $words[1];
      $size = $words[2]/1000000;
      $used = 0; 
      $free = $size;
      $pct  = 0;
      $type = "TEMPFILE";
      $flag = "OK";  # Default 1.22

      printf("DBAMON_TS_SIZE %-16s %8d %8d %8d %7.1f %s %s\n",$ts,$size,$used,$free,$pct,$flag,$type);
    }

    if ($verb eq "DBAMON_TS_SIZE") 
    { 
      $ts   = $words[1];
      $size = $words[2]/1000000;
      $used = $words[3]/1000000;
      $free = $words[4]/1000000;
      $pct  = $words[5];
      $type = $words[6];
 
      $flag = "OK";  # Default 1.22

      # MB Needed to get to 5% below warning threshold

      $t_warn = 6 if $t_warn <= 5;
      $needed = int($used / (($t_warn-5)/100) - $size);
      if ($needed < 100)
      {
        $needed = 50;
      }
      if ($needed > 2000)
      {
        $needed = 2000;
      }

      if ($pct  >= $t_warn && $pct < $t_crit) { $flag = "W" . " " . $needed; }
      if ($pct  >= $t_crit )                  { $flag = "C" . " " . $needed; }
      if ($free >= $t_free)                   { $flag = "OK";  }

      if ($type eq "UNDO")
      {
        $flag = "OK";
        ##print "undo ts\n";
      }

      if ($flag ne "OK" && $ts =~ /RBS/ && $already_rbs != 1 && $t_crit != 999)
      {
        printf("%s %-16s %8d %8d %8d %7.1f %s\n",$verb,$ts,$size,$used,$free,$pct);
        print "\n--------------------------------------------------------------------\n";
        print "RBSSHRINKING: $ts \n";
        push(@recheck_list,$ts);

        &rbs_shrink();

	print "$P | ts_full() will be re-invoked to recheck ts=$ts\n";
        $already_rbs = 1;
        $ts_recheck = 1;
        print "--------------------------------------------------------------------\n\n";
        next;
      }

      if ($flag ne "OK" && $ts =~ /TEMP/ && $already_temp != 1 && $t_crit != 999)
      {
        printf("%s %-16s %8d %8d %8d %7.1f %s\n",$verb,$ts,$size,$used,$free,$pct);
        print "\n--------------------------------------------------------------------\n";
        print "COALESCING: $ts \n";
        push(@recheck_list,$ts);

        &ts_coalesce("$ts");

	print "$P | ts_full() will be re-invoked to recheck ts=$ts\n";
        $already_temp = 1;
        $ts_recheck = 1;
        print "--------------------------------------------------------------------\n\n";
        next;
      }

      if ($ts =~ /TEMP/ && $flag ne "OK")
      {
	print "  (Since the following tablespace is a TEMP tablespace, it\n";
	print "   will not be eligible for automatic datafile addition)\n";
      }
      elsif ($flag ne "OK" && $t_cmd ne "" && $t_crit != 999)
      {
        # Try to add space
        $cmd = $t_cmd;
	print "$P ======================================================\n";
	print "$P Command before {TS} $ts {SIZE} $needed substitution: $cmd\n";
	$cmd =~ s/\{TS\}/${ts}/g;
	$cmd =~ s/\{SIZE\}/${needed}/g;
	print "$P About to issue: $cmd\n";
        $rc = system("$cmd 2>&1 > $tmplog");
        $rc = $rc / 256;
        print "DBAMON_TS_ADD $rc $ts $pct $needed\n"; 
        system("cat $tmplog >> $tmpdir/DBAmon_TS_Add.log");
        system("cat $tmplog");
        system("rm $tmplog");
	print "$P ======================================================\n";
        if ($rc == 0)
        {
          $flag = "OK";
        }
      }
      printf("%s %-16s %8d %8d %8d %7.1f %s %s\n",$verb,$ts,$size,$used,$free,$pct,$flag,$type);
    }
  }
  close(INDB);

  if ($ts eq "")
  {
    print "$P | ERROR: No tablespaces found (something is wrong) - Diagnostic output:\n";
    print $diagout;
  }
  print "$P | Done \n\n";
}

# ----------------------------------------------------------------------
sub ts_coalesce
# ----------------------------------------------------------------------
{
  my $ts = $_[0];
  $sqlstmt = 'select \'X\', pct_increase from dba_tablespaces;';
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words       = split();
    $verb        = $words[0];
    if ($verb eq "X")
    {
      $pctincrease  = $words[1];
      print "$P | PCTINCREASE For ts=$ts is $pctincrease\n";
      last;
    }
  }
  close(RO);

  if ($pctincrease == 0) 
  {
    print "$P | Since PCTINCREASE is 0, we will temporarily alter it to 1 for the coalesce \n";
    $stmt1 = "alter tablespace $ts default storage ( pctincrease 1 );"; 
    $stmt2 = "alter tablespace $ts default storage ( pctincrease 0 );"; 
  }
  print "$P | Running alter table space coalesce 3 times for ts=$ts \n";
  system("$os_scmd <<! 
    $os_conn
    $stmt1
    alter tablespace $ts coalesce;
    alter tablespace $ts coalesce;
    alter tablespace $ts coalesce;
    $stmt2
  !");
  print "$P | Done \n";
}

# ----------------------------------------------------------------------
sub rbs_shrink
# ----------------------------------------------------------------------
{
  $sqlstmt = 'select \'X\', 
      SEGMENT_NAME,
      OWNER,
      TABLESPACE_NAME,
      SEGMENT_ID,
      INITIAL_EXTENT,
      NEXT_EXTENT,
      MIN_EXTENTS,
      MAX_EXTENTS,
      PCT_INCREASE,
      STATUS
    from dba_rollback_segs;';

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words   = split();
    $verb    = $words[0];
    $name    = $words[1];
    $owner   = $words[2];
    $tsrbs   = $words[3];
    #$id      = $words[4];
    #$xinit   = $words[5];
    #$xnext   = $words[6];
    #$xmin    = $words[7];
    #$xmax    = $words[8];
    #$xpct    = $words[9];
    $status  = $words[10];
    if ($verb eq "X" && $tsrbs eq $ts)
    {
      print "$P | Found rollback segment: $name in ts: $tsrbs - shrinking\n";
      system("$os_scmd <<! 
	$os_conn
	alter rollback segment $name shrink;
      !");
    }
  }
}

# ----------------------------------------------------------------------
sub lsnrSecurity
# ----------------------------------------------------------------------
{
  &msg("lsnrSecurity Starting ===========================");

  if ($thisos eq "Linux")
  {
    &msg("This routing DOES NOT WORK on Linux");
    return;
  }

  if (`lsnrctl status 2>&1` =~ /dld.sl/) 
  {
    &msg("SHLIB_PATH is needed - Setting");
    $ENV{'SHLIB_PATH'} = "$oh/lib";
    system("env | grep 'SHLIB'");
  }

  if (! -r "$lsnr_file")
  {
    &msg("THIS WILL NOT WORK - I do not have READ access to listener.ora file $lsnr_file");
    return;
  }

  &msg("Looking for password in $lsnr_file");
  chop($pwlines = `grep -v '^#' $lsnr_file | grep -i 'passwords_listener' | wc -l`); 
  &msg("pwlines=$pwlines");
  if ($pwlines < 1)
  {
    &msg("Listener password is not set - Setting");

    # Isolate new pw 
    chop($newlispw = `cat $progdir/dbamon_ora.key2 | crypt dbamon`);
    if ($newlispw eq "")
    {
      system("echo ' ' | mailx -s 'DBAmon | $host/$os - key2 pw is NULL thisos=$thisos' border\@dba.it.agilent.com") if $company eq "agilent";
      return;
    }

    $! = ""; 

    open(LSNR, ">>$lsnr_file");  
    if ($! ne "")
    {
      &msg("Cannot open append $lsnr_file - $!");
      system("ll $lsnr_file | mailx -s 'DBAmon | $host/$os/$userid - Cannot open output $lsnr_file - $!' border\@dba.it.agilent.com") if $company eq "agilent";
      return;
    }

    print LSNR "
# listener_password Added by DBAmon $date - Start
PASSWORDS_LISTENER = $newlispw
ADMIN_RESTRICTIONS_LISTENER = ON
# listener_password Added by DBAmon $date - End
";
    close(LSNR);
    &msg("Listener password added to $lsnr_file ");
    &msg("Issuing: lsrnctl reload");
    system("lsnrctl reload 2>&1");
    &msg("Issuing: lsrnctl status");
    system("lsnrctl status 2>&1");
    if (system("grep -i 'passwords_listener' $lsnr_file") != 0)
    {
      system("echo ' ' | mailx -s 'DBAmon | $host/$os/$userid - listener password add did NOT work' border\@dba.it.agilent.com") if $company eq "agilent";
    }
    else
    {
      system("cat $lsnr_file | mailx -s 'DBAmon | $host/$os/$userid - listener password add DID work' border\@dba.it.agilent.com") if $company eq "agilent";
      print "$S$P LSNRPWSET: Set password\n"; 
    }
  }
  else
  {
    &msg("The Listener password is already enabled");
  }

  &msg("lsnrSecurity Done ===========================");
}

# ----------------------------------------------------------------------
sub lsnr
# ----------------------------------------------------------------------
{
  if    (-e "/etc/listener.ora")                { $lsnr_file = "/etc/listener.ora"; }
  elsif (-e "${oh}/network/admin/listener.ora") { $lsnr_file = "${oh}/network/admin/listener.ora"; }
  else
  {
    &msg("Cannot find listener.ora file in the usual locations");
  }

  $S = "Stack: oraLsnrStack ";

  if ($listener_security == 1 && $runmon ne "B")
  {
    &lsnrSecurity; 
  }

  if (`lsnrctl status 2>&1` =~ /dld.sl/)
  {
    &msg("SHLIB_PATH is needed - Setting");
    $ENV{'SHLIB_PATH'} = "$oh/lib";
    system("env | grep 'SHLIB'");
  }

  # Check for lsnr process
  if (&lsnr_check() == 0)
  {
    print "$S$P LSNROK: Listener is active and pingable\n"; 
    return 0;
  }

  # Restart it
  if (&lsnr_start() != 0)
  {
    print "$S$P lsnr: Error trying to restart listener:\n"; 
    return 0;
  }

  # Check it again
  if (&lsnr_check() == 0)
  {
    print "$S$P LSNRRESTARTED: Restarted listener:\n"; 
  }
  else
  {
    print "$S$P LSNRNORESTART: Error trying to restart listener:\n"; 
  }
}

# ----------------------------------------------------------------------
sub datafiles
# ----------------------------------------------------------------------
{
  &msg("Checking datafiles");

  $S = "Stack: oraDatafilesStack ";

  $sqlstmt = 'select
      \'Y\', controlfile_type from v\$database;
    select
      \'Z\',
      d.file#,
      d.status,
      to_char(d.last_time,\'YYYY/MM/DD-HH24:MM:SS\') as timestamp,
      t.name,
      d.name
    from
      v\$datafile   d,
      v\$tablespace t
    where
      d.ts# = t.ts#
      and d.status != \'ONLINE\'
      and d.status != \'SYSTEM\';';
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "Y" && $words[1] eq "STANDBY") 
    { 
      $standby = 1;
    }
    if ($verb eq "Z") 
    { 
      #if (!$flag)
      #{
      #  print "$S\n";
      #  print "$S                            DFNum  Status      LastUpdated          TSName            DFName\n";
      #  print "$S                            =====  ==========  ===================  ================  =======================\n";
      #  $flag = 1;
      #}

      $dfnum     = $words[1];
      $status    = $words[2];
      $timestamp = $words[3];
      $tsname    = $words[4];
      $dfname    = $words[5];

      # There is an oracle bug where it is normal for ts's to be
      # in recover mode on standby db's - 8.1.6
      if ($standby == 1 && $status eq "RECOVER")
      {
        printf "$S$P Standby Datafile in RECOVER: %5d  %-10s  %-19s  %-16s  %-50s\n",$dfnum,$status,$timestamp,$tsname,$dfname; 
      }
      else
      {
	printf "$S$P DatafileDown: %5d  %-10s  %-19s  %-16s  %-50s\n",$dfnum,$status,$timestamp,$tsname,$dfname; 
        $dfdownnum++;
      }
    }
  }

  if ($flag)
  {
    print "$S$P | $dfdownnum Offline Datafile(s)\n"; 
  }
  else
  {
    print "$S$P | datafiles - OK\n";
  }
}

# ----------------------------------------------------------------------
sub extents
# ----------------------------------------------------------------------
{
  #
  # Look for tables that are almost out of extents
  #
  &msg("Looking for tables at or near max_extents");
  $sqlstmt = "select 
    \'X\', 
    owner, 
    segment_name, 
    segment_type, 
    extents, 
    max_extents,
    round(nvl(extents,0.00) / max_extents * 100.0, 2) as x_pct
  from
    dba_segments 
  where
    segment_type in (\'TABLE\',\'INDEX\')
    and owner not in (\'SYS'\,\'SYSTEM\')
    --and (round(nvl(extents,0.00) / max_extents * 100.0, 2)) > $arg1 
  ;";

  ##print "sqlstmt=$sqlstmt\n";

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X") 
    { 
      $owner = $words[1];
      $sname = $words[2];
      $stype = $words[3];
      if ($words[4] =~ /PARTITION/) { shift @words; }
      $exts  = $words[4];
      $max   = $words[5];
      $pct   = $words[6] * 1;
      $rawcount++;

      if ($stype ne "TABLE" && $stype ne "INDEX")
      {
        print;
        &msg("Invalid stype=$stype - $owner=$owner sname=$sname");
        next;
      }
 
      $ext_hwm = $pct if $pct > $ext_hwm;

      if ($pct > $arg1)
      {
        $foundtbls++;
        $tbl = $owner . "." . $sname;
        if (!$titleflag)
        {
          print "\n"; 
          print "        Object(s) With Number of Extents >= Threshold\n"; 
          print "               Of $arg1% Of Max_Extents\n\n"; 
          print "Segment                                  Segment Extent    Max  Pct Of\n"; 
          print "Name                                     Type     Count Extent     Max\n"; 
          print "======================================== ======= ====== ====== =======\n"; 
          $titleflag = 1;
        }
        printf "%-40s %-7s %6i %6i %7.2f\n",$tbl,$stype,$exts,$max,$pct; 
        if (uc $arg2 eq "FIX")
        {
          #if the tbl name  contains a $, we have to fix that
          $tbl =~ s/\$/\\\$/g;

          print "$P | extents() - Attempting to ALTER this object to set MAXEXTENTS to UNLIMITED\n";
          print "$P | extents() - Issuing:  alter $stype $tbl storage ( maxextents unlimited );\n";
	  system("$os_scmd <<! 
          $os_conn
	  alter $stype $tbl storage ( maxextents unlimited );
!");
          $fixedtbls = " FIXED";
        }
      }
    }
  }
  print "\n";
  if ($foundtbls > 0)
  {
    print "$P FOUND $foundtbls $fixedtbls\n";
    &msg("Extents/Max - FOUND $fixedtbls - Found $foundtbls table(s) with > $arg1 percent of extents used - rawcount=$rawcount ext_hwm=$ext_hwm");
  }
  else
  {
    &msg("Extents/Max - OK - No tables with > $arg1 percent of extents used - rawcount=$rawcount ext_hwm=$ext_hwm");
  }

  #
  # Look for tables where the next extent will not fit in the largest
  #   freespace hole for their tablespace
  #
  &msg("Looking for tables whose next extent will not fit");
  $sqlstmt = '
    select
      \'X\',
      segment_owner,
      segment_name,
      segment_type,
      ts1,
      max_chunk,
      chunk_needed
    from
      (select
	 tablespace_name ts1,
	 max(bytes) as max_chunk
       from dba_free_space
       group by tablespace_name),
      (select
	 owner segment_owner,
	 segment_name,
	 segment_type,
	 tablespace_name ts2,
	 next_extent chunk_needed
	 from dba_segments)
    where
      ts1=ts2
      and chunk_needed> max_chunk
  ';
  &oraSQL($sqlstmt);

  $foundtbls = 0;
  $titleflag = 0;

  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X") 
    { 
      $owner           = $words[1];
      $sname           = $words[2];
      $stype           = $words[3];
      $ts              = $words[4];
      $ts_max_free     = $words[5];
      $s_next_size     = $words[6];

      $foundtbls++;
      $tbl = $owner . "." . $sname;

      if (!$titleflag)
      {
	print "\n"; 
	print "                          Object(s) Whose Next_Extent Will Not Fit\n"; 
	print "\n"; 
	print "Segment                                  Segment Tablespace               Object Next  TS Max Contig.\n"; 
	print "Name                                     Type    Name                     Extent Size      Free Space\n"; 
	print "---------------------------------------- ------- -------------------- --------------- ---------------\n"; 
	$titleflag = 1;
      }
      printf "%-40s %-7s %-20s %15i %15i\n",$tbl,$stype,$ts,$s_next_size,$ts_max_free; 
    }
  }
  print "\n";
  if ($foundtbls > 0)
  {
    print "$P NEXTNOFIT $foundtbls\n";
    &msg("Extents/Nextfit - Found $foundtbls whose next extent will not fit");
  }
  else
  {
    &msg("Extents/NextFit OK - No tables whose next extent will not fit");
  }
}

# ----------------------------------------------------------------------
sub arclogs 
# ----------------------------------------------------------------------
{
  $t_warn = $arg1;
  $t_crit = $arg2;
  $t_cmd  = $arg3;
  print "$P | t_warn=$t_warn t_crit=$t_crit t_cmd=$t_cmd \n";

  $sql = '
    archive log list;
    select \'X\', destination from v\$archive_dest 
      where arcmode = \'MUST SUCCEED\' and destination like \'/%\';
    select \'Y\', destination from v\$archive_dest 
      where binding = \'MANDATORY\' and destination like \'/%\';
  ';
  
  @oraSQLout = `$os_scmd <<!
    $os_conn;
    $os_set
    $sql
    exit
  !`;

  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    #if (/> Database log mode/) 
    #{ 
    #  $arclogmode = $words[5];
    #}
    if (/Database log mode/ && /    Archive Mode/) 
    { 
      $arclogmode = "Archive";
    }
    if (substr($_,0,18) eq "Automatic archival") 
    { 
      $arcauto    = $words[2];
    }
    if (substr($_,0,19) eq "Archive destination" && $words[2] =~ /^\//) 
    { 
      $arcdirraw  = $words[2];
      push(@arcdirs,$arcdirraw);
    }
    if ($verb eq "X")
    {
      #8.0
      $arcdirraw  = $words[1];
      push(@arcdirs,$arcdirraw) unless $arcdirs[0] eq $arcdirraw;
    }
    if ($verb eq "Y")
    {
      #8.1
      $arcdirraw  = $words[1];
      push(@arcdirs,$arcdirraw) unless $arcdirs[0] eq $arcdirraw;
    }
  }

  # Process each @arcdirs
  $z = $#arcdirs + 1;
  &msg("$z Archivelog destinations will be considered");
  @arcdirssorted = sort @arcdirs;
  foreach $arcdirraw (@arcdirssorted)
  {
    &msg("Processing arcdirraw=$arcdirraw");
    $flag = "";

    # if the 1st char of arcdirraw is ?, then xlate it to oracle_home
    #if (substr($arcdirraw,0,1) eq "?")
    if ($arcdirraw =~ /\?/)
    {
      $arcdirraw =~ s/\?/$oh/;
      print "$P | arcdirraw Promoted from ?... to $arcdirraw \n";
    }

    # Cut off last level of (example) /oracle/PD1/saparch/PD1arch,
    # only if the last char is NOT a SLASH
    $lastchar = substr($arcdirraw,length($arcdirraw)-1);
    $z = -e $arcdirraw;
    ##print "lastchar=$lastchar z=$z\n";
    if ($lastchar ne "/" && $z == 0)
    {
      @words = split(/\//,$arcdirraw);
      $numlevels = $#words - 1;
      $arcdir = "";
      for ($i = 1; $i <= $numlevels; $i++)
      {
	$arcdir = $arcdir . "/" . $words[$i];
      }
    }
    else
    {
      $arcdir = $arcdirraw;
    }

    if ($arcdir ne "") 
    {
      open(BDF, "$bdfcmd $arcdir 2>/dev/null | tail -1 |") || &error("Error running $bdfcmd");
      while(<BDF>)
      {
	@words = split;
	$arcdirfull = $words[$#words-1];
	$arcfs      = $words[$#words];
	# Remove the % sign
	chop $arcdirfull;
      }
      close(BDF);
      if ($arcdirfull  >= $t_warn && $arcdirfull < $t_crit) { $flag = "W"; }
      if ($arcdirfull  >= $t_crit )                         { $flag = "C"; }
    }
    else
    {
      $flag = "E";
    }

    # Count the number of *arch* files in the archivelog directory
    &msg("Reading directory arcdir=$arcdir - looking for arch* files");
    opendir(ARCDIR,"$arcdir");
    @filenames = sort readdir(ARCDIR);
    closedir(ARCDIR);

    $arclogcount = 0;
    ARCFOREACH: foreach $file (@filenames)
    {
      $filefull = $arcdir . "/" . $file;
      next if -d "$filefull";
      next if $file !~ /^arch/;
      $arclogcount++;
    }
    &msg("Archivelog dir=$arcdir contains $arclogcount ^arch* files");

    # Run the specfied archivelog backup command
    $arccmdrun = "N";
    #if (($flag eq "W" || $flag eq "C") && $t_cmd ne "")
    if (($flag eq "W" || $flag eq "C") 
         && $t_cmd ne "" 
         && $arclogcount > 0)
    {
      $cmd = $t_cmd;
      print "$P Issuing at now command to run arclog backup:\n";
      print "$P Command before {SID} substitution: $cmd\n";
      $cmd =~ s/\{SID\}/$os/g;
      $cmd = "echo 'time $cmd' | at now";
      print "$P About to issue: $cmd\n";
      $rc = system($cmd);
      print "$P Back from issuing at now - rc=$rc\n";

      $arccmdrun = "Y";
    }
    if (($flag eq "W" || $flag eq "C") 
         && $arclogcount == 0)
    {
      &msg("<b>================================================================</b>");
      &msg("<b>NOTE: The archivelog filesystem $arcfs</b>");
      &msg("<b>  is over its threshold full, but because there are 0 </b>");
      &msg("<b>  archivelogs to backup for this filesystem, it would not </b>");
      &msg("<b>  do any good to run an archivelog backup. I will assume that </b>");
      &msg("<b>  the filesystem is full because of another DB which is also </b>");
      &msg("<b>  using this filesystem. The sev of this event will be $flag. </b>");
      &msg("<b>================================================================</b>");
    }

    print "$P arcdirraw:  $arcdirraw\n";
    print "$P arclogmode: $arclogmode\n";
    print "$P arcauto:    $arcauto\n";
    print "$P arcdir:     $arcdir\n";
    print "$P arcfs:      $arcfs\n";
    print "$P arcdirfull: $arcdirfull\n";
    print "$P arcflag:    $flag\n";
    print "$P arccmdrun:  $arccmdrun\n";
  } # end of foreach arcdirs

  # Check archivelog destination status
  ARCLOG_FINISH: $sqlstmt = 'select \'X\',
      DEST_ID      ,
      STATUS       ,
      BINDING      ,
      NAME_SPACE   ,
      TARGET       ,
      REOPEN_SECS  ,
      DESTINATION  ,
      to_char(FAIL_DATE,\'YYYY/MM/DD-HH24:MM:SS\') ,
      ERROR
    from v\$archive_dest ;';
  
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X") 
    { 
      $dest    = $words[1];
      $status  = $words[2];
      $binding = $words[3];
      $name    = $words[4];
      $target  = $words[5];
      $reopen  = $words[6];
      $dest2   = $words[7];
      $faildt  = $words[8];
      $error   = $words[9] . " " . $words[10]  . " " . $words[11]  . " " . $words[12]  . " " . $words[13]  . " " . $words[14]  . " " . $words[15]  . " " . $words[16];

      if ($faildt ne "") { $faildt = "Failure Date: $faildt"; }
       
      # Try to fix dest
      if ($status eq "ERROR") 
      { 
        &arcDestFix;
      }

      # See if it is still broken
      if ($status eq "ERROR") 
      { 
	$desterror = 1; 
	$color = "red";
	$desterrorlist .= $dest . ",";
      }
      else
      {
        $color = "lime";
      }
      $arcdesthtml .= "<tr bgcolor=$color><td><h5>$dest<td><h5>$status<td><h5>$binding<td><h5>$name<td><h5>$target<td><h5>$reopen<td><h5>$dest2<td><h5>$faildt $error </tr>\n";
    }
  }
  print "<p><hr><b>Archive Log Destination Status:</b>\n";
  print "<table border=1>\n";
  print "<tr bgcolor=slate><th>Dest<br>Number<th>Status<th>Binding<th>System/<br>Session<th>Primary/<br>Standby<th>Reopen<br>Secs.<th>Destination<th>Error(s)\n";

  print $arcdesthtml;

  print "</table><hr>\n";

  if ($desterror)
  {
    chop $desterrorlist;
    print "$P arcdesterror: Y $desterrorlist $destfixmsg\n";
  }
  if ($destwarning)
  {
    chop $destwarninglist;
    print "$P arcdestwarning: Y $destwarninglist $destfixmsg\n";
  }

}

# ----------------------------------------------------------------------
sub arcDestFix
# ----------------------------------------------------------------------
{
  print "$P | Attempting to fix archive dest $dest\n";

  print "$P | Looking for init.ora log_archive_dest_$dest parm\n";
  open(RO,"$oh/dbs/init$os.ora") || &error("Cannot OPEN init.ora file=$oh/dbs/init$os.ora");
  while (<RO>)
  {
    $initora .= $_;
    @words = split(/=/);
    if (substr($words[0],0,5) eq "ifile")
    {
      $ifile = $words[$#words];
      $initora .= "#**** Contents of ifile: $ifile *****\n";
      # if the 1st char of ifile is ?, then xlate it to oracle_home
      if ($ifile =~ /\?/)
      {
	$ifile =~ s/\?/$oh/;
	&msg("ifile Promoted from ?... to $ifile");
      }
      open(RO2,"$ifile") || &error("Cannot OPEN init.ora file=$ifile");
      while (<RO2>)
      {
        $initora .= $_;
      }
      close(RO2);
      $initora .= "#**** End of ifile: $ifile *****\n";
    }
  }
  close(RO);

  @lines = split(/\n/,$initora);
  foreach (@lines) 
  {
    if (/^log_archive_dest_$dest/)
    {
      $initdest = $_;
    }
  }
  print "$P | log_archive_dest_$dest init.ora value: $initdest\n";

  if ($initdest eq "")
  {
    print "$P | ERROR: Could not find a valid log_archive_dest_2 parm in init.ora\n";
    print "$P | Auto restart of broken destintaion cannot occur\n";
    return;
  }

  print "$P | Running $os_scmd - alter system set $initdest;\n";
  $rc = system("$os_scmd <<! 
    $os_conn
    set echo on;
    alter system set $initdest;
    alter system archive log current;
  !");
  print "$P | $os_scmd Done rc=$rc\n";

  print "$P | Sleeping 60\n";
  system("sleep 60");

  print "$P | Rechecking status of dest $dest\n";

  &oraSQL('select \'X\', STATUS, from v\$archive_dest where dest_id = $dest');
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X") 
    { 
      $status  = $words[1];
      last;
    }
  }
  print "$P | Status is $status\n";

  ##print "$P | diag. email\n";
  ##system("echo ' ' | mailx -s 'DBAmon | arcDestFix $host $os $status $initdest' border\@dba.it.agilent.com") if $company eq "agilent";

  if ($status eq "ERROR")
  {
    $destfixmsg = " (Restart attempted, not successful)";
  }
  else
  {
    $destfixmsg = " (Restart successful)";
    $destwarning = 1;
  }
}

# ----------------------------------------------------------------------
sub aliasInfo
# ----------------------------------------------------------------------
{
  &msg("aliasInfo() - Starting");
  # Send host/alias info
  print "$P FQHOSTNAME:     $fqhost\n";
  print "$P HOSTNAME:       $host\n";

  @alias_list = `netstat -i | grep ':' | grep 'com \$' | awk '{ print \$NF }'`;
  foreach (@alias_list)
  {
    chop;
    print "$P NETALIAS:       $_\n";
  }
  &msg("aliasInfo() - Done");
}

# ----------------------------------------------------------------------
sub upParm
# ----------------------------------------------------------------------
{

  $id3file = "/tmp/dbamon_ora_${os}_id3.txt";
  &msg("up() - Saving ID3 value of $arg2 to $id3file");
  system("echo '$arg2' > $id3file 2>&1");

  if ($ENV{'PAGER_EMAIL'} ne "")
  {
    $pagerfile = "/tmp/dbamon_ora_${os}_pager.txt";
    &msg("up() - Saving PAGER_EMAIL value of $ENV{'PAGER_EMAIL'} to $pagerfile");
    $z = $ENV{'PAGER_EMAIL'};
    system("echo '$z' > $pagerfile 2>&1");
  }

  $backup_tape_io_slaves = "";
  $dbwr_io_slaves = "";
  $large_pool_size = "";

  &msg("Checking for conflicting init.ora parameters");
  $parms = lc "'large_pool_size','backup_tape_io_slaves','dbwr_io_slaves','control_file_record_keep_time'";
  &oraSQL("select  \'Z\', name, value from v\\\$parameter where name in ($parms);");
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    if ($words[0] eq "Z")
    {
      $stmt = "\$$words[1] = '$words[2]'\;";
      ##print "stmt=$stmt \n";
      eval $stmt;
    }
  }

  if ($large_pool_size > 0 && $backup_tape_io_slaves eq "FALSE" && $dbwr_io_slaves eq "0")
  {
    print "$P PARMERROR: large_pool_size > 0 & backup_tape_io_slaves = FALSE & dbwr_io_slaves = 0 \n";
  }
  #if ($control_file_record_keep_time < 45 && substr($oraver,0,3) ge "8.1" && $bktype eq "bkRMAN")
  #{
  #  print "$P PARMERROR: Since this 8.1+ instance uses RMAN for backups and no RCVCAT, control_file_record_keep_time (now $control_file_record_keep_time) must be >= 45 \n";
  #}

}

# ----------------------------------------------------------------------
sub alertlog
# ----------------------------------------------------------------------
{
  $S = "Stack: oraAlertsStack ";

  $tmpincl = "$tmpdir/dbamon_ora_alertlog.incl";
  $tmpexcl = "$tmpdir/dbamon_ora_alertlog.excl";
  $tmpout1 = "$tmpdir/dbamon_ora_alertlog.out1";
  $tmpout2 = "$tmpdir/dbamon_ora_alertlog.out2";

  &oraSQL('select value from v\$parameter where name =\'background_dump_dest\'');
  foreach (@oraSQLout)
  {
    if (/^\//)
    {
      @words = split;
      $fname = $words[0];
    }
  }
  print "$S$P alertlog: fname=$fname\n";

  # Isolate INCL lines
  system("grep '^CHKALERT I' $cfg | awk '{ print \$3 }' > $tmpincl");

  # Isolate EXCL lines
  system("grep '^CHKALERT E' $cfg | awk '{ print \$3 }' > $tmpexcl");

  # NOTE: On 21Jan03 I changed the 'grep' in these 2 test from grep to fgrep
  #   for Solaris. 

  # grep for INCL lines
  system("tail -50 $fname/alert_$os.log 2>/dev/null | fgrep -if  $tmpincl > $tmpout1");

  # grep for EXCL lines
  system("cat $tmpout1 | fgrep -vif $tmpexcl > $tmpout2");

  $lines = `wc -l $tmpout2 | awk '{ print \$1 }'`;
  chop $lines;
  print "$S$P alertlog: lines=$lines\n";

  if ($lines > 0)
  {
    print "$S$P alertlog: ALERT Messages Found:\n"; 
    print "$S=======================================================\n";
    @z = `cat $tmpout2`; 
    foreach (@z) { print "$S$_ "; }
    print "$S=======================================================\n";
  }

  # For lightbulb
  print "<b>$dashes</b>\n";
  print "<b> Last 50 lines of Oracle Alert Log: \n   $fname/alert_$os.log    </b>\n";
  print "<b>$dashes</b>\n";

  # Display the last 50 lines of the alert log
  open(CAT,"tail -50 $fname/alert_$os.log 2>/dev/null |") || &error("Cannot cat alert log file $fname");
  while (<CAT>)
  {
    print "${S}alerttext: $_";
  }
  close(CAT);
  
  print "$S$P alertlog: OK\n";
}

# ----------------------------------------------------------------------
sub alertlogWWW
# ----------------------------------------------------------------------
{
  $tmpincl = "$tmpdir/dbamon_ora_alertlog.incl";
  $tmpexcl = "$tmpdir/dbamon_ora_alertlog.excl";
  $tmpout1 = "$tmpdir/dbamon_ora_alertlog.out1";
  $tmpout2 = "$tmpdir/dbamon_ora_alertlog.out2";

  &oraSQL('select value from v\$parameter where name =\'background_dump_dest\'');
  foreach (@oraSQLout)
  {
    if (/^\//)
    {
      @words = split;
      $fname = $words[0];
    }
  }
  &msg("Checking Alert Log $fname For Critical Messages");

  # Isolate INCL lines
  system("grep '^CHKALERT I' $cfg | awk '{ print \$3 }' > $tmpincl");

  # Isolate EXCL lines
  system("grep '^CHKALERT E' $cfg | awk '{ print \$3 }' > $tmpexcl");

  # NOTE: On 21Jan03 I changed the 'grep' in these 2 test from grep to fgrep
  #   for Solaris. 

  # grep for INCL lines
  system("tail -50 $fname/alert_$os.log 2>/dev/null | fgrep -if  $tmpincl > $tmpout1");

  # grep for EXCL lines
  system("cat $tmpout1 | fgrep -vif $tmpexcl > $tmpout2");

  $lines = `wc -l $tmpout2 | awk '{ print \$1 }'`;
  chop $lines;
  &msg("Found $lines Critical Alert Messages");

  $alertmsg = "OK";

  if ($lines > 0)
  {
    print "$S=======================================================\n";
    @z = `cat $tmpout2`; 
    foreach (@z) { print "$S$_"; }
    print "$S=======================================================\n";
    $alertflag = 1;
    $alertmsg = ">>> Critical Error Message(s) Found <<<";
  }
}

# ----------------------------------------------------------------------
sub DBOpenCheck
# ----------------------------------------------------------------------
{
  $dboc_debug = 1;
  &msg("DBOpenCheck() - Ensuring DB is OPEN") if $dboc_debug;

  my $dboc_status = "?";
  my $dboc_diag   = "";
 
  my $dboc_sql = 'select \'X\', status from v\$instance;';

  my @dboc_oraSQLout = `$os_scmd <<!
    $os_conn;
    $os_set
    $dboc_sql
    ;
  !`;

  foreach (@dboc_oraSQLout)
  {
    print if /^ORA/;
    @dboc_words = split;
    $dboc_verb  = $dboc_words[0];
    if ($dboc_verb eq "ORA-00904:") 
    {
      #---- This must be oracle v7 - return false open
      &msg("This must be Oracle V7 - returning false open"); 
      $dboc_status = "OPEN";
      last;
    }
    if ($dboc_verb eq "X") { $dboc_status = $dboc_words[1]; }
    $dboc_diag .= $_;
  }
  &msg("DBOpenCheck() - Returning $dboc_status") if $dboc_debug;

  if ($dboc_status ne "OPEN")
  {
    &msg("DBOpenCheck() - DB Status is $dboc_status (should be OPEN) - $os_scmd Output: \n$dboc_diag");
    ##system("echo ' ' | mailx -s 'DBAmon | $host/$os - Dropped-OPEN - dboc_status=$dboc_status' border\@dba.it.agilent.com") if $company eq "agilent";
  }

  return $dboc_status;
}

# ----------------------------------------------------------------------
sub dfCheck
# ----------------------------------------------------------------------
{
  #---- Count the rows in dba_data_files - This will catch DB's with df problems 
  &msg("Counting rows in dba_data_files");
 
  $sqlstmt = '
    select \'X\', count(*) from dba_data_files;
  ';

  $dfcount = "?";

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    print if /^ORA/;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X")
    {
      $dfcount = $words[1];
    }
  }
  &msg("Rows in dba_data_files: $dfcount");
 
  if ($dfcount eq "?")
  {
    system("echo ' ' | mailx -s 'DBAmon | $host/$os - Cound not select dba_data_files' border\@dba.it.agilent.com") if $company eq "agilent";
    &error("Cound not run 'select count(*) from dba_data_files'");
  }
}

# ----------------------------------------------------------------------
sub filesList
# ----------------------------------------------------------------------
{
  # List all datafiles, controlfile and redo logs as comments for the alertlog
  print "Stack: oraSaveCfgStack # \n";
  print "Stack: oraSaveCfgStack # Report of all Datafiles, RedoLogs and ControlFiles:\n";
  print "Stack: oraSaveCfgStack # \n";

  #---- BEFORE select \'X\', file_name, tablespace_name from dba_data_files

  $sqlstmt = '
  select \'X\', V\$datafile.name, V\$tablespace.name from V\$datafile, V\$tablespace where V\$datafile.TS# = V\$tablespace.TS#
  union all
  select \'Y\', member, \' \' from v\$logfile
  union all
  select \'Z\', name, \' \' from v\$controlfile
  union all
  select \'W\', to_char(dbid), \' \' from v\$database
  order by 1, 2;';

  ##&msg("Running sqlstmt=$sqlstmt"); 

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    $ts      = $words[2];
    if ($verb eq "X")
    {
      $f = $words[1];
      print "Stack: oraSaveCfgStack # Datafile:    $f  (TS=$ts)\n";
    }
    if ($verb eq "W")
    {
      $dbid = $words[1];
      print "Stack: oraSaveCfgStack # DBID:  $dbid \n";
    }
    if ($verb eq "Y")
    {
      $f = $words[1];
      print "Stack: oraSaveCfgStack # RedoLog:     $f \n";
    }
    if ($verb eq "Z")
    {
      $f = $words[1];
      print "Stack: oraSaveCfgStack # ControlFile: $f \n";
    }
  }
}

# ----------------------------------------------------------------------
sub onlineRedo
# ----------------------------------------------------------------------
{
  print "$P | Checking for Online Redo Logs in exception status \n";

  $sqlstmt = 'select \'Y\', group#, status, member from v\$logfile where status is not null';
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    $group      = $words[1];
    $status     = $words[2];
    $member     = $words[3];
    if ($verb eq "Y")
    {
      print "ONLINEREDOSTATUS: Group=$group File=$member \n";
    }
  }

  $sqlstmt = 'select 
    \'X\', 
    archiver,
    translate(log_switch_wait,\' \',\'_\')
  from
    v\$instance; 
  select
    \'Y\', 
    log_mode
  from
    v\$database; 
  ';
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X")
    {
      $archiver      = $words[1];
      $logswitchwait = $words[2];
      print "$P ARCHIVER:      $archiver\n";
      print "$P LOGSWITCHWAIT: $logswitchwait\n";
    }
    if ($verb eq "Y")
    {
      $arclogmode = $words[1];
      print "$P ARCLOGMODE:    $arclogmode\n";
    }
  }

  &msg("Running orastat -l to display current log details");
  &msg("================================================================");
  system("$progdir/dbamon_orastat.pl -l 2>&1");
  &msg("================================================================");
}

# ----------------------------------------------------------------------
sub processesCount
# ----------------------------------------------------------------------
{
  print "$P | Counting PROCESSES vs. init.ora value \n";
  $sqlstmt = '
  select \'Y\', count(*) from v\$process;
  select \'Z\', value from v\$parameter where name = \'processes\';';
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    $p_count  = $words[1] if ($verb eq "Y");
    $p_config = $words[1] if ($verb eq "Z");
  }
  print "$P PROCESSES_THRESHOLD: $p_count $p_config \n";
}

# ----------------------------------------------------------------------
sub lsnr_start
# ----------------------------------------------------------------------
{
  # dr 01Oct01 Unset SHLIB_PATH prior to starting the listener. If there
  # are multiple oracle versions, SHLIB_PATH cannot be set when
  # starting the listener.
  
  print "$S$P lsnr: Attempting to restart listener:\n";
  $ENV{'SHLIB_PATH'} = "";
  $rc = system('lsnrctl <<! 
$lsnr_pwline
start
quit
!');
  print "$S$P lsnr: Restart rc=$rc\n";
  $lsnrerror = 'Lsnr Restart Attempted - rc=$rc';
}

# ----------------------------------------------------------------------
sub lsnr_check
#
# 01/24/00 DR: Allow for multiple tnslsnr processes, as you can run
#              more than one listener on a machine.
# ----------------------------------------------------------------------
{
  # Extract lsnr password (if any)

  
  if ($lsnr_file ne "")
  {
    chop($lsnr_pw = `cat $lsnr_file | grep -v '^#' | grep -i 'PASSWORDS_LISTENER' | awk -F= '{ print \$NF }'`);
  }

  if ($lsnr_pw ne "")
  {
    $lsnr_pwline = "set password $lsnr_pw";
    &msg("Listener password is set - lsnr_cmd=$lsnr_pwline");
  }
  else
  {
    $lsnr_pwline = "";
    &msg("Listener password is not set - lsnr_cmd=$lsnr_pwline");
  }

  $lsnr_cmd = "lsnrctl <<!
    $lsnr_pwline
    status
  !";

  # Check for lsnr process
  $os_type = `uname -a`;
  $lc_os_type=lc($os_type);
  if ($lc_os_type =~ "linux")
  {
    $numlsnr = `ps -ef --cols 200 | grep 'tnslsnr' | grep -v 'grep' | wc -l`;
  }
  else 
  {
    $numlsnr = `ps -ef | grep 'tnslsnr' | grep -v 'grep' | wc -l`;
  }
  chop $numlsnr;
  print "$S$P NUMLSNR: $numlsnr\n";

  print "$P | Creating /tmp/$tmp_tnsnames_dir/tnsnames.ora\n";
  
  # See if this is an SAP system
  if (system("grep 'SAP.WORLD' $oh/network/admin/listener.ora 2>&1") == 0)
  {
    print "$P | This is a SAP system - altering tnsnames data\n";
    $tns_port = 1527;
    $tns_comm = "(COMMUNITY = SAP.WORLD)";
  }
  else
  {
    print "$P | This is a NOT an SAP system\n";
    $tns_port = 1521;
    $tns_comm = "";
  }

  # Create tnsnames.ora file
  $entry = "$os=
 (DESCRIPTION =
  (ADDRESS_LIST =
   (ADDRESS=(PROTOCOL=TCP)  $tns_comm  (HOST=$fqhost) (PORT=$tns_port)))
    (CONNECT_DATA = (SID = $os)))
$os.world=
 (DESCRIPTION =
  (ADDRESS_LIST =
   (ADDRESS=(PROTOCOL=TCP)  $tns_comm  (HOST=$fqhost) (PORT=$tns_port)))
    (CONNECT_DATA = (SID = $os)))";

  &msg("Cleanup up possible old files (error messages are OK)");
  system("rm -fr /tmp/$tmp_tnsnames_dir 2>&1");
  system("mkdir  /tmp/$tmp_tnsnames_dir 2>&1");
  system("rm -fr    /tmp/$tmp_tnsnames_dir/tnsnames.ora 2>&1");
  system("touch     /tmp/$tmp_tnsnames_dir/tnsnames.ora 2>&1");
  system("chmod 777 /tmp/$tmp_tnsnames_dir/tnsnames.ora 2>&1");

  &msg("Writing new /tmp/$tmp_tnsnames_dir/tnsnames.ora");

  close(TNS);
  open(TNS, ">/tmp/$tmp_tnsnames_dir/tnsnames.ora") || &error("can't open output /tmp/$tmp_tnsnames_dir/tnsnames.ora - $!");
  print TNS "$entry \n";
  close(TNS);

  &msg("Dynamically created TNSNAMES dir");
  system("ls -l /tmp/$tmp_tnsnames_dir/*");
  system("cat /tmp/$tmp_tnsnames_dir/*");

  $ENV{"TNS_ADMIN"} = "/tmp/$tmp_tnsnames_dir";
  
  # Check listener status
  &msg("lsnr_check(): Running lsntctl status");
  $lsnrout = `$lsnr_cmd`;
  print "\nlsnrout=$lsnrout \n\n";
  if (index($lsnrout,"successfully") > -1)
  {
    $lsnrstat = 1;
  }
  else
  {
    $lsnrstat = 0;
  }
  print "$S$P LSNRSTAT: $lsnrstat $lsnrerror\n";

  # try tnsping
  # Bug in 8.1.6 when running MTS prevents tnsping from working.
  # If using MTS on an OPS system, keyword REGISTER appears in
  # tnsnames, so look for that.

  $mts = `grep -i 'register' /etc/tnsnames.ora 2>/dev/null | wc -l`;
  chop $mts;
  if ($mts == 0)
  { 
    &msg("lsnr_check(): Running tnsping $os.world");
    $tnspingout = `tnsping $os.world`;
    print $tnspingout;
    if (index($tnspingout,"OK") > -1)
    {
      $tnsstat = 1;
    }
    else
    {
      &msg("=============================================================");
      &msg("Current dynamic /tmp/$tmp_tnsnames_dir/tnsnames.ora file contents:");
      system("cat /tmp/$tmp_tnsnames_dir/tnsnames.ora 2>&1");
      &msg("=============================================================");
      $tnsstat = 0;
      $z = substr($tnspingout,index($tnspingout,'TNS-'));
      @words = split(/\n/,$z);
      $tnserror = $words[0];
    }
    print "$S$P TNSPING: $tnsstat $tnserror\n";
  }
  else 
  { 
    $tnsstat = $lsnrstat; 
    print "$S$P TNSPING: $tnsstat\n";
  }

  # Remove /tmp/$tmp_tnsnames_dir to resolve a bug with db_links
  system("rm -fr /tmp/$tmp_tnsnames_dir 2>&1");

  if ($numlsnr >= 1)
  {
    return 0;
  }
  else
  {
    return 1;
  }
}

# ----------------------------------------------------------------------
sub userPassword
# ----------------------------------------------------------------------
{
  $runmon = $arg4;   # is this instance backup-only monitoring?
  
  if ($thisos ne "Linux")
  {
    # Isolate pw lines
    chop($newsyspw = `cat $progdir/dbamon_ora.key1 | crypt dbamon`);
  }

  # TESTING
  ##print "newsyspw=$newsyspw\n";

  # one-time agilent test
  if (-e "/opt/oracle/adm/config/.system" && 
     system("grep 'manager' /opt/oracle/adm/config/.system") == 0)
  {
    system("echo $os | mailx -s 'DBAmon | $host/$os - .system Is set to manager [AUTO]' border\@dba.it.agilent.com");
  }

  if ($logswitchwait eq "ARCHIVE_LOG")
  {
    &msg("This DB appears to be hung due to no free archivelog destinations - skipping PW check");
    return;
  }
  
  # See how many users there are
  &msg("Looking for non-SYS users granted the DBA role");
  $sqlstmt = 'select  \'X\', grantee from dba_role_privs where granted_role = \'DBA\' and grantee not in (\'SYS\',\'SYSTEM\',\'CTXSYS\',\'WKSYS\',\'SYSMAN\');';
  &oraSQL($sqlstmt);
  $grantdba = 0;
  foreach (@oraSQLout)
  {
    ##print;
    print if /^ORA/;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X")
    {
      $grantee = $words[1];
      print "$P DBAROLE: $grantee\n";
      $grantdba++
    }
  }
  &msg("There are $grantdba rows in DBA_USERS");

  # See how many users there are
  &msg("Counting rows in DBA_USERS");
  # Collect user exception data
  $sqlstmt = 'select \'X\', count(*) from dba_users;';
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X")
    {
      $usercount = $words[1];
    }
  }
  &msg("There are $usercount rows in DBA_USERS");

  # we don't want to check more than 100 userids each iteration. so
  # if there are > 100 then we want to take a random sample of userids
  if ($usercount > 100)
  {
    &msg("Since there are > 100 users, we will do RANDOM checking"); 
    $rand = int($usercount / 100);
    # add a random number between 1 and 5
    $z = int(rand 5);
    $rand += $z;
    &msg("rand=$rand z=$z"); 
  }
  else
  {
    $rand = 0;
  }

  &msg("Checking for non-system users with obvious passwords");
  # Collect user exception data
  $sqlstmt = '
    select
      \'USER_LIST\',
      us.username
    from
      dba_users us
    where
      us.username not in
	(\'SYS\',\'SYSTEM\',\'OUTLN\',\'DBSNMP\',\'RMAN\')
    order by
      2
      ;
  ';

  &oraSQL($sqlstmt);
  $usersql = "";
  $userstocheck = 0;
  $thisuser = 0;
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if (
        $verb eq "USER_LIST" 
       )
    {
      $thisuser++;
    
      if ($rand > 0)  # if we are just sampling ...
      {
        if ( ($thisuser / $rand) != int($thisuser / $rand) )
        {
          next;
        } 
      }
      ##print "thisuser=$thisuser rand=$rand\n";

      $user   = $words[1];

      $usersql .= "disconnect ;\n";
      $usersql .= "connect $user/$user ;\n";
      $usersql .= "select 'X', user from dual ;\n";

      $userstocheck++;
    }
  }

  ##print "usersql=$usersql\n";
  &msg("Begining user connection test for $userstocheck userids");

  $s_time = time;

  $save_os_conn = $os_conn;
  $os_conn = "";
  &oraSQL($usersql);

  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if ( $verb eq "X" )
    {
      $user = $words[1];
      print "$P USER=PW: $user\n";
    }
  } 

  $os_conn = $save_os_conn;

  $elapsed = time - $s_time;
  &msg("previous check took $elapsed seconds");
  if ($elapsed > 300)
  {
    system("echo $os | mailx -s 'DBAmon | $host/$os - Non-sys pw check took $elapsed seconds t=300 [AUTO]' border\@dba.it.agilent.com") if $company eq "agilent";
  }

  &msg("Checking for system users with obvious passwords");
  $usersql = "";

  $usersql .= "disconnect ;\n";
  $usersql .= "connect system/manager ;\n";
  $usersql .= "select 'X', user from dual ;\n";

  $usersql .= "disconnect ;\n";
  $usersql .= "connect system/system ;\n";
  $usersql .= "select 'X', user from dual ;\n";

  $usersql .= "disconnect ;\n";
  $usersql .= "connect sys/changeoninstall ;\n";
  $usersql .= "select 'X', user from dual ;\n";

  $usersql .= "disconnect ;\n";
  $usersql .= "connect sys/sys ;\n";
  $usersql .= "select 'X', user from dual ;\n";

  $usersql .= "disconnect ;\n";
  $usersql .= "connect outln/outln ;\n";
  $usersql .= "select 'X', user from dual ;\n";

  $usersql .= "disconnect ;\n";
  $usersql .= "connect dbsnmp/dbsnmp ;\n";
  $usersql .= "select 'X', user from dual ;\n";

  ##print "usersql=$usersql\n";

  $save_os_conn = $os_conn;
  $os_conn = "";
  &oraSQL($usersql);

  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if ( $verb eq "X" )
    {
      $user = $words[1];

      if ($newsyspw ne "" && $runmon ne "B")
      {
	# Change pw
	&msg("Changing password for user=$user to cfg value");
	&oraSQL("alter user $user identified by $newsyspw;");
	foreach (@oraSQLout)
	{
	  print;
	}
        print "$P USER=PW-CHANGED: $user\n";
      }
      else
      {
        print "$P USER=PW-SYS: $user\n";
      }

    }
  } 
  $os_conn = $save_os_conn;

}

# ----------------------------------------------------------------------
sub perf
# ----------------------------------------------------------------------
{
  &msg("Running SQL to collect perf data");

  $sqlstmt = '
  alter session set NLS_NUMERIC_CHARACTERS = \'.,\';
  select
    \'F\', count(*) from v\$bh where status = \'free\';
  select
    \'P\',
    name,
    100* (1-(physical_reads / (DB_BLOCK_GETS + CONSISTENT_GETS)))
    from
      v\$buffer_pool_statistics;
  select
    \'V\', 
    round((sum(gethits) / sum(gets) * 100),2),
    sum(gets)
     from v\$librarycache;
  select 
    \'U\', 
    round((sum(gets) - sum(getmisses)) / sum(gets) * 100,2),
    sum(gets)
     from v\$rowcache;
  select
    \'G\', count(*) from dba_tables 
      where owner = \'SYS\' and last_analyzed is not null and table_name not in (\'DUAL\',\'PLAN_TABLE\');
  select
    \'R\', bytes from v\$sgastat where name = \'free memory\' and pool = \'shared pool\';
  select
    \'S\', value from v\$parameter where name = \'shared_pool_size\';
  select
    \'T\', value from v\$parameter where name = \'db_block_buffers\';
  select
    \'J\', value from v\$parameter where name = \'db_block_size\';
  select
    \'L\', sum(totalq) from v\$queue;
  select
    \'M\', value from v\$parameter where name = \'undo_management\';
  select
    \'N\', force_logging from v\$database;
  select
    \'Q\', value from v\$parameter where name = \'db_cache_size\';
  select
    \'H\', max(granule_size) from v\$sga_dynamic_components; 
  select
    \'K\', value from v\$parameter where name = \'spfile\';
  select
    \'X\', value from v\$parameter where name = \'sql_trace\';
  select 
    \'W\',
       sum(decode(name, \'consistent gets\', value, 0)) consistent,
       sum(decode(name, \'db block gets\',   value, 0)) db_blockgets,
       sum(decode(name, \'physical reads\',  value, 0)) physical,
       round(((sum(decode(name, \'consistent gets\', value, 0)) +
               sum(decode(name, \'db block gets\',   value, 0)) -
               sum(decode(name, \'physical reads\',  value, 0))) /
              (sum(decode(name, \'consistent gets\', value, 0)) +
               sum(decode(name, \'db block gets\',   value, 0)))) * 100, 2)
    hitratio from v\$sysstat;
  select
    \'Z\',
    100* (1-(sum(physical_reads) / (sum(DB_BLOCK_GETS) + sum(CONSISTENT_GETS))))
  from
    v\$buffer_pool_statistics
  where
    (db_block_gets + consistent_gets) > 0
    and name != \'RECYCLE\';
  ';

  $sqlgood = 0;
  $free_buffers = -1;
  
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if ($verb eq "F")
    {
      print "F $words[1]\n";
      $free_buffers = $words[1];
    }

    if ($verb eq "R")
    {
      ##print "R $words[1]\n";
      $sg_free = $words[1];
    }

    if ($verb eq "S")
    {
      ##print "S $words[1]\n";
      $sg_size = $words[1];
      if ($sg_size =~ /M$/)
      {
        chop($sg_size);
        $sg_size *= 2**20;
      }
    }

    if ($verb eq "K") { $spfile = $words[1]; }
    if ($verb eq "T") { $db_block_buffers = $words[1]; }
    if ($verb eq "J") { $db_block_size = $words[1]; }
    if ($verb eq "Q") { $db_cache_size = $words[1]; }
    if ($verb eq "X") { $sql_trace = $words[1]; }

    if ($verb eq "G") { $sys_analyzed = $words[1]; }

    if ($verb eq "L") { $mts_queue = $words[1]; }
    if ($verb eq "M") { $undo_mgmt = $words[1]; }
    if ($verb eq "N") { $force_logging = $words[1]; }

    if ($verb eq "H") { $granule_size = $words[1]; }

    if ($verb eq "P")
    {
      $pool_keep_hitratio    = $words[2] if $words[1] eq "KEEP";
      $pool_recycle_hitratio = $words[2] if $words[1] eq "RECYCLE";
      $pool_default_hitratio = $words[2] if $words[1] eq "DEFAULT";
    }

    if ($verb eq "W") 
    { 
      $bufcongets   = $words[1];
      $bufblkgets   = $words[2];
      $bufphysreads = $words[3];
      $bufhitratio  = $words[4];
    }

    if ($verb eq "Z") 
    { 
      $bufhitratio_xr   = $words[1];
    }

    if ($verb eq "V") 
    { 
      $libcachehitratio = $words[1];
      $libcachegets     = $words[2];
    }

    if ($verb eq "U") 
    { 
      $dictcachehitratio = $words[1];
      $dictcachegets     = $words[2];
    }
  }
  close(INDB);

  #Edits
  $pool_default_hitratio = sprintf("%.2f", $pool_default_hitratio) if $pool_default_hitratio ne "";
  $pool_recycle_hitratio = sprintf("%.2f", $pool_recycle_hitratio) if $pool_recycle_hitratio ne "";
  $pool_keep_hitratio    = sprintf("%.2f", $pool_keep_hitratio)    if $pool_keep_hitratio ne "";

  if ($sg_size > 0)
  {
    $sg_pct = 100 - (($sg_free / $sg_size) * 100);
  }
  else
  {
    $sg_pct = ".";
  }

  chop($num_otrace = `ls -l \$ORACLE_HOME/otrace/admin/*.dat 2>/dev/null | wc -l`); 

  print "$P SGA:              $sg_pct $sg_size $sg_free \n";
  print "$P POOL_DEFAULT_HR:  $pool_default_hitratio\n"; 
  print "$P POOL_RECYCLE_HR:  $pool_recycle_hitratio\n"; 
  print "$P POOL_KEEP_HR:     $pool_keep_hitratio\n"; 
  print "$P BUFBLKGETS:       $bufblkgets\n"; 
  print "$P BUFCONGETS:       $bufcongets\n"; 
  print "$P BUFPHYSREADS:     $bufphysreads\n"; 
  print "$P BUFHITRATIO:      $bufhitratio\n"; 
  print "$P BUFHITRATIO_XR:   $bufhitratio_xr\n"; 
  print "$P LIBHITRATIO:      $libcachehitratio\n"; 
  print "$P LIBGETS:          $libcachegets\n"; 
  print "$P DICTHITRATIO:     $dictcachehitratio\n"; 
  print "$P DICTGETS:         $dictcachegets\n"; 
  print "$P DB_BLOCK_BUFFERS: $db_block_buffers\n"; 
  print "$P DB_BLOCK_SIZE:    $db_block_size\n"; 
  print "$P DB_CACHE_SIZE:    $db_cache_size\n"; 
  print "$P GRANULE_SIZE:     $granule_size\n"; 
  print "$P NUM_OTRACE:       $num_otrace\n"; 
  print "$P SQL_TRACE:        $sql_trace\n"; 
  print "$P FREE_BUFFERS:     $free_buffers\n"; 
  print "$P SPFILE:           $spfile\n"; 
  print "$P MTS_QUEUE:        $mts_queue\n"; 
  print "$P UNDO_MGMT:        $undo_mgmt\n"; 
  print "$P FORCE_LOGGING:    $force_logging\n"; 
  print "$P SYS_ANALYZED:     $sys_analyzed\n"; 

  # Find the names of suitable USER* and TEMP* tablespaces
  &msg("Looking for suitable USER* and TEMP* tablespaces");
  $sqlstmt = '
    select
      \'USER_TS\',
      ts.tablespace_name
    from
      dba_tablespaces ts
    where
      ts.tablespace_name like \'USER%\'
    order by
      1 ;

    select
      \'ALT_USER_TS\',
      us.default_tablespace,
      count(*) 
    from
      dba_users us
    where 
      us.default_tablespace != \'SYSTEM\'
    group by
      \'ALT_USER_TS\',
      us.default_tablespace
    order by 3 desc
      ;

    select
      \'ALT_TEMP_TS\',
      us.temporary_tablespace,
      count(*) 
    from
      dba_users us
    where 
      us.temporary_tablespace != \'SYSTEM\'
    group by
      \'ALT_TEMP_TS\',
      us.temporary_tablespace
    order by 3 desc
      ;

    select
      \'TEMP_TS\',
      ts.tablespace_name
    from
      dba_tablespaces ts
    where
      ts.tablespace_name like \'TEMP%\'
      and ts.contents = \'TEMPORARY\'
    order by
      1; ';

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if ($verb eq "TEMP_TS" && $good_temp_ts eq "")
    {
      $good_temp_ts = $words[1];
      print "good_temp_ts=$good_temp_ts\n";
    }
    if ($verb eq "USER_TS" && $good_users_ts eq "")
    {
      $good_users_ts = $words[1];
      print "good_users_ts=$good_users_ts\n";
    }
    if ($verb eq "ALT_USER_TS" && $alt_users_ts eq "")
    {
      $alt_users_ts = $words[1];
      print "alt_users_ts=$alt_users_ts\n";
    }
    if ($verb eq "ALT_TEMP_TS" && $alt_temp_ts eq "")
    {
      $alt_temp_ts = $words[1];
      print "alt_temp_ts=$alt_temp_ts\n";
    }
  }

  if ($good_users_ts eq "") { $good_users_ts = $alt_users_ts; }
  if ($good_temp_ts eq "")  { $good_temp_ts  = $alt_temp_ts; }

  if ($good_temp_ts eq "" || $good_temp_ts eq "")  
  { 
    system("echo ' ' | mailx -s 'DBAmon | perf() temp=$good_temp_ts users=$good_users_ts alt_users_ts=$alt_users_ts alt_temp_ts=$alt_temp_ts border\@dba.it.agilent.com") if $company eq "agilent";
  }

  print "$P GOOD_USERS_TS: $good_users_ts\n";
  print "$P GOOD_TEMP_TS:  $good_temp_ts\n";

  # START - TEMP FIX - there are many DB's where there is a RPT user that keeps 
  #  getting set to temp=tools; Try to fix that. 

  &oraSQL("select 'X' from dba_users where username = 'RPT' and TEMPORARY_TABLESPACE = 'TOOLS';");
  &msg("Looking for RPT user with TOOLS as temporary tablespace");
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X")
    {
      &msg("Altering RPT to set temporary tablespace to TEMP");
      &oraSQL("alter user RPT temporary tablespace TEMP;");
    }
  }

  # END   - TEMP FIX - there are many DB's where there is a RPT user that keeps 
  #  getting set to temp=tools; Try to fix that. 

  &msg("Done collecting user exception data - temp=$good_temp_ts users=$good_users_ts alt_users_ts=$alt_users_ts alt_temp_ts=$alt_temp_ts");

  #---- User exceptions - Fix 
  &userException("Fix");

  #---- User exceptions - Report 
  &userException("Report");

  #---- Collect ts exception data
  &msg("Running SQL to find tablespaces in exception status");
  $sqlstmt = '

    select
      \'TEMP_PERM\',
      ts.tablespace_name
    from
      dba_tablespaces ts
    where
      tablespace_name = \'TEMP\'
      and ts.contents = \'PERMANENT\'
      ;
  ';

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if (
        $verb eq "TEMP_PERM" 
       )
    {
      $ts   = $words[1];
      print "$P TS_EXCEPTION: $verb: $ts\n";
    }
  }
  &msg("Done collecting tablespace exception data");

  &perfRollback;

  if ($thisos eq "HP-UX")
  {
    #
    # Sample NFILE usage
    #

    open(IN, "sar -v 1 | tail -1 |") || &error("Error running sar -v");

    while(<IN>)
    {
      ##print;
      @words     = split;
      $nfileraw  = $words[7];
      ##print "nfileraw=$nfileraw\n";
      @words2    = split(/\//,$nfileraw);
      $current   = $words2[0];
      $max       = $words2[1];
      if ($max > 0)
      {
        $pct       = ($current/$max*100);
      }
      else
      {
        $pct       = "?";
      }
     
    }
    printf("$P NFILE: %4d %4d %4d\n",$current,$max,$pct);
  }

  &msg("Done collecting perf data");
}

# ----------------------------------------------------------------------
sub userException
# ----------------------------------------------------------------------
{
  my $mode = $_[0];
  &msg("userException() - mode=$mode");

  # Collect user exception data
  &msg("Running SQL to find users in exception status");
  $sqlstmt = '
    select
      \'DFLT_SYS\',
      us.username,
      us.default_tablespace,
      us.temporary_tablespace
    from
      dba_users us
    where
      default_tablespace = \'SYSTEM\'
      and us.username not in
	(\'SYS\',\'SYSTEM\',\'OUTLN\',\'DBSNMP\',\'DBAMON\')
      and us.username not like \'%\$%\';
      ;

    select
      \'TEMP_SYS\',
      us.username,
      us.default_tablespace,
      us.temporary_tablespace
    from
      dba_users us
    where
      temporary_tablespace  = \'SYSTEM\'
      and us.username not in
	(\'SYS\',\'SYSTEM\',\'OUTLN\',\'DBSNMP\',\'DBAMON\')
      and us.username not like \'%\$%\';
      ;

    select
      \'TEMP_PERM\',
      us.username,
      us.default_tablespace,
      us.temporary_tablespace
    from
      dba_users us,
      dba_tablespaces ts
    where
      us.temporary_tablespace = ts.tablespace_name
      and ts.contents = \'PERMANENT\'
      and us.default_tablespace != \'SYSTEM\'
      and us.username not in
	(\'SYS\',\'SYSTEM\',\'OUTLN\',\'DBSNMP\',\'DBAMON\')
      and us.username not like \'%\$%\';
      ;
  ';

  my $sql = "";
  my $count = 0;

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    
    if (
        $verb eq "DFLT_SYS" ||
        $verb eq "TEMP_SYS" ||
        $verb eq "TEMP_PERM" 
       )
    {
      $user   = $words[1];
      $dfltts = $words[2];
      $tmpts  = $words[3];

      $count++;

      if ($mode eq "Report") 
      {
        print "$P USER_EXCEPTION: $verb: $user $dfltts $tmpts\n";
      }
      else  #---- mode=Fix
      {
        &msg("Found user=user dfltts=$dfltts tmpts=$tmpts"); 

        if ($verb eq "DFLT_SYS" && $good_users_ts)
        {
	  $sql .= "ALTER USER $user DEFAULT TABLESPACE $good_users_ts;\n";
        }
        if ($verb eq "TEMP_SYS" && $good_temp_ts)
        {
	  $sql .= "ALTER USER $user TEMPORARY TABLESPACE $good_temp_ts;\n";
        }
        if ($verb eq "TEMP_PERM" && $good_temp_ts)
        {
	  $sql .= "ALTER USER $user TEMPORARY TABLESPACE $good_temp_ts;\n";
        }
      }
    }
  }

  if ($sql)
  {
    &msg("Running sql=\n$sql"); 
    system("$os_scmd <<! 
	  $os_conn
	  $sql
	!");
  }

  &msg("Done collecting user exception data - count=$count mode=$mode");
}

# ----------------------------------------------------------------------
sub perfRollback
# ----------------------------------------------------------------------
{
  &msg("Checking rollback segments");
  #
  # Check for rollback segment header waits
  #

  $sqlstmt = 'select \'X\', 
    rn.NAME,
    rs.EXTENTS,
    rs.EXTENDS,
    rs.RSSIZE,
    rs.XACTS,
    rs.GETS,
    rs.HWMSIZE,
    rs.WRAPS,
    rs.AVEACTIVE,
    rs.STATUS,
    rs.WAITS,
    rs.optsize
  from 
    v\$rollstat rs, 
    v\$rollname rn 
  where 
    rn.usn=rs.usn;';

  print "Name          Extents  Extends         RSSize   A Xn's      Gets         HWMSize  Wraps   Avg Actv  Status   Waits Optimal\n";
  print "------------ -------- -------- -------------- -------- ---------- -------------- ------  --------- -------- ------ -------\n";

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words     = split();
    $verb      = $words[0];
    $name      = $words[1];
    $extents   = $words[2];
    $extends   = $words[3];
    $rssize    = $words[4];
    $xacts     = $words[5];
    $num_gets  = $words[6];
    $hwmsize   = $words[7];
    $wraps     = $words[8];
    $aveactive = $words[9];
    $status    = $words[10];
    $waits     = $words[11];
    $optimal   = $words[12];
    if ($verb eq "X")
    {
      printf "%-12s %8d %8d %14d %8d %10d %14d %6s %10d %-8s %6d %-8s\n",
        $name,$extents,$extends,$rssize,$xacts,$num_gets,$hwmsize,$wraps,$aveactive,$status,$waits,$optimal;
      $t_waits += $waits;
      $t_gets  += $num_gets;
    }
  }

  $waits_gets_r = 100 * ($t_waits/$t_gets) if $t_gets > 0;
  $waits_gets_r = sprintf("%8.5f",$waits_gets_r);

  print "$P Rollback_Total_Waits:   $t_waits\n";
  print "$P Rollback_Total_Gets:    $t_gets\n";
  print "$P Rollback_Waits_To_Gets: $waits_gets_r%\n";

  if ($waits_gets_r > 1)
  {
    &msg("Because the ratio is > 1, add more rollback segments");
  }

}

# ----------------------------------------------------------------------
sub up
# ----------------------------------------------------------------------
{
  &msg("up() - Checking for Oracle Up");

  # bb 10Jan00 - The Roseville dbshut and dbstart scripts create an 
  #  entry in an "inhibit" file when the DB is intentionally down
  #  for backups. Check that file. If the oracle_sid is present, don't
  #  bother to monitor and mark it inhibited.

  #---- test
  ##system("ps -ef | grep 'ora_pmon_$os\$' | grep -v 'grep' | wc -l"); 

  chop($numpmon = `ps -ef | grep 'ora_pmon_$os\$' | grep -v 'grep' | wc -l`); 
  if ($numpmon eq "0") 
  {
    print "$P | PMON for ORACLE_SID $os is not active \n";
  } 

  # create var of init.ora background_dump_dest
  &msg("Reading init.ora file $oh/dbs/init$os.ora");
  open(RO,"$oh/dbs/init$os.ora") || &error("Cannot OPEN init.ora file=$oh/dbs/init$os.ora");
  while (<RO>)
  {
    $initora .= $_;
    @words = split(/=/);
    if (substr($words[0],0,5) eq "ifile")
    {
      $ifile = $words[$#words];
      $initora .= "#**** Contents of ifile: $ifile *****\n";
      # if the 1st char of ifile is ?, then xlate it to oracle_home
      if ($ifile =~ /\?/)
      {
	$ifile =~ s/\?/$oh/;
        chop($z = $ifile);
	&msg("ifile Promoted from ?... to $z");
      }
      open(RO2,"$ifile") || &error("Cannot OPEN init.ora file=$ifile");
      while (<RO2>)
      {
        $initora .= $_;
      }
      close(RO2);
	      $initora .= "#**** End of ifile: $ifile *****\n";
	    }
	  }
	  close(RO);


	  # Check for pmon process
	  if ($numpmon eq "0")
	  {
	    print "$P | PMON is not active - checking to see how Oracle come down \n";

	    # find background_dump_dest
	    @lines = split(/\n/,$initora);
	    foreach (@lines) 
	    {
	      if (/^background_dump_dest/)
	      {
		$z1 = index($_,"=");
		$z2 = substr($_,$z1 + 1); 
		$bdumpdir1 = (split(/ /,$z2)) [0];
		$bdumpdir2 = (split(/ /,$z2)) [1];
		$bdumpdir = $bdumpdir1 if $bdumpdir1 ne "";
		$bdumpdir = $bdumpdir2 if $bdumpdir2 ne "";
		print "$P | background_dump_dest=$bdumpdir \n";
        $alertlog = "${bdumpdir}/alert_$os.log";
        print "$P | alertlog=$alertlog \n";
        $last2 = `tail -2 $alertlog`;
        if (index($last2,"terminating instance due to error") > -1)
        {
          $ora_crashed = 1;

          @lastlines = split(/\n/,$last2);
          $errorcode = (split(/ /,$lastlines[0])) [6];
          $errortext = `oerr ora $errorcode`;
          
          print "$P | -----------------------------------------------------------------\n";
          print "$P | Oracle crashed - Error: ORA-$errorcode - Explanation: \n";
          print "$errortext \n";

          print "$P | Diagnostic Data: \n";
          print $last2;

          if ($arg1 ne "N" && $errortext =~ /Warm start instance/)
          {
            print "\n$P | Since Must_Be_Up:=Y and Explanation (above) recommends 'Warm start instance', Initiating Auto Restart\n";

            system("$os_scmd <<! 
              $os_conn
	      startup;
	    !");

            $autorestart = "(Auto_Restart Was Attempted)";
            $autorestarted = 1;
          }
          else
          {
            print "$P | Since 'Warm start instance' was not found in the oerr output -OR- DBC Must_Be_Up:=N, Not Initiating Auto Restart\n";
            $autorestart = "(Auto_Restart Not attempted because DBC Must_Be_Up:=N)";
          }
          print "$P | -----------------------------------------------------------------\n";
        }
        else
        {
          print "$P | It appears that Oracle was brought down intentionally - Diagnostic Data: \n";
          print $last2;
        }
      }
    }
  }
  
  #if ($company eq "hp" || $company eq "agilent")
  #{
  #  ## the ora_dbshut script put in the sid followed by comments - hmmmmm
  #  $rc = system("grep '^$os' /opt/oracle/adm/config/ora_chk_instance.excl");
  #  if ($rc == 0) 
  #  { 
  #    print "$P INHIBITED: by ora_dbshut\n"; 
  #    return 2;
  #  }
  #}

  # Try to run an SQL statement
  &msg("Running SQL against v\$instance to see if DB is up");
  $sqlstmt = 'select \'X\', status from v\$instance;';

  $db_status_nocolumn = 0;
  $dbgoodflag         = 0;
  $oradebug           = ""; 

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    print if /^ORA/;
    @words = split;
    $verb  = $words[0];
    if ($verb eq "X")
    {
      $db_status = $words[1];
      $dbgoodflag = 1; 
    }
    if ($verb =~ /ORA-00904/)
    {
      $db_status_nocolumn = 1;
    }
    $oradebug .= $_; 
  }

  #---- 1.45 BB - New debugging code to solve mysterious connection lost problems
  if (! $dbgoodflag)
  {
    &msg("Something went wrong - dbgoodflag=$dbgoodflag - Oracle output:"); 
    print $oradebug; 
  }

  print "$P DBSTATUS: $db_status\n";

  if ($db_status_nocolumn == 1)
  {
    &msg("This version of Oracle does not have a v\$instance/STATUS column");
  }
  elsif ($db_status ne "OPEN")
  {
    &msg("Because the DB is not OPEN, exiting");
    &msg("END_OF_PROBE");
    exit 0;
  }

  if ($ENV{'WWW'})
  {
    &checkRollback;
  }

  # Try to run an SQL statement
  &msg("Running SQL to see if DB is up");
  $sqlstmt = 'select
    \'THIS\', banner from v\$version;
  select
    \'Y\', 
    controlfile_type, 
    dbid, 
    name 
    from v\$database;
  select 
    count(*) from dba_doesnt_exist;
  select
    \'THAT\', 
    parallel, 
    logins, 
    to_char(startup_time,\'YYYY/MM/DD-HH24:MI:SS\') startup_time,
    sysdate - startup_time
    from v\$instance;
  select
    \'Z\', inst_number, inst_name from v\$active_instances;
  select
    \'X\', SESSIONS_HIGHWATER from v\$license;
  select
    \'Q\', count(*) from DBAMON.TIMESTAMP;
  SELECT
    \'RG\',
    version,
    status,
    comp_name
    FROM dba_registry
    where version is not null
    order by
      1, 3;
  ';

  $sqlgood = 0;

  &oraSQL($sqlstmt);

  foreach (@oraSQLout)
  {
    ##print;
    push(@oraout,$_);
    @words = split;
    $verb    = $words[0];
    # Check for message which indicates the need for Oracle patch
    if ($verb eq "ORA-00020:") 
    { 
      print "$P | ***************************************************\n";
      print "$P | Oracle Max Processes Exceeded \n";
      print "$P | ***************************************************\n";
      print "$P MAXPROCESSES: $words[5] \n";
      $sqlgood = 0; 
    }
    if ($verb eq "ORA-01219:") 
    { 
      print "$P | ***************************************************\n";
      print "$P | Oracle DB is Not OPEN\n";
      print "$P | ***************************************************\n";
      $sqlgood = 0; 
    }
    if ($verb eq "SVRMGR>" && $words[1] eq "MGR-11401:") 
    { 
      print "$P | ***************************************************\n";
      print "$P | Oracle 'input error, unable to read input line'\n";
      print "$P | error encountered while executing sql. This \n";
      print "$P | error indicates the need to apply the patch for \n";
      print "$P | bug 622943 to this Oracle version (7.3.4). See: \n";
      print "$P | http://dba.it.agilent.com/dba/processes/oracle/patches.html \n";
      print "$P | ***************************************************\n";
      $sqlgood = 0; 
      $oraver  = $words[3] if $words[3] ne "error,";
    }

    if ($verb eq "RG")
    {
      shift @words;
      $rg_component = "";
      $rg_version   = shift @words;
      $rg_status    = shift @words;
      foreach (@words)
      {
        $rg_component .= "$_ ";
      }

      ##print "oraver=$oraver rg_version=$rg_version\n";
      if ($oraver && $rg_version && $rg_version lt $oraver)
      {
	if ($parallel eq "YES" && $instnum > 1)
	{
	  &msg("Since this RAC/OPS instance number is > $instnum, not performing Registry check");
	}
        else
        {
          $registry_events .= "$P REGISTRY_COMPONENT_OLD: |$rg_version|$rg_status|$rg_component|\n";
        }
      }
    }
    
    if ($verb eq "Q")
    {
      $timestamp_rows = $words[1];
    }

    if ($verb eq "Z" && $words[2] eq "$host:$os") 
    { 
      $instnum  = $words[1];
    }
    if ($verb eq "THAT") 
    { 
      $parallel  = $words[1];
      $logins    = $words[2];
      $startts   = $words[3];
      $hoursup   = $words[4] * 24;
      $hoursup   = sprintf("%.2f", $hoursup);
    }
    if ($verb eq "Y") 
    { 
      $standby  = $words[1];
      $dbid     = $words[2];
      $dbname   = $words[3];

      $standby_flag = 1 if $standby eq "STANDBY";
    }
    if ($verb eq "X") 
    { 
      $sessionshwm = $words[1];
    }
    if ($verb eq "THIS" && substr($words[1],0,6) eq "Oracle") 
    { 
      $sqlgood = 1; 
      $oraver  = $words[3];
      if ($oraver eq "Edition") { $oraver = $words[5]; }
      if ($oraver eq "Release") { $oraver = $words[4]; }
      $mode64 = 1 if /64bit/;
    }
    if ($verb eq "THIS" && /10g/) 
    { 
      $sqlgood = 1; 
      $oraver  = $words[7];
      $mode64 = 1 if /64b/;
    }
  }
  close(INDB);

  #---- Get OEM WWW Port, if any
  chop($oemport = `grep 'Enterprise Manager Console HTTP Port ($os)' $oh/install/portlist.ini 2>/dev/null | awk '{ print \$NF }'`);
  $oemport = "?" if ! $oemport;

  if (!$sqlgood)
  {
    print "svgmgrl Diagnostic Output Start: ==============================\n";
    print "@oraout";
    print "svgmgrl Diagnostic Output End: ================================\n";
  }

  $z = substr($oraver,length($oraver)-1,1);
  ##print "oraver=$oraver z=$z\n";
  if ($z eq ",")
  {
    chop($oraver);
    $oraver = $oraver . "(64)";
  }
  if ($mode64)
  {
    $oraver = $oraver . "(64)";
  }

  if ($ENV{'WWW'})
  {
    &alertlogWWW;
    &filecheck;
    &dfCheck; 

    if ($thisos eq "HP-UX")
    {
      &serverInfoHPUX;
    }

    $www_bgcolor = "pink";
    $www_bgcolor = "lime" if $sqlgood == 1 && $alertflag != 1;
   
    $www_status = ">>> Down <<<";
    $www_status = "Up" if $sqlgood == 1;

    $daysup = $hoursup / 24; 
    $daysup = sprintf("%.2f", $daysup);
    
    &msg("WWW Status Summary:");
    print "<table border=0 \n"; 
    print "<tr bgcolor=slate><td><font size=2>Item<td><font size=2>Status\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Oracle Status<td><h5>$www_status\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>DB Status<td><h5>$db_status\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Alert Log Status<td><h5>$alertmsg\n"; 
    print "<tr bgcolor=$www_bgcolor><td colspan=2>\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Hostname<td><h5>$host\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>ORACLE_SID<td><h5>$os\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>DB Name<td><h5>$dbname\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Oracle Version<td><h5>$oraver\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>CF Status<td><h5>$standby\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Logins<td><h5>$logins\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Oracle Started<td><h5>$startts\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Oracle Has Been Up<td><h5>$hoursup Hours / $daysup Days\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>FILECHECK File Last Updated<td><h5>$fc_age_hours Hours Ago (File: $fc_filename) \n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>Datafile Count<td><h5>$dfcount\n"; 
    print "<tr bgcolor=$www_bgcolor><td><font size=2>OEM WWW Port<td><h5>$oemport\n"; 

    if ($mem_gb > 0)
    {
      print "<tr bgcolor=$www_bgcolor><td colspan=2>\n"; 
      print "<tr bgcolor=$www_bgcolor><td><font size=2>Server Model:<td><h5>$servermodel\n"; 
      print "<tr bgcolor=$www_bgcolor><td><font size=2>Server CPU Count:<td><h5>$num_cpus\n"; 
      print "<tr bgcolor=$www_bgcolor><td><font size=2>Server Physical Memory:<td><h5>$mem_gb (gB)\n"; 
      print "<tr bgcolor=$www_bgcolor><td><font size=2>Server Memory Free:<td><h5>$memfree (gB)\n"; 

      if    ($mempct >= 98.5) { $mem_color = "pink"; }
      elsif ($mempct >= 95)   { $mem_color = "yellow"; }
      else                    { $mem_color = "lime"; }
      print "<tr bgcolor=$mem_color><td><font size=2>Server Memory % Used:<td><h5>${mempct}%\n"; 
    }
   
    print "</table>\n"; 

    &msg("Removing tmp files");
    system("rm $tmp1 $tmp2 2>&1");

    &msg("Done");
    &msg("END_OF_PROBE");
    exit 0;
  }

  print "$P SQLGOOD:        $sqlgood\n"; 
  print "$P ORAVER:         $oraver\n"; 
  print "$P PARALLEL:       $parallel $instnum\n"; 
  print "$P LOGINS:         $logins\n"; 
  print "$P STANDBY:        $standby\n"; 
  print "$P DBID:           $dbid\n"; 
  print "$P SESSIONSHWM:    $sessionshwm\n"; 
  print "$P ORACLE_STARTED: $startts\n"; 
  print "$P TIMESTAMP_ROWS: $timestamp_rows\n"; 
  print "$P OEMPORT:        $oemport\n"; 

  if ($registry_events)
  {
    print $registry_events;
  }

  # Uptime processing
  if ($startts ne "")
  {
    # @timesplit = split(/:/,$startts);
    # 2001/08/09-15:52:53
    @dateary=(
      substr($startts,17,2),
      substr($startts,14,2),
      substr($startts,11,2),
      substr($startts,8,2),
      (substr($startts,5,2)) - 1,
      (substr($startts,0,4)) - 1900
    );
    $startts_num = &timelocal(@dateary);
    chop($date = `date +%Y/%m/%d-%H:%M:%S`);
    $time = time;
    print "$P UPTIME: $startts $startts_num $date $time \n";
  }

  #TEST 
  #($sec,$min,$hour,$mday,$mon,$year) = localtime($startts_num);
  #print "$sec,$min,$hour,$mday,$mon,$year \n";
  #print "startts_num=$startts_num \n";

  # Check for pmon process

  $numpmon = `ps -ef | grep 'ora_pmon_$os\$' | grep -v 'grep' | wc -l`; 
  chop $numpmon;
  print "$P NUMPMON: $numpmon\n"; 

  # Check for lgwr process

  $numlgwr = `ps -ef | grep 'ora_lgwr_$os\$' | grep -v 'grep' | wc -l`; 
  chop $numlgwr;
  print "$P NUMLGWR: $numlgwr\n"; 

  # Check for smon process

  $numsmon = `ps -ef | grep 'ora_smon_$os\$' | grep -v 'grep' | wc -l`; 
  chop $numsmon;
  print "$P NUMSMON: $numsmon\n"; 

  # Check for dbw. process

  $numdbwr = `ps -ef | grep 'ora_dbw._$os\$' | grep -v 'grep' | wc -l`; 
  chop $numdbwr;
  print "$P NUMDBWR: $numdbwr\n"; 

  # Check for dbw. process

  &msg("Looking for i1|i2 ora_i I/O Slave Background Processes");
  $zcmd = "ps -ef | grep 'ora_i' | grep '$os' | grep -v 'grep' | egrep 'i1|i2'";
  system("$zcmd"); 
  chop($numioslave = `$zcmd | wc -l`); 
  print "$P NUMIOSLAVE: $numioslave\n"; 

  # Old code
  ## system("ps -ef | grep 'ora_i..._$os\$' | grep -v 'grep'"); 
  ## chop($numioslave = `ps -ef | grep 'ora_i..._$os\$' | grep -v 'grep' | wc -l`); 

  # OS Stuff

  system("uname -a");
  system("model");
  system("echo 'MODE' `getconf SC_KERNEL_BITS 2>&1`") if $thisos ne "Linux";

  if ($thisos ne "SunOS" && $thisos ne "Linux")
  {
    &serverInfoHPUX;
  }

  &msg("Running $bdfcmd to check $oh filesystem");
  # Check for ORACLE_HOME full
  $homefull = `$bdfcmd $oh | tail -1 | awk '{ print \$5 }'`;
  chop $homefull;
  chop $homefull;
  if ($homefull >= 99)
  {
    $homeflag = "C";
  }
  elsif ($homefull >= 95)
  {
    $homeflag = "W";
  }
  else
  {
    $homeflag = "-";
  }
  print "$P HOMEFULL: $homefull $homeflag\n"; 

  &msg("Determining backup type");
  # Determine backup type
  $bktype = "bkNone";

  # If there is a recent row in dbamon.backup_age, then that also 
  # counts as a backup_type 
  &bkage();

  chop($orabk1 = `crontab -l | grep -v '^#' | grep 'ora_backup ' | grep '$os' | wc -l`);
  chop($orabk2 = `cat /usr/local/dba/tools/ora_backup_sched.cfg | grep -v '^#' | grep '$host' | grep '$os' | wc -l`);
  chop($orabk3 = `cat /usr/local/dba/tools/ora_backup_sched.cfg | grep -v '^#' | grep '$host' | awk '{ print \$2 }' | grep '*' | wc -l`);

  print "$P Backup_Checks: \n"; 
  print "$P orabk1: $orabk1 \n"; 
  print "$P orabk2: $orabk2 \n"; 
  print "$P orabk3: $orabk3 \n"; 

  if ($bktype eq "bkNone" && ($orabk1 > 0 || $orabk2 > 0 || $orabk3 > 0))
  {
    $bktype = "bkRMAN";
  }

  # See if an RMAN or TBS or EXP or FULL backup is running
  chop($numrunning_rman = `ps -ef | grep 'ora_backup' | grep '$os' | grep -v 'grep' | grep 'LVL' | wc -l`);
  chop($numrunning_tbs  = `ps -ef | grep 'ora_tbs_bk' | grep '$os' | grep -v 'grep' | wc -l`);
  chop($numrunning_exp  = `ps -ef | grep 'ora_exp_bk' | grep '$os' | grep -v 'grep' | wc -l`);
  chop($numrunning_full = `ps -ef | grep 'ora_full_bk'| grep '$os' | grep -v 'grep' | wc -l`);

  if (
      $numrunning_rman > 0 ||
      $numrunning_tbs  > 0 ||
      $numrunning_exp  > 0 ||
      $numrunning_full > 0 
     )
  {
    $running = "Y";
  }
  else 
  {
    $running = "N";
  }
  print "$P BACKUP_RUNNING: $running\n";

  # This var is misspelled - i will comment it for now
  ### if ($bktype eq "bkNone" && $bcklevel ne "")
  ### {
  ###   $bcktype = "bk$bcklevel";
  ### }

  if (($company eq "hp" || $company eq "agilent") && $bktype eq "bkNone")
  {
    chop($bkscheds = `grep '$os' /opt/oracle/adm/config/*.sched | grep -v "#$os" | tail -1 | awk -F: {' print \$1 }'`);
    if ($bkscheds eq "/opt/oracle/adm/config/ora_tbs_bk.sched")
    {
      $bktype = "bkTBSold";
    }
    if ($bkscheds eq "/opt/oracle/adm/config/ora_full_bk.sched")
    {
      $bktype = "bkFULLold";
    }
    if ($bkscheds eq "/opt/oracle/adm/config/ora_exp_bk.sched")
    {
      $bktype = "bkEXPold";
    }
    chop($bknum    = `grep -l '$os' /opt/oracle/adm/config/*.sched | wc -l`);
    if ($bknum > 1)
    {
      $bktype = "bkMULTold";
    }
    chop($cronout = `crontab -l | grep -v '^#' | grep 'ora' | grep '$os'`);
    if ($cronout =~ /ora_tbs_bk/)
    {
      $bktype = "bkTBS";
    }
    if ($cronout =~ /ora_exp_bk/)
    {
      $bktype = "bkEXP";
    }
    if ($cronout =~ /ora_full_bk/)
    {
      $bktype = "bkFULL";
    }
    if ($cronout =~ /ora_backup_catexp/)
    {
      $bktype = "bkCATExp";
    }
    #if ($cronout =~ /db_export/)
    #{
    #  $bktype = "bkEXPUsr";
    #}
    if ($cronout =~ /ora_backup_catexp/)
    {
      $bktype = "bkCATExp";
    }
  }

  if ($parallel eq "YES" && $instnum > 1)
  {
    $bktype = "bkOPSN2_NA";
  }
  if ($standby eq "STANDBY" && !$bckfromfile)
  {
    $bktype = "bkSTBY_NA";
  }

  print "$P BKTYPE: $bktype\n"; 

  # Summarize for dbamon_check (WWW)
  if ($sqlgood && $numpmon && $numlgwr && $numsmon && $numdbwr)
  {
    print "<b>$dashes</b>\n";
    print "<b> Because SQL Ran OK and the PMON, LGWR, SMON and DBWR Tasks are active,\n   Oracle is UP and logins are ${logins}.</b>\n";
    print "<b>$dashes</b>\n";
    $autorestart = "(Auto_Restart Was Successful)" if $autorestarted;
  }
  else
  {
    print "<table border=3>\n";
    print "<tr><th>Oracle is DOWN.\n";
    print "</table>\n";
  }
  if ($ora_crashed)
  {
    print "$P ORACRASHED: $autorestart\n";
  }

  # List stack=oraSaveCfgStack
  @lines = split(/\n/,$initora);
  foreach (@lines) 
  {
    print "Stack: oraSaveCfgStack $_ \n";
  }

  #---- Config stack tablespaces
  print "Stack: oraSaveCfgStack # \n";
  print "Stack: oraSaveCfgStack # Report of all Tablespaces:\n";
  print "Stack: oraSaveCfgStack # \n";
  foreach (`$progdir/dbamon_orastat.pl -ts`)
  {
    print "Stack: oraSaveCfgStack $_ \n";
  }

  &filesList();

  &upParm();

  &onlineRedo();

  &processesCount();
}

# ----------------------------------------------------------------------
sub checkRollback
# ----------------------------------------------------------------------
{
  &msg("checkRollback() - Running 'select count(*) from v\$rollstat' to check DB health"); 
  $sqlstmt = ' select \'X\', count(*) from v\$rollstat; ';
  &oraSQL($sqlstmt);
  $rbcount = -1;
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb  = $words[0];
    if ($verb eq "X") 
    { 
      $rbcount = $words[1];
    }
  }
  if ($rbcount > -1)
  {
    &msg("checkRollback() - Result is $rbcount - Check successful"); 
  }
  else
  {
    &msg("<font color=red>Result is $rbcount - SOMETHING IS WRONG With This Oracle Instance</font>"); 
    system("echo ' ' | mailx -s 'DBAmon | $host/$os - rbcount=$rbcount [AUTO]' border\@dba.it.agilent.com") if $company eq "agilent";
  }
}

# ----------------------------------------------------------------------
sub bkage
# ----------------------------------------------------------------------
{
  $lvlage = 99999999;  # Age in hours of most recent LVL? backup
  $sqlstmt = "
  select 
    'X', 
    bcklevel, 
    max(tsnum),
    max(tstext) 
  from dbamon.backup_age
  --where sid = '$os' and bcklevel != 'ARC'
  where sid = '$os' and bcklevel not like 'DF%'
  group by 'X', bcklevel
  order by 3;";

  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "X") 
    { 
      $bcklevel = $words[1];
      $maxts    = $words[2];
      $maxtstxt = $words[3];
      $now      = time();
      $agehrs   = ($now - $maxts) / 3600;
      $agehrs   = sprintf("%.0f", $agehrs);
      print "$P BACKUP_AGE: $bcklevel $agehrs\n";

      #---- Save TS of most recent non-ARC backup
      if ($maxtstxt gt $mr_ts_txt && $bcklevel !~ /^ARC/)
      {
        $mr_ts_txt = $maxtstxt;
      } 

      if (substr($bcklevel,0,3) eq "LVL")
      {
        $bktype = "bkRMAN";
	if ($agehrs < $lvlage) { $lvlage = $agehrs; }
        $bklvlts = $maxtstxt; 
      }
      #new 1.28 block
      if (substr($bcklevel,0,3) eq "OFF")
      {
        $bktype = "bkRMAN";
	if ($agehrs < $lvlage) { $lvlage = $agehrs; }
        $bklvlts = $maxtstxt; 
      }
      if (substr($bcklevel,0,3) eq "EXP")
      {
        $bktype = "bkEXP";
	if ($agehrs < $lvlage) { $lvlage = $agehrs; }
      }
      if (substr($bcklevel,0,3) eq "TBS")
      {
        $bktype = "bkTBS";
	if ($agehrs < $lvlage) { $lvlage = $agehrs; }
      }
      if (substr($bcklevel,0,3) eq "BCV")
      {
        $bktype = "bkBCV";
	if ($agehrs < $lvlage) { $lvlage = $agehrs; }
      }
    }
  }
  print "$P BACKUP_AGE_MR: $lvlage\n";
  print "$P BACKUP_TYPE_MR: $bcklevel\n";
  print "$P BACKUP_DATE_MR: $mr_ts_txt\n";

  $bckage_file = "/opt/oracle/adm/log/DBAmon_Backup_Age_$os.txt";
  if (-e $bckage_file)
  {
    &msg("Reading $bckage_file");
    open(BCKAGE, "$bckage_file") || &error("Cannot open input $bckage_file - $!");
    while(<BCKAGE>)
    {
      $bckfromfile = 1;
      @words  = split();
      $ts = $words[5];
      $bcklevel = $words[1];
      $agefilehrs = int((time - $ts) / 3600); 
      print "$P BACKUP_AGE_FILE: $words[0] $words[1] $words[2] $words[3] $words[4] $words[5] AgeHrs: $agefilehrs\n";
      if ($bcklevel =~ /^LVL/ || 
          $bcklevel eq "OFFLINE" 
         )
      {
        $bktype = "bkRMAN";
      }
      ##print "bcklevel=$bcklevel bktype=$bktype\n";
    }
    close(BCKAGE);
    print "$P BACKUP_AGE_FILE_MR: $agefilehrs\n";
  }

  # See if there have been any unrecoverable changes since the last LVLx backup
  if ($bktype eq "bkRMAN"  && ! $standby_flag)
  {
    &msg("Looking for v\$datafile unrecoverable changes since the last LVLx RMAN backup");
    $sqlstmt = '
      select
        \'X\',
	name, 
        to_char(unrecoverable_time,\'YYYY/MM/DD-HH24:MM:SS\')
      from 
	v\$datafile 
      where 
	unrecoverable_time > 
	  to_date( 
            -- ( select max(tstext) from dbamon.backup_age where bcklevel like \'LVL%\'), 
               ( select max(tstext) from dbamon.backup_age where bcklevel like \'LVL%\' or bcklevel = \'OFFLINE\'), 
          \'YYYY/MM/DD-HH24:MI:SS\');
    ';
    ##print "sqlstmt=\n$sqlstmt\n";
    &oraSQL("$sqlstmt");
    $unreccnt = 0;
    foreach (@oraSQLout)
    {
      ##print;
      print if /^ORA/;
      @words  = split();
      $verb = $words[0];
      if ($verb eq "X")
      {
        $dfname = $words[1];
        $dfts   = $words[2];
        print "$P UNREC_RMAN_CHANGE: $dfname $dfts $bklvlts\n";
        $unreccnt = 1;
      }
    }
    if ($unreccnt == 0)
    {
      &msg("No unrecoverable changes since the last LVLx RMAN backup were found");
    }
    else
    {
      &msg("$unreccnt unrecoverable changes since the last LVLx RMAN backup were found");
    }
    ### TEST
    ## print "$P UNREC_RMAN_CHANGE: mytestdfname 2009-1-3 $bklvlts\n";
  }
}

# ----------------------------------------------------------------------
sub filecheck
# ----------------------------------------------------------------------
{
  chop($fc_filename = `grep '^FILECHECK' $cfg | awk '{ print \$2 }'`);
  if ($fc_filename eq "")
  {
    print "$P FILECHECK: Parm not found in $cfg\n";
    return;
  }
  print "$P FILECHECK: fc_filename=$fc_filename\n";
  if (! -e $fc_filename)
  { 
    print "$P FILECHECK: NotFound fc_age_hours: . $fc_filename\n";
    return;
  }
  $fc_mtime = (stat "$fc_filename") [9];
  $fc_age = time - $fc_mtime;
  $fc_age_hours = $fc_age / 3600;
  $fc_age_hours = sprintf("%.2f", $fc_age_hours);
  #print "$P FILECHECK: file Found fc_age_hours: $fc_age_hours $fc_filename\n";
  print "$P FILECHECK: Found fc_age_hours: $fc_age_hours $fc_filename\n";
}

# ----------------------------------------------------------------------
sub bkrun
# ----------------------------------------------------------------------
{
  $cmd = $arg1;
  print "$P bkrun: Issuing at now command to run backup:\n";
  print "$P bkrun: Command before {SID} substitution: $cmd\n";
  $cmd =~ s/\{SID\}/$os/g;
  $cmd = "echo 'time $cmd' | at now 2>&1";
  print "$P bkrun: About to issue: $cmd\n";
  $rc = system($cmd);
  print "$P bkrun: Back from issuing at now - rc=$rc\n";
  print "<table bgcolor=skyblue>
  <tr><td><pre>

  Backup Was Invoked - cmd=$cmd
  </table>";
}

# --------------------------------------------------------------------
sub inSyncCheckInhibit 
# --------------------------------------------------------------------
{
  $tmpout = "$tmpdir/dbamon_ora_checkinhib_${$}.txt";

  #---- Check to see if DBAmon standby refresh inhibit is
  #----   turned on now (this is used for outages)
  &msg("Running ora_check_inhibit to see if sby refresh is inhibited");

  $rc = system("/usr/local/dba/tools/ora_check_inhibit_sby > $tmpout 2>&1");
  $rc /= 256;

  &msg("Back from ora_check_inhibit_sby - rc=$rc - Output:");
  system("cat $tmpout");

  if ($rc == 2)
  {
    &msg("=========================================================================================");
    &msg("This check is being cancelled because sby refresh is now inhibited via DBAmon.");
    &msg("=========================================================================================");
    system("cat $tmpout | mailx -s '$P | ora_check_inhibit_sby rc=$rc $host/$os - [AUTO]' border\@dba.it.agilent.com");
    exit 0;
  }
}

# --------------------------------------------------------------------
sub inSyncSby
# --------------------------------------------------------------------
{
  $is_prihost   = $arg1;
  $is_prisid    = $arg2;
  &msg("inSyncSby(): is_prihost=$is_prihost is_prisid=$is_prisid");

  if ($company eq "agilent")
  {
    if (system("crontab -l | grep 'ora_standby_refresh_drp' >/dev/null 2>&1") == 0)
    {
      &msg("Invoking inSyncCheckInhibit"); 
      &inSyncCheckInhibit;
    }
  }

  &msg("inSyncSby(): Obtaining local Standby measurement");
  $sqlstmt = 'select \'X\', sysdate-CONTROLFILE_TIME from v\$database;';
  &oraSQL("$sqlstmt");
  foreach (@oraSQLout)
  {
    ##print;
    print if /^ORA/;
    @words  = split();
    $verb = $words[0];
    if ($verb eq "X")
    {
      $minago = $words[1] * 24 * 60;
      $minago = '.' if $minago eq "";
      printf "INSYNC: SBY %10.2f\n", $minago;
    }
  }

  # Get info from primary
  &msg("inSyncSby(): Obtaining remote Primary measurement");
  @rshout = `$comm_method $is_prihost 'export ORACLE_SID=$is_prisid; $progdir/dbamon_ora.pl inSyncPri $host $os'`;
  foreach (@rshout)
  {
    ##print;
    @words  = split();
    $verb = $words[0];
    ##print "verb=$verb w1=$words[1]\n";
    if ($verb eq "INSYNC:" && $words[1] eq "PRI")
    {
      $is_pritime = $words[2]; 
    }

    print;

    #if ($verb eq "INSYNC:" && $words[1] eq "NOLOG")
    #{
    #  print;
    #}
    #if ($verb eq "INSYNC:" && $words[1] eq "PRIUP")
    #{
    #  print;
    #}
    #if ($verb eq "INSYNC:" && $words[1] eq "PRIUNREC")
    #{
    #  print;
    #}
  }
  printf "INSYNC: PRI %10.2f\n", $is_pritime;
  &msg("inSyncSby(): Calculating delta");
  $delta = $minago - $is_pritime;
  printf "INSYNC: DELTA %10.2f\n", $delta;

}

# --------------------------------------------------------------------
sub inSyncPri
# --------------------------------------------------------------------
{
  $is_sbyhost   = $arg1;
  $is_sbysid    = $arg2;
  &msg("inSyncPri(): is_sbyhost=$is_sbyhost is_sbysid=$is_sbysid");

  $priup = "N";
 
  # Get controlfile_time age
  $sqlstmt = 'select \'X\', sysdate-CONTROLFILE_TIME from v\$database;';
  &oraSQL("$sqlstmt");
  foreach (@oraSQLout)
  {
    ##print;
    print if /^ORA/;
    @words  = split();
    $verb = $words[0];
    if ($verb eq "X")
    {
      $priup = "Y";
      $minago = $words[1] * 24 * 60;
      $minago = '.' if $minago eq "";
      printf "INSYNC: PRI %10.2f\n",
        $minago;
    }
  }

  # Get list of nologging objects
  $sqlstmt = '
    select
      \'X\',
      \'TABLE\',
      owner,
      table_name
    from
      dba_tables
    where
      logging = \'NO\'
      and table_name != \'DEF\$_TEMP\$LOB\'
      and table_name != \'ATEMPTAB\$\'
      and tablespace_name is not null
      and temporary = \'N\'
      ;
    select
      \'X\',
      \'INDEX\',
      owner,
      index_name
    from
      dba_indexes
    where
      logging = \'NO\'
      and tablespace_name is not null
      and temporary = \'N\'
      ;
  ';

  &oraSQL("$sqlstmt");
  foreach (@oraSQLout)
  {
    ##print;
    print if /^ORA/;
    @words  = split();
    $verb = $words[0];
    if ($verb eq "X")
    {
      $priup = "Y";
      $obj = "$words[1]=$words[2].$words[3]";
      print "INSYNC: NOLOG $obj\n";
    }
  }

  print "INSYNC: PRIUP $priup\n";

  # Check the dbamon.standby_rebuild table

  &msg("inSyncSby(): Selecting dbamon.standby_rebuild");
  $sqlstmt = "select 
    \'X\', 
      to_char(ts,\'YYYY/MM/DD-HH24:MM:SS\'),
      prisid,
      prihost,
      sbysid,
      sbyhost,
      tool
    from 
      dbamon.standby_rebuild
    where 
      sbysid      = '$is_sbysid'
      and sbyhost = '$is_sbyhost'
    order by 2;";

  &oraSQL("$sqlstmt");

  $tbl_exists = 0;
  $tbl_rows  = 0;

  foreach (@oraSQLout)
  {
    ##print;
    print if /^ORA/;
    @words  = split();
    $verb = $words[0];
    if ($verb eq "X")
    {
      if ($tbl_exists == 0)
      {
        $tbl_exists = 1;
        print "\n";
        print "             Rebuild history for this standby database\n";
        print "\n";
        print "Rebuild Timestamp   Pri SID  Pri Host Sby SID  Sby Host Tool\n";
        print "------------------- -------- -------- -------- -------- ----------------\n";
      }
      $tbl_rows++; 
      $ts      = $words[1];
      $prisid  = $words[2];
      $prihost = $words[3];
      $sbysid  = $words[4];
      $sbyhost = $words[5];
      $sbtool  = $words[6];
      printf "%-19s %-8s %-8s %-8s %-8s %-16s\n", $ts,  $prisid, $prihost, $sbysid, $sbyhost, $sbtool;
      $lastts = $ts;
    }
  }
  
  print "\n";
  &msg("DBAMON.STANDBY_REBUILD contains $tbl_rows row(s) - most recent rebuild: $lastts");

  if ($tbl_exists)
  {
    &msg("Seeing if there are any vdatafile unrecoverable_timestamps greater than the last rebuild of this stby");

    $sqlstmt = "select 
      \'X\', 
      df.name,
      to_char(df.unrecoverable_time,\'YYYY/MM/DD-HH24:MM:SS\')
    from
      v\\\$datafile df
    where
      df.unrecoverable_time > 
        (select 
           max(ts) 
         from 
           dbamon.standby_rebuild          
         where 
           sbysid      = '$is_sbysid'
           and sbyhost = '$is_sbyhost');";

    &oraSQL("$sqlstmt");

    $tbl_exists = 0;
    $tbl_rows = 0;
    $unrec_changes = 0;

    foreach (@oraSQLout)
    {
      ##print;
      print if /^ORA/;
      @words  = split();
      $verb = $words[0];
      if ($verb eq "X")
      {
	if ($tbl_exists == 0)
	{
	  $tbl_exists = 1;
	  print "\n";
	  print "  Datafiles that have unrecoverable changes that have occured after the \n";
	  print "            the most recent standby rebuild which occured\n";
	  print "                        at $lastts\n";
	  print "\n";
	  print "Datafile                                           Last Unrec. Change\n";
	  print "-------------------------------------------------- ------------------\n";
          $unrec_changes = 1;
	}
	$tbl_rows++; 
	$df      = $words[1];
	$dftime  = $words[2];
	printf "%-50s %-20s \n", $df,  $dftime;
      }
    }
    print "\n";
    if ($unrec_changes)
    {
      print "INSYNC: PRIUNREC $tbl_rows\n\n";
    }
  }
}


# =========================================================
sub rmanHung
# =========================================================
{
  # Look for OLD rman processes
  $rman_days_threshold = 0.75;

  # Force UX95 mode for ps command
  $ENV{"UNIX95"} = 1;

  &msg("Looking for hung 'rman ' user=$userid processes");
  open(PS, "ps -ef -oetime,pid,tty,comm,ruser | grep 'rman ' | egrep -v 'sh|grep' | grep '$userid' |") || die "can't run ps - $!";

  $rman_hungcnt = 0;

  while(<PS>)
  {
    ##print;
    @words = split;
    $etime = $words[0];
    $pid   = $words[1];
    $tty   = $words[2];
    $comm  = $words[3];

    @t = split(":-",$etime);
    $secs = -1;
    if (length($etime) == 5)  { $secs = ($t[0] * 60) + $t[1]; }
    if (length($etime) == 8)  { $secs = ($t[0] * 3600) + ($t[1] * 60) + $t[2]; }
    if (length($etime) == 10) { $secs = ($t[0] * 86400) + ($t[1] * 3600) + ($t[2] * 60) + $t[3]; }
    if (length($etime) == 11) { $secs = ($t[0] * 86400) + ($t[1] * 3600) + ($t[2] * 60) + $t[3]; }
    $days = sprintf("%.2f", int($secs / (24*60*60)));

    print "pid=$pid comm=$comm ty=$tty secs=$secs days=$days\n";

    if ($days > $rman_days_threshold)
    {
      print "\n";
      print "$P RMANHUNG: Found rman process (pid=$pid) that has been running for $days days (threshold=$rman_days_threshold days)\n";
      system("ps -ef | grep '$pid'");
      $rman_hungcnt++;
    }
  }
  close(PS);

  &msg("Found $rman_hungcnt possible hung RMAN process(es)");
}

# =========================================================
sub fs
# =========================================================
{
  $fsmask = $arg3;

  &msg("Looking for filesystems that match $fsmask");

  open(DF, "$bdfcmd | grep '$fsmask' |") || die "can't run ps - $!";

  $fs_cnt = 0;
  $max_fsfree_gb = 0;

  while(<DF>)
  {
    print;
    @words = split;

    if ($words[0] =~ /^\//)
    {
      shift @words;
    }
    $fssize = $words[0];
    $fsused = $words[1];
    $fsfree = $words[2];
    #$fspct  = $words[3];
    #$fsname = $words[4];

    $fssize_gb = $fssize / 1000000;
    $fsused_gb = $fsused / 1000000;
    $fsfree_gb = $fsfree / 1000000;

    $tot_fssize_gb += $fssize_gb;
    $tot_fsused_gb += $fsused_gb;
    $tot_fsfree_gb += $fsfree_gb;

    $max_fsfree_gb = $fsfree_gb if $fsfree_gb > $max_fsfree_gb;
    $fs_cnt++;
  }
  close(DF);

  &msg("Found $fs_cnt filesystems");
 
  $pct_fsused = 100 * ($tot_fsused_gb / $tot_fssize_gb) if $tot_fssize_gb > 0;

  $tot_fssize_gb = sprintf("%.2f", $tot_fssize_gb);
  $tot_fsused_gb = sprintf("%.2f", $tot_fsused_gb);
  $tot_fsfree_gb = sprintf("%.2f", $tot_fsfree_gb);
  $max_fsfree_gb = sprintf("%.2f", $max_fsfree_gb);
  $pct_fsused = sprintf("%.2f", $pct_fsused);

  print "$P TOT_FSSIZE_GB:  $tot_fssize_gb\n";
  print "$P TOT_FSUSED_GB:  $tot_fsused_gb\n";
  print "$P TOT_FSFREE_GB:  $tot_fsfree_gb\n";
  print "$P MAX_FSFREE_GB:  $max_fsfree_gb\n";
  print "$P PCT_FSUSED:     $pct_fsused\n";
}

# =========================================================
sub fsGeneric
# =========================================================
{
  $fsmask = "/var/opt/oracle/disk";
  &msg("Looking for filesystems that match $fsmask");

  open(DF, "$bdfcmd | grep '$fsmask' |") || die "can't run ps - $!";

  $fs_cnt = 0;
  $max_fsfree_gb = 0;

  while(<DF>)
  {
    print;
    @words = split;

    if ($words[0] =~ /^\//)
    {
      shift @words;
    }
    $fssize = $words[0];
    $fsused = $words[1];
    $fsfree = $words[2];
    #$fspct  = $words[3];
    #$fsname = $words[4];

    $fssize_gb = $fssize / 1000000;
    $fsused_gb = $fsused / 1000000;
    $fsfree_gb = $fsfree / 1000000;

    $tot_fssize_gb += $fssize_gb;
    $tot_fsused_gb += $fsused_gb;
    $tot_fsfree_gb += $fsfree_gb;

    $max_fsfree_gb = $fsfree_gb if $fsfree_gb > $max_fsfree_gb;
    $fs_cnt++;
  }
  close(DF);

  &msg("Found $fs_cnt filesystems");
 
  $pct_fsused = 100 * ($tot_fsused_gb / $tot_fssize_gb) if $tot_fssize_gb > 0;

  $tot_fssize_gb = sprintf("%.2f", $tot_fssize_gb);
  $tot_fsused_gb = sprintf("%.2f", $tot_fsused_gb);
  $tot_fsfree_gb = sprintf("%.2f", $tot_fsfree_gb);
  $max_fsfree_gb = sprintf("%.2f", $max_fsfree_gb);
  $pct_fsused = sprintf("%.2f", $pct_fsused);

  print "$P TOT_FSSIZE_GB:  $tot_fssize_gb\n";
  print "$P TOT_FSUSED_GB:  $tot_fsused_gb\n";
  print "$P TOT_FSFREE_GB:  $tot_fsfree_gb\n";
  print "$P MAX_FSFREE_GB:  $max_fsfree_gb\n";
  print "$P PCT_FSUSED:     $pct_fsused\n";
}


# ----------------------------------------------------------------------
sub bkbl
# ----------------------------------------------------------------------
{
  &bkTypeCheck;

  $bktype = $arg1;
  $what   = $arg2;

  if ($bk_type eq "ora_backup")
  {
    $logpath = "/opt/oracle/adm/cronlog/ora_backup_${os}_${bktype}.log";
  }
  else
  {
    ##$z = "/opt/app/oracle/esoadmin/log/${os}_rman2ob_*.log";
    $z = "/opt/oracle/adm/cronlog/rman2ob_${os}*.log";
    chop($logpath = `ls -l $z | tail -1 | awk '{ print \$NF }'`);
  }

  &msg("bkbl() - what=$what bktype=$bktype logpath=$logpath");

  system("ls -l $logpath 2>&1");
  if (! -e $logpath)
  {
    &error("Log file $logpath does not exist");
  }

  chop($lines = `wc -l $logpath | awk '{ print \$1 }'`);
  &msg("File $logpath contains $lines lines");

  if    ($what eq "view") { $cmd = "cat"; }
  elsif ($what eq "tail") { $cmd = "tail -50 "; }
  &msg("Running $cmd on $logpath");

  print "<table bgcolor=skyblue>
  <tr><td><pre>
  <b>Start of ora_backup Log: $logpath</b><hr>";

  system("$cmd $logpath 2>&1");

  print "<hr>
  <b>End of ora_backup Log: $logpath</b></table>\n"; 
}

# ----------------------------------------------------------------------
sub bkrb
# ----------------------------------------------------------------------
{
  &bkTypeCheck;

  $bk_torun = $arg1;
  &msg("Invoking an $bk_torun $bk_type backup");

  if ($bk_type eq "ora_backup")
  {
    $bk_cmd = "/usr/local/dba/tools/ora_backup $os $bk_torun >> /opt/oracle/adm/cronlog/ora_backup_${os}_${bk_torun}.log 2>&1";
  }
  else  # Must be rman2ob
  {
    if ($bk_torun eq "ARCEMERGENCY")
    {
      $bk_cmd = "/usr/local/dba/tools/rman2ob.pl -D $os -d 0 >> $tmpdir/dbamon_ora_bkrb.log 2>&1";
    }
    elsif ($bk_torun eq "ARC")
    {
      $bk_cmd = "/usr/local/dba/tools/rman2ob.pl -D $os -a   >> $tmpdir/dbamon_ora_bkrb.log 2>&1";
    }
    elsif ($bk_torun eq "LVL0")
    {
      $bk_cmd = "/usr/local/dba/tools/rman2ob.pl -D $os -O 0 >> $tmpdir/dbamon_ora_bkrb.log 2>&1"; 
    }
    elsif ($bk_torun eq "LVL1")
    {
      $bk_cmd = "/usr/local/dba/tools/rman2ob.pl -D $os -O 1 >> $tmpdir/dbamon_ora_bkrb.log 2>&1"; 
    }
    else
    {
      &error("Backup type of $bk_torun is not valid for rman2ob");
    }
  }

  &msg("Setting NOTIX environment variable to 1");
  $ENV{"NOTIX"} = "1";
  system("env | grep 'NOTIX'"); 

  $cmd = "echo 'time $bk_cmd' | at now";
  $rc = system($cmd);

  print "<table bgcolor=skyblue>
  <tr><td><pre>

  Backup Was Invoked - cmd=$cmd
  </table>";
  &msg("Back from issuing at now - rc=$rc");
}

# ----------------------------------------------------------------------
sub bkhi
# ----------------------------------------------------------------------
{
  &bkTypeCheck;

  print "<table bgcolor=skyblue border=1>
  <tr bgcolor=slate>
    <th><font size=2>Backup Type
    <th><font size=2>Timestamp 
    <th><font size=2>Backup<br>Duration<br>(Minutes)  
    <th><font size=2>Backup Age<br>(Hours)
    <th><font size=2>DB Size<br>(MB Used)
  ";

  $now = time;
  $rowcount = 0;
  $sqlstmt = 'select \'Y\', SID, BCKLEVEL, TSNUM, TSTEXT, DBSIZE, ELAPSED
    from dbamon.backup_age
    order by 5 desc;';
  &oraSQL($sqlstmt);
  foreach (@oraSQLout)
  {
    ##print;
    print if /^ORA/;
    @words = split;
    $verb    = $words[0];
    if ($verb eq "Y")
    {
      #$sid     = $words[1];
      $level   = $words[2];
      $tsnum   = $words[3];
      $tstext  = $words[4];
      $dbsize  = $words[5];
      $elapsed = int($words[6]/60);
      $age     = ($now - $tsnum) / (60*60);

      $age = sprintf("%.2f",$age);

      print "<tr>
        <td><font size=2>$level
        <td><font size=2>$tstext
        <td align=right><font size=2>$elapsed
        <td align=right><font size=2>$age
        <td align=right><font size=2>$dbsize
      "; 
     
      $rowcount++;
    }
  }
  print "</table>";
  &msg("$rowcount Backup rows found");
}

# ----------------------------------------------------------------------
sub bkTypeCheck
# ----------------------------------------------------------------------
{
  &msg("Determining backup type");
  if (-e "/opt/app/oracle/esoadmin/log/${os}_rman2ob_1.log")
  {
    $bk_type = "rman2ob";
  }
  elsif (-e "/opt/oracle/adm/cronlog/${os}_rman2ob_1.log")
  {
    $bk_type = "rman2ob";
  }
  elsif (-e "/opt/oracle/adm/cronlog/ora_backup_${os}_LVL0.log")
  {
    $bk_type = "ora_backup";
  }
  elsif (-e "/usr/openv/netbackup/logs/user_ops/dbext/logs/")
  {
    $bk_type = "ora_backup";
  }
  else
  {
    system("echo ' ' | mailx -s 'DBAmon | $host/$os - Cannot determine bk_type ' border\@dba.it.agilent.com") if $company eq "agilent";
    &error("Cannot determine backup type - ora_backup and rman2ob.pl residue not found - BB was contacted");
  }

  ##if ($bk_type eq "rman2ob")
  ##{
  ##  $bk_dir = "/opt/app/oracle/esoadmin/log";
  ##}
  ##else
  ##{
  ##  $bk_dir = "/opt/oracle/adm/cronlog";
  ##}

  $bk_dir = "/opt/oracle/adm/cronlog";
   
  &msg("Derived bk_type=$bk_type bk_dir=$bk_dir");
}

# ----------------------------------------------------------------------
sub bkps
# ----------------------------------------------------------------------
{
  &bkTypeCheck;

  $cmd = "ps -ef | grep '$bk_type' | grep -v 'grep'";
  &msg("Running $cmd");
  print "<table bgcolor=skyblue>
  <tr><td><pre>
  <b>Start of $bk_type Running Process List</b><hr>";
  system("$cmd 2>&1");
  print "<hr>
  <b>End of $bk_type Running Process List</b><hr></table>";

  &msg("bkps() Listing $bk_type Logs In /opt/oracle/adm/cronlog");
  print "<table bgcolor=skyblue>
  <tr><td><pre>
  <b>Start - $bk_type Log Files</b><hr>";
  system("ls -l $bk_dir  2>&1 | grep '$bk_type'");
  print "<hr>
  <b>End - $bk_type Log Files</b><hr></table>";
}

# ----------------------------------------------------------------------
sub patches
# ----------------------------------------------------------------------
{
  $patch_dir  = "$oh/patch";
  $patch_file = "$patch_dir/applied.txt";

  &msg("patches() - Listing patches that have been applied (patch_dir=$patch_dir patch_file=$patch_file)");

  if (! -e $patch_dir)
  {
    &msg("Directory $patch_dir Does not exist - creating");
    $rc = system("mkdir $patch_dir");
    if ($rc != 0)
    {
      system("echo '$!' | mailx -s 'DBAmon | $host/$os - patches() Error mkdir $patch_dir' border\@dba.it.agilent.com") if $company eq "agilent";
      &error("Error mkdir $patch_dir - $!");
    }
    system("echo '' | mailx -s 'DBAmon | $host/$os - patches() patch_dir=$patch_dir created' border\@dba.it.agilent.com") if $company eq "agilent";
    $rc = system("chmod 775 $patch_dir");
  }
  
  if (! -e $patch_file || system("grep 'Version_' $patch_file") != 0)
  {
    &msg("File $patch_file Does not exist - creating");
    system("echo '' | mailx -s 'DBAmon | $host/$os - patches() patch_file=$patch_file created' border\@dba.it.agilent.com") if $company eq "agilent";
    $rc = open(PFILE, ">$patch_file");
    if (! defined $rc)
    {
      system("echo '$!' | mailx -s 'DBAmon | $host/$os - patches() Error open $patch_file' border\@dba.it.agilent.com") if $company eq "agilent";
      &error("Error open $patch_file - $!");
    }
    print PFILE "
#-------------------------------------------------------------------------------------
#
# DBAmon Patch Tracking
#
# Version_1.02
#-------------------------------------------------------------------------------------
#  You can enter data into this file in order to inform DBAmon
#  that Oracle patches have been applied to this ORACLE_HOME.
#-------------------------------------------------------------------------------------
# Example Entry:
#
# NOTE: It is VERY IMPORTANT that the column alignment for all entries be the
#   same as this example.
#
#Patch Num.  DBMS Version  YYYY/MM/DD Your Name                 Comments 
#----------> <-----------> <--------> <-----------------------> <---------------------------------->
#223344555   9.2.0.2.1     2004/07/21 John Doe                  This is a test comment
#-------------------------------------------------------------------------------------
";
    close(PFILE) || &msg("Error closing PFILE - $!");
    $rc = system("chmod 775 $patch_file");
  }

  #---- Read applied.txt file
  open(PFILE, "$patch_file") || &error("Cannot open input $patch_file - $!");
  while(<PFILE>)
  {
    next if /^$/;
    next if /^#/;
    ##print;

    $pa_num   = substr($_,0,12);
    $pa_ver   = substr($_,13,10);
    $pa_date  = substr($_,27,10);
    $pa_name  = substr($_,38,25);
    $pa_desc  = substr($_,64);

    #---- Data cleanup
    chomp $pa_desc;
  
    $pa_num  =~ s/\'/\"/g;   #---- single quotes are bad for SQL insert statements
    $pa_ver  =~ s/\'/\"/g;
    $pa_date =~ s/\'/\"/g;
    $pa_name =~ s/\'/\"/g;
    $pa_desc =~ s/\'/\"/g;

    $pa_num  =~ s/\n//g;   #---- get rid of newlines
    $pa_ver  =~ s/\n//g;
    $pa_date =~ s/\n//g;
    $pa_name =~ s/\n//g;
    $pa_desc =~ s/\n//g;

    #---- Display clean rows
    if ($pa_num && $pa_ver && $pa_date && $pa_name)
    {
      print "$P PATCH: |$pa_num|$pa_ver|$pa_date|$pa_name|$pa_desc|\n";
    }
  }
  close(PFILE);
  
  &msg("patches() - Done");
}

# ----------------------------------------------------------------------
sub orphanDFs
# ----------------------------------------------------------------------
{
  &msg("orphanDFs() - Listing datafiles to obtain dfmask");

  if (&DBOpenCheck ne "OPEN") { &error("DB must have Dropped-OPEN - terminating"); }

  chop($systemdffirst = `$progdir/dbamon_orastat.pl -df | grep '^SYSTEM' | head -1 | awk '{ print \$3 }'`);
  $dfmask = $systemdffirst; 

  if (!$systemdffirst)
  {
    &msg("Cannot look for orphan datafiles - systemdffirst is null");
    return
  }

  # Save a list of all DB files
  $all_db_files = `$progdir/dbamon_orastat.pl -de`;

  while (length($dfmask) > 0)
  {
    if ($dfmask =~ /\/$/) { last; }
    chop $dfmask;
  }

  $dfmask =~ s/[0-9]/\*/g;
  $dfmask .= "*";
  &msg("Looking for orphan datafiles - dfmask=$dfmask");

  $num_orphans = 0;
  $size_orphans = 0;

  open(FN,"find $dfmask -type f -mtime +14 | grep 'dbf\$' |" || &error("Cannot run find"));
  while(<FN>)
  {
    chop($df_orphan = $_);

    next if $df_orphan !~ /dbf$/;
    next if $df_orphan =~ /\/arch/;

    #---- Version 1.06 don't report TEMP orphan datafiles for standby DB's
    if ($standby && $df_orphan =~ /\/temp/i) 
    {
      next;
    }

    # Make sure that it belongs to this db (sid-subset)
    next if $df_orphan !~ /\/$os\//;

    $z = index($all_db_files,$df_orphan);
    next if $z > -1;  # Skip if found in list

    $df_mtime = (stat "$df_orphan") [9];
    $df_age_days = int((time - $df_mtime) / (60*60*24));
    $df_size  = (stat "$df_orphan") [7];
    $df_size_gb = $df_size / 1000000000;
    $df_size_gb = sprintf("%.2f", $df_size_gb);

    print "$P ORPHANDF: $df_orphan $df_age_days $df_size_gb\n";

    $num_orphans++;
    $size_orphans += $df_size_gb;
  }

  &msg("Found $num_orphans orphaned datafiles - $size_orphans gB");
  return;
}

# ----------------------------------------------------------------------
sub filePermNum
# ----------------------------------------------------------------------
{
  my $fpm_string = $_[0];
  if    (substr($fpm_string,0,3) eq "rwx") { $cfp_perm_num .= "7"; }
  elsif (substr($fpm_string,0,3) eq "rw-") { $cfp_perm_num .= "6"; }
  elsif (substr($fpm_string,0,3) eq "r-x") { $cfp_perm_num .= "5"; }
  elsif (substr($fpm_string,0,3) eq "r--") { $cfp_perm_num .= "4"; }
  elsif (substr($fpm_string,0,3) eq "-wx") { $cfp_perm_num .= "3"; }
  elsif (substr($fpm_string,0,3) eq "-w-") { $cfp_perm_num .= "2"; }
  elsif (substr($fpm_string,0,3) eq "--x") { $cfp_perm_num .= "1"; }
  elsif (substr($fpm_string,0,3) eq "---") { $cfp_perm_num .= "0"; }
  return $fpm_string;
}

# ----------------------------------------------------------------------
sub checkFilePermission 
# ----------------------------------------------------------------------
{
  $cfp_file = $_[0];

  if (! -e $cfp_file)
  {
    &msg("  File $cfp_file does not exist"); 
    return;
  }
  if (-l $cfp_file)
  {
    &msg("  File $cfp_file is a link - skipping"); 
    return;
  }

  chop($cfp_permission = `ls -l $cfp_file | cut -b 2-11`) if -e $cfp_file;
  return if ! $cfp_permission;
  #---- translate to numeric
  $cfp_perm_num = ""; 
  &filePermNum(substr($cfp_permission,0,3));
  &filePermNum(substr($cfp_permission,3,3));
  &filePermNum(substr($cfp_permission,6,3));

  &msg("  File permission of $cfp_file is $cfp_perm_num ($cfp_permission)"); 

  if (substr($cfp_permission,7,1) eq "w")
  {
    print "$P ORAFILEPERM: Event: $cfp_file is world-writable ($cfp_perm_num)\n" if ! $sec_fix; 
  }

  return $cfp_perm_num;
}

# ----------------------------------------------------------------------
sub securityMain 
# ----------------------------------------------------------------------
{
  #if ($host eq "zzdocevil")
  if (1 == 1)
  {
    $sec_fix = 1; 
    &msg("Running securityChecks() with sec_fix=$sec_fix");
    &securityChecks;
  }

  $sec_fix = 0; 
  &msg("Running securityChecks() with sec_fix=$sec_fix");
  &securityChecks;
}

# ----------------------------------------------------------------------
sub securityChecks 
# ----------------------------------------------------------------------
{
  &msg("Checking permission of tnsnames/listener files");

  $sec_fp = &checkFilePermission("/etc/listener.ora"); 
  if ($sec_fp && 
      $sec_fp ne "700" && 
      $sec_fp ne "600" && 
      $sec_fp ne "640")
  {
    if ($sec_fix)
    {
      chmod(0600, "/etc/listener.ora");
    }
    else
    {
      print "$P ORAFILEPERM: Event: $cfp_file has file permission of $sec_fp/$cfp_permission - Should be 700/600/640\n"; 
    }
  }

  $sec_fp = &checkFilePermission("$oh/network/admin/listener.ora"); 
  if ($sec_fp && 
      $sec_fp ne "700" && 
      $sec_fp ne "600" && 
      $sec_fp ne "640")
  {
    if ($sec_fix)
    {
      chmod(0600, "$oh/network/admin/listener.ora");
    }
    else
    {
      print "$P ORAFILEPERM: Event: $cfp_file has file permission of $sec_fp/$cfp_permission - Should be 700/600/640\n"; 
    }
  }

  if (! $sec_fix) 
  {
    # GROUP 
    $my_gid   = (split(' ',$()) [0];
    $my_group = (getgrgid($my_gid)) [0];
    $my_userid = $userid;
    &msg("Checking group=dba - my_gid=$my_gid my_group=$my_group my_userid=$my_userid");

    #---- Look for members specified in /etc/group
    &msg("Looking for $my_group members specifed in /etc/group other than ora* and dba"); 
    $dba_members = (getgrgid($my_gid)) [3];
    $dba_members =~ s/,/ /g;
    &msg("DEBUG dba_members=$dba_members");
    foreach (split(/ /,$dba_members))
    {
      next if $_ =~ /^ora/;
      next if $_ eq "dba";
      print "$P DBAGROUP_GROUP: $_\n";
    }

    #---- Look for members specified in /etc/passwd
    &msg("Looking for gid=$my_gid members specifed in /etc/passwd other than ora* and dba"); 
    while (($name, $crud, $crud, $gid) = getpwent)
    {
      next if $gid != $my_gid;
      next if $name =~ /^ora/;
      next if $name eq "dba";
      print "$P DBAGROUP_PASSWD: $name\n";
    }
  }

}

# ----------------------------------------------------------------------
sub oddJob1 
# ----------------------------------------------------------------------
{
  $thiskey = $_[0];
  $sshdir  = "$ENV{'HOME'}/.ssh";
  $keyfile = "$sshdir/authorized_keys";
  &msg("oddJob1() Starting - thiskey=$thiskey sshdir=$sshdir keyfile=$keyfile");

  #---- Set correct permissions
  system("chmod 755 $sshdir 2>&1"); 
  system("chmod 644 $keyfile 2>&1"); 
 
  #---- See if sshdir exists - if not, create it 
  &msg("  Seeing if $sshdir exists");
  if (! -e "$sshdir")
  {
    &msg("  $sshdir Does not exist - Creating");
    $rc = system("mkdir $sshdir 2>&1"); 
    if ($rc != 0)
    {
      system("echo ' ' | mailx -s 'DBAmon | $host/$os - mkdir $sshdir no workie - $!' border\@dba.it.agilent.com");
    }
    system("echo ' ' | mailx -s 'DBAmon | $host/$os - Created $sshdir' border\@dba.it.agilent.com");
  }

  #---- See if authkeys file contains this entry - if not, add it 
  &msg("  Looking in $keyfile for $thiskey DSA entry");
  if (system("grep 'ssh-dss' $keyfile | grep '$thiskey' >/dev/null") != 0)
  {
    &msg("  $keyfile Does not contain a DSA entry for $thiskey - Adding entry");
    $rc = system("grep '$thiskey' $progdir/dbamon_ora.dsa1 >> $keyfile 2>&1"); 
    if ($rc != 0)
    {
      system("echo ' ' | mailx -s 'DBAmon | $host/$os - Add $thiskey no workie - $!' border\@dba.it.agilent.com");
    }
    system("echo ' ' | mailx -s 'DBAmon | $host/$os - Added DSA $thiskey to $keyfile' border\@dba.it.agilent.com");
  }
  
  &msg("oddJob1() Done");
}

# ----------------------------------------------------------------------
sub serverInfoHPUX 
# ----------------------------------------------------------------------
{
  if ($thisos ne "HP-UX") 
  { 
    &msg("This routine only works for thisos=HP-UX"); 
    return;
  }
  &msg("serverInfoHPUX - Collecting server info (5 second delay)");

  chop($servermodel = `model`); 

  &msg("serverInfoHPUX - Running cpuinfo") if !$WWW; 
  #system("$progdir/dbamon_cpuinfo 2>&1");
  open(CPUINFO, "$progdir/dbamon_cpuinfo 2>&1|") || &error("Cannot open input $tmp1 - $!");
  while(<CPUINFO>)
  {
    if (/^Phy/)
    {
      $phy_ram_mb = (split) [1];
    }
    if (/^Num/)
    {
      $num_cpus   = (split) [1];
    }
  }
  close(CPUINFO);
  if ($phy_ram_mb)
  {
    $mem_gb = sprintf("%.2f", $phy_ram_mb / (1024));

    print "$P Physical_RAM_MB: $phy_ram_mb\n" if !$WWW;  
    print "$P Num_Processors:  $num_cpus\n"   if !$WWW;  
    print "$P Memory_Size_GB:  $mem_gb\n"   if !$WWW;  
  }

  #---- Run "top" to collect memory usage info 
  &msg("serverInfoHPUX - Running top to gather memory usage info") if !$WWW; 
  $rc = system("top -d 1 -f $tmp1 2>&1"); 
  if ($rc != 0)
  {
    &msg("top Failed - rc=$rc - $!");
    system("cat $tmp1 | mailx -s '$P | $P rc=$rc $host/$os - [AUTO]' border\@dba.it.agilent.com");
  }

  open(TOP, "$tmp1") || &error("Cannot open input $tmp1 - $!");
  while(<TOP>)
  {
    if (/^Memory/)
    {
      $memfree = (split) [7];
    }
  }
  close(TOP); 
  system("rm -fr $tmp1 2>&1");

  if ($memfree > 0)
  {
    $memfree = sprintf("%.2f", $memfree / (1024*1024));
    $mempct = sprintf("%.2f", (($mem_gb - $memfree) / $mem_gb) * 100);
    print "$P Memory_Free_GB:  $memfree\n" if !$WWW; 
    print "$P Memory_Pct_Full: $mempct\n" if !$WWW; 
  }

  &msg("serverInfoHPUX - Done - mem_gb=$mem_gb memfree=$memfree mempct=$mempct") if !$WWW; 
}

# ----------------------------------------------------------------------
#
# Main
#
# ----------------------------------------------------------------------

$stime = time;

#
# Check to see if Oracle is up
#
if ($verb eq "up") 
{
  $runmon = $arg4;   # is this instance backup-only monitoring?

  $subrc  = &up();
  $subrc += &securityMain();    
  $subrc += &patches();    
  $subrc += &orphanDFs();
  $subrc += &rmanHung();
  $subrc += &filecheck();
  $subrc += &lsnr() unless $rq_listener_check eq "N";
  $subrc += &lan();
  $subrc += &datafiles();
  $subrc += &alertlog();
  $subrc += &aliasInfo();
  $subrc += &perf();
  $subrc += &fs() if $arg3 ne "" && $arg3 ne ".";
  $subrc += &userPassword() if $runmon ne "B" && ! $standby_flag;

  ##if ($company eq "agilent" && $host eq "acomr20")
  if ($company eq "agilent")
  {
    &oddJob1("dbamon\@bigdog");
    &oddJob1("dbamon\@lvmon01");
  }
}

#
# Check to see if the listener is up - start it if not
#
if ($verb eq "lsnr") 
{
  $subrc = &lsnr();
}

#
# Check for full tablespaces
#
if ($verb eq "ts_full") 
{
  &msg("DEBUG - Before ts_full - parallel=$parallel instnum=$instnum"); 
  if ($parallel eq "YES" && $instnum > 1)
  {
    &msg("Since this RAC/OPS instance number is > $instnum, not performing TS checks (they occur only on instance=1");
  }
  else
  {
    $subrc += &fsGeneric();
    $subrc = &ts_full();
    if ($ts_recheck) 
    { 
      $ts_where = "and a.tablespace_name in ( ";
      foreach (@recheck_list) 
      {
	$ts_where .= "'$_',"; 
      }
      chop($ts_where);
      $ts_where .= ")";
      print "$P | Re-invoking ts_full() with where clause=$ts_where \n";
      $subrc = &ts_full();
    }
  }
}

#
# Check the alertlog
#
if ($verb eq "alertlog") 
{
  $subrc = &alertlog();
}

#
# Check object extents
#
if ($verb eq "extents") 
{
  $subrc = &extents();
}

#
# Check for offline datafiles
#
if ($verb eq "datafiles") 
{
  $subrc = &datafiles();
}

#
# Check object extents
#
if ($verb eq "arclogs") 
{
  $subrc = &arclogs();
}

#
# Run a backup
#
if ($verb eq "bkrun") 
{
  $subrc = &bkrun();
}

#
# Standby inSync - routine #1
#
if ($verb eq "inSyncSby") 
{
  $subrc = &inSyncSby();
}

#
# Standby inSync - routine #2
#
if ($verb eq "inSyncPri") 
{
  $subrc = &inSyncPri();
}

# TEST
if ($verb eq "security")  { $subrc = &securityMain(); }
if ($verb eq "perftest")  { $subrc = &perf(); }
if ($verb eq "odd1test")  { $subrc = &oddJob1("dbamon\@bigdog"); }
if ($verb eq "bkagetest") { $subrc = &bkage(); }
if ($verb eq "dftest")    { &orphanDFs(); }
if ($verb eq "patches")   { &patches(); }

#
# Magnifying glass routines
#
if ($verb eq "bkbl") { $subrc = &bkbl(); }
if ($verb eq "bkps") { $subrc = &bkps(); }
if ($verb eq "bkhi") { $subrc = &bkhi(); }
if ($verb eq "bkrb") { $subrc = &bkrb(); }

#
# Terminate
#
$etime = time;
$elapsed = $etime - $stime;
&msg("Done - elapsed: $elapsed - subrc: $subrc");

&msg("Removing tmp files");
system("rm $tmp1 $tmp2 2>&1");

&msg("END_OF_PROBE");
exit 0;
