sub event
#--------------------------------------------------------------------------------
# Generic event creator
# Usage:
#   &event( 
#           "D",           (Severity: C, W, I, U, S, P)
#           1,             (Host #)
#           "Event Text",  (Text)
#           "Shorttxt",    (Short Text)
#           "DBA001I",     (MsgID)
#           "/tmp/zzz"     (Optional - File to append to msg)
#           'NOTIX'        (Optional - don't create ticket) 
#           "$emailaddr"   (Optional - email address for this event only) 
#           "H"            (Optional - email address for this event only) 
#         )
#--------------------------------------------------------------------------------
# Changes:
#  bb 07Mar03: big change - moved insert to events hist to the end. this means
#   that dups will not be logged any more - db will not grow so fast.
#  bb 27Oct05: another big change - New server_wide_event logic 
#--------------------------------------------------------------------------------
{
  ($e_sev, $e_h, $e_msg, $e_short, $e_msgid, $e_file, $e_notix, $e_email, $e_pri_override) = @_;

  if ($events_dbc[$e_h] eq "N")
  {
    &logit("Events: Set to N for this instance - NOT performing event notification",$e_h);
    return;
  }

  #---- Variables
  $e_file_grep_exclude = "^Stack|^instanceDetails:"; 

  ##foreach (@_) { print; print "\n"; }
  ##print "e_msgid=$e_msgid\n";
  ##print "e_notix=$e_notix\n";
  ##print "e_email=$e_email\n";

  # Remove all quotes and shorten if neccessary
  $_ = $e_msg;
  s/'//g;
  s/"//g;
  s/\n/ /g;
  $e_msg = $_;
  $e_msg = substr($e_msg,0,490) if length($e_msg) > 490;
  $ticket_id = "N";

  $e_msgid = "${e_msgid}${e_sev}" if length($e_msgid) < 7;
  $e_msgid_short = substr($e_msgid,0,6);

  if ($server_wide_events)
  {
    #---- See if this event is a server-wide-event. if it is, then see
    #----   if this hostname-userid has reported this for this iteration
    #----   if this is true, then skip this event
    &logit("DEBUG - e_msgid_short=$e_msgid_short swe=$server_wide_events server_events_this_iter{$hostname_userid[$e_h]}=$server_events_this_iter{$hostname_userid[$e_h]}",$e_h);
    if ($server_wide_events =~ /$e_msgid_short/)
    {
      &logit("DEBUG - This event ($e_msgid_short) is specified a server-wide-event",$e_h);

      #---- Has this event already occured during this iteration for this server_userid combo?
      if ($server_events_this_iter{$hostname_userid[$e_h]} =~ /$e_msgid_short/)  
      {
	&logit("This event ($e_msgid_short) has already occured for server/userid $hostname_userid[$e_h] - suppressing",$e_h);
        system("cat $e_file | mailx -s 'DBAmon event() $titleraw[$thishost] $e_msgid_short Suppressed - $e_msg' border\@dba.it.agilent.com") if $debug_409;
        return;
      }

      $server_events_this_iter{$hostname_userid[$e_h]} .= "${e_msgid_short},"; 
    }
  }

  # See if this event should be suppressed by the DBC Suppress_Events: parameter
  if ($suppress_events[$e_h] && index($suppress_events[$e_h],$e_msgid_short) > -1)
  {
    &logit("This event ($e_msgid_short) is specified in DBC Suppress_Events: - No notification will occur",$e_h);
    return;
  }

  if ($e_sev eq "C")
  {
    $e_str = "!!!";
    $e_color = "red";
    $e_text = "Critical";
    $m_critical[$e_h]++;
    ##print "e_msg=$e_msg e_sev=$e_sev e_short=$e_short\n";
  }
  elsif ($e_sev eq "W")
  {
    $e_str = "***";
    $e_color = "yellow";
    $e_text = "Warning";
    $m_warning[$e_h]++;
  }
  elsif ($e_sev eq "I")
  {
    $e_str = "...";
    $e_color = "cyan";
    $e_text = "Info";
    $m_info[$e_h]++ if $e_msgid ne "DBA301";
  }
  elsif ($e_sev eq "U")
  {
    $e_str = "---";
    $e_color = "green";
    $e_text = "User";
    $m_user[$e_h]++;
  }
  elsif ($e_sev eq "S")
  {
    $e_str = "###";
    $e_color = "purple";
    $e_text = "Security";
    $m_sec[$e_h]++;
  }
  elsif ($e_sev eq "P")
  {
    $e_str = "===";
    $e_color = "blue";
    $e_text = "Perf";
    $m_perf[$e_h]++;
  }
  elsif ($e_sev eq "N")
  {
    $e_str = "\@\@\@"; 
    $e_color = "lime";
    $e_text = "Notation";
    $m_note[$e_h]++;
  }
  else
  {
    &error_msg("Invalid severity: $e_sev passed to /opt/dbamon/lib/event.pl - e_h=$e_h");
  }
  
  $z = "";
  $z = "${routine}() " if $routine ne "";
  $e_msg = "$e_str ${z}$e_msg $e_str";

  $e_msgsave = $e_msgid;

  if (
      $id8[$e_h] eq "Gold" or
      $id8[$e_h] eq "Silver" or
      $id8[$e_h] eq "Bronze" or
      $id8[$e_h] eq "Copper"
     )
  {
    $e_msgmod = "DBAmon | $titleraw[$e_h] ($id8[$e_h]) | ${e_msgid}: $e_msg";
  }
  else
  {
    $e_msgmod = "DBAmon | $titleraw[$e_h] | ${e_msgid}: $e_msg";
  }

  # Check for event overrides
  if ($use_oracle eq "Y")
  {
    &eventOCheck($e_msgid_short);
    ##print "Back from eventOCheck - $e_msgid_short - ov_ticket=$ov_ticket ov_pager=$ov_pager ov_email=$ov_email \n";
    if (length("$ov_ticket$ov_pager$ov_email") > 0)
    {
      &logit("Event_Override: ov_ticket=$ov_ticket ov_pager=$ov_pager ov_email=$ov_email",$e_h);
    }
  }

  #
  #
  #

  # Write to www event log for this instance
  &eventLog($e_msg,$e_h);

  if ($e_file ne "") 
  {
    &log_file($e_file,$e_h);
    $e_long = `cat $e_file | egrep -v '$e_file_grep_exclude'`;
  }

  #
  #
  #

  # Write event text to Day log
  &logit("$e_msgid $e_text Event: $e_msg",$e_h);
  if ($e_file ne "") 
  {
    $z1 = `cat $e_file | egrep -v '$e_file_grep_exclude'`;
    $z2 = "Event Details: \n-----(Event Details Start)-----\n$z1\n------(Event Details End)------"; 
    &logit($z2,$e_h);
  }

  # Add to the list of short messages for this instance
  if ($e_short ne "")
  {
    $z = "<font color=$e_color>$e_short</font>";
    if (index($msgs[$e_h],$z) == -1) { $msgs[$e_h] .= "$z|"; }
  }
  
  # Save this message for the next iteration - prevent consecutive dup messages

  chomp($z_msg      = $e_msg);
  chomp($z_host     = $titleraw[$e_h]);
  chomp($z_userlist = $userlist[$h_userlist[$e_h]]);
  $savemsg = "host=$z_host userlist=$z_userlist msg=$z_msg";
  system("echo '$savemsg' >> $tmpdir/dbamon_savemsg_next");   # Hmmmm

  # See if this message was logged during the last iteration
  ##print "grep '$savemsg' $tmpdir/dbamon_savemsg_this >/dev/null\n";
  ##$rc = system("grep -x '$savemsg' $tmpdir/dbamon_savemsg_this 2>&1"); 
  ##print "grep rc=$rc \n";
  ##system("cat $tmpdir/dbamon_savemsg_this");

  if (system("grep '$savemsg' $tmpdir/dbamon_savemsg_this >/dev/null") == 0)
  {
    $duplicate_flag = 1;
    ##print "Debug Info:\n";
    ##print "grep '$savemsg' $tmpdir/dbamon_savemsg_this 2>&1\n";
    ##system("grep '$savemsg' $tmpdir/dbamon_savemsg_this 2>&1");
  }
  else
  {
    $duplicate_flag = 0;
  }

  if ($duplicate_flag)
  {
    # create entries for dbamon.event_detail
    #push @eventdb_ticket,   "N/A Dup-1";
    #push @eventdb_text,     "$titleraw[$h] $e_msg";
    #push @eventdb_sev,      $e_sev;
    #push @eventdb_msgid,    $e_msgid_short;
    #push @eventdb_instance, $h;

    &logit("NOT Performing Notification for Consecutive Duplicate Message",$e_h);

    return;
  }

  # 
  # tmpfl5 will contain the header and text of message
  #
  system("rm $tmpfl5 2>/dev/null");
  $date = &getdate();
  open(TMPFL5,  ">$tmpfl5") || &error_msg("event(#1) - Can't open output $tmpfl5 - $!");
 
  $mailformat = "html";   # html or text

  if ($mailformat eq "text")
  {
    print TMPFL5 "--------------- DBAmon Event Notification ---------------\n";
    print TMPFL5 "  Instance: $title[$e_h]\n";
    print TMPFL5 "      DBMS: $dbms[$e_h]\n";
    print TMPFL5 " Timestamp: $date\n";
    print TMPFL5 "  Severity: $e_text\n";
    print TMPFL5 "     Event: <font color=$e_color><b>$e_msg</b></font>\n";
    print TMPFL5 "---------------------------------------------------------\n";
    if ($e_msgsave ne "") 
    {
      print TMPFL5 "\nFor more info on this event, see: <a href='$dbamon_home/errors.shtml#$e_msgid_short'<b>DBAmon Event Reference: $e_msgsave</b></a>\n";
    }
    if ($e_file ne "") 
    {
      print TMPFL5 "<hr>";
      print TMPFL5 "<b>Diagnostic Data ($e_file):</b><br>\n";
      print TMPFL5 `cat $e_file | egrep -v '$e_file_grep_exclude'`;
      print TMPFL5 "<hr>";
    }
    $htmlsuf = " ";
  }
  else
  {
    print TMPFL5 "</pre> <font face=$fontdq size=2>
<table border=0 width=90%>
<tr bgcolor=darkblue><td><center><font color=white size=6>DBAmon</font></center>
<tr bgcolor=wheat><td><center><b>Event Notification</b></center>
</table>
<ul>
  <li><b>Instance: </b> $title[$e_h]
  <li><b>DBMS: </b> $dbms[$e_h]
  <li><b>Timestamp: </b> $date
  <li><b>Severity: </b> <font color=$e_color>$e_text</font>
  <li><b>Event: <font color=$e_color>$e_msg</b></font> \n
  <li><b>DBAmon Mode: </b> $modeout \n";
    if ($e_msgsave ne "") 
    {
      print TMPFL5 "<li><b>Event URL: <a href='$dbamon_home/errors.shtml#$e_msgid_short'>DBAmon Event Reference: $e_msgsave</b></a>\n";
    }
    print TMPFL5 "</ul><hr>";
    if ($e_file ne "") 
    {
      print TMPFL5 "<b>Diagnostic Data ($e_file):</b><pre>\n";
      print TMPFL5 "<b>=============== Start of Probe Output ===============</b>\n";
      print TMPFL5 `cat $e_file | egrep -v '$e_file_grep_exclude'`;
      print TMPFL5 "<b>================ End of Probe Output ================</b>\n";
      print TMPFL5 "</pre><hr>";
    }
    $htmlsuf = " -h";
  }

  close(TMPFL5);
  # TEST
  #system("cp $tmpfl5 /tmp/z");

  #
  # Optionally create ticket 
  #
  if (
      $e_sev eq "C" && 
      $make_tickets eq "Y" && 
      $make_tickets_dbc[$e_h] ne "N" && 
      -x "/opt/dbamon/bin/userexit/ticket" && 
      $ov_ticket ne "N" && 
      $e_notix ne 'NOTIX'
     )
  {
    &logit("Invoking userexit /opt/dbamon/bin/userexit/ticket to create ticket",$e_h);
    system("rm $tmpfl8 2>/dev/null");
    $ENV{"INSTANCE"}=$title[$e_h];
    $ENV{"ERRFILE"} =$tmpfl5;
    $ENV{"ERRMSG"}  =$e_msgmod;
    $ENV{"DBMS"}    =$dbms[$e_h];
    $ENV{"HOST"}    =$hostfix[$e_h];
    $ENV{"ID1"}     =$id1[$e_h];
    $ENV{"ID2"}     =$id2[$e_h];
    $ENV{"ID3"}     =$id3[$e_h];
    $ENV{"ID4"}     =$id4[$e_h];
    $ENV{"ID5"}     =$id5[$e_h];
    $ENV{"ID6"}     =$id6[$e_h];
    $ENV{"ID7"}     =$id7[$e_h];
    $ENV{"ID8"}     =$id8[$e_h];
    $ENV{"ID9"}     =$id9[$e_h];
    $ENV{"ID10"}    =$id10[$e_h];
    $ENV{"SEV"}     =$e_sev;
    $ENV{"MSGID"}   =$e_msgid;
    $ENV{"LONG"}    =$e_long;
    $ENV{"SHORT"}   =$e_short;
    $ENV{"MODE"}    =$modeout;
    $ENV{"RESULTS_FILE"} =$tmpfl8;

    if ($e_pri_override) 
    {
      $ENV{"PRIORITY"} = $e_pri_override;
    }
    else
    {
      $ENV{"PRIORITY"}=$priority[$e_h];
    }

    if ($pager_flag[$e_h] && $g_pager eq "Y" && $e_sev eq "C" && $ov_pager ne "N")
    {
      $ENV{"PAGER"} ="Y";
    }
    else
    {
      $ENV{"PAGER"} ="";
    }

    $ticket_id = "";
    system("rm $tmpfl8 2>/dev/null");

    $rc = system("/opt/dbamon/bin/userexit/ticket");
    if (-e "$tmpfl8")
    {
      chop($ticket_id = `cat $tmpfl8`);
      &logit("Resulting ticket_id=$ticket_id",e_h);
    }
  }
  #push @eventdb_ticket, $ticket_id;

  $page_sent = "";

  # Send page
  if ($pager_flag[$e_h] && $g_pager eq "Y" && $e_sev eq "C" && $ov_pager ne "N")
  {
    $page_sent = "Y";
    #
    # Optionally invoke userexit 
    #
    if (-x "/opt/dbamon/bin/userexit/pager")
    {
      &logit("Invoking userexit: pager");
      system("export D_HOST=$title[$e_h];export D_MSG='$e_msg';export D_FILE=$e_file;/opt/dbamon/bin/userexit/pager");
    }

    $mx = sprintf("Sending pager notification Subject=%s EMail=%s",$pager_subject[$e_h],$pager_email[$e_h]);
    &logit($mx,$e_h);

    # Customization for HP/Agilent
    if (($company eq "hp" || $company eq "agilent") && $pager_subject[$e_h] eq "DUMMY")
    {
      &logit("Invoking page_dbagap for hp/agilent",$e_h);
      $cmd = "echo '(DBAmon) $titleraw[$e_h] $e_msg' | /usr/local/sap/tools/page_dbagap";
      $rc = system($cmd);
      if ($debug_crit)
      {
        printf("page_dbagap rc=%s cmd=%s\n",$rc,$cmd);
      }
    }
    else
    {
      $mx = sprintf("Sending EMail to %s",$pager_email[$e_h])
      &logit($mx,$e_h);

      if ($pager_subject[$e_h] eq "") 
      {
        $pager_subject[$e_h] = "DBAmon Notification";
      }

      $cmd = "echo '(DBAmon) [$id3[$e_h]] $titleraw[$e_h]-$e_msg ' | dbamon_mailer -m -s '$pager_subject[$e_h]' $pager_email[$e_h]";
      ##print "cmd=$cmd";
      $rc = system($cmd);
    }
  }

  # 
  # If this host uses OpC, send opcmsg
  #

  if ($opc_appl[$e_h] ne "" && $opc eq "Y")
  {
    $msg_fix = $e_msg;
    $msg_fix =~ s/!//g; 

    &logit("Sending OpC Message",$e_h);

    $cmd = "remsh $opc_gw[$e_h] '/opt/OV/bin/OpC/opcmsg severity=$e_text application=\"$opc_appl[$e_h]\" object=$opc_obj[$e_h] msg_text=\"$msg_fix $opc_trailer\" msg_grp=$opc_mg[$e_h]'";

    if ($debug_opc || $debug_crit) 
    {
      printf("About to execute: %s\n",$cmd);
    }
    $rc = system($cmd);
    if ($rc != 0) 
    {
      printf("Command rc=%s\n",$rc);
    }
  }

  #
  # "Hard-Coded" email messages
  #

  ##print "e_msgid=$e_msgid\n";
  if ($e_msgid eq "DBA238W")
  {
    $email_address = "dba-sap-infmx_pdl-hosting\@agilent.com";
    $cmd = "cat $tmpfl5 | dbamon_mailer $htmlsuf -m -s '$e_msgmod' $email_address";
    $rc = system($cmd);
    &logit("Sent Hard-Coded EMail to $email_address for event $e_msgid - rc=$rc",$e_h);
  }

  #
  # Send notification
  #
  if ($e_sev eq "C" || $e_sev eq "U" || $ov_email eq "Y")
  {
    ##print "hu=($h_userlist[$e_h])\n";
    if ($h_userlist[$e_h] eq "[STAR]")
    {
      &eventNotifySTAR;
    }
    elsif ($h_userlist[$e_h] ne "")
    {
      &eventNotify;
    }
    else
    {
      &logit("Notify not sent - Notify_List: is null",$e_h);
    }

  }

  if ($e_email ne "")
  {
    $cmd = "cat $tmpfl5 | dbamon_mailer $htmlsuf -m -s '$e_msgmod' $e_email";
    $rc = system($cmd);
  }
  
  # Log to history table
  if (length($e_msg) > 495)
  {
    $e_msg = substr($e_msg,0,485) . " (Truncated)";
  }


  if ($e_sev eq "C")
  {
    $sqltext  = "insert into event_history (dbamonmode,system,title,dbms,sev,text,msgid,ticket) \n";
    $sqltext .= "values ('$modeout',
			 '$hostname[$e_h]',
			 '$title[$e_h]',
			 '$dbms[$e_h]',
			 '$e_sev',
			 '$e_msg',
			 '$e_msgid',
			 '$ticket_id')";
    runSQL($sqltext);
  }

  # insert message to repository (dbamon.events)
  &eventHistory($e_msg,$e_h);

}
#--------------------------------------------------------------------------------
1;
