#------------------------------------------------------------------------------
sub record_age
#------------------------------------------------------------------------------
{
  &msg("record_age() - Recording timestamp of this backup");


  # You need to set:
  $os          = "ABC";    #The ORACLE_SID of this DB
  $backup_type = "LVL0";   # LVL0, LVL1, ARC
  $dbsize = 500;           # The size of your DB in MB
  $elapsed = 300;          # How many seconds it ran for


  # Ensure that DBAMon user exists
  &msg("Seeing if DBAMON user exists");
  if (system("orastat -us | grep 'DBAMON'") != 0)
  {
    &msg("Oracle user DBAMON does not exist - creating");
    $ts = `orastat -ts | grep USER | head -1 | awk '{ print \$1 }'`;
    chop $ts;
    if ($ts eq "")
    {
      $ts = "SYSTEM";
    }
    $tm = `orastat -ts | grep TEMP | head -1 | awk '{ print \$1 }'`;
    chop $tm;
    if ($tm eq "")
    {
      $tm = $ts;
    }
    system("svrmgrl <<!
      connect internal;
      create user DBAMON identified by NOPASS
      default tablespace $ts temporary tablespace $tm;
      grant connect to DBAMON;
      grant unlimited tablespace to DBAMON;
    !");
    if (system("orastat -us | grep 'DBAMON'") != 0)
    {
      &error("User DBAMON still does not exist - Something is wrong"); 
    }
  }

  # Ensure that table DBAMON.BACKUP_AGE exists
  &msg("Seeing if table DBAMON.BACKUP_AGE exists");
  if (system("echo 'connect internal\nselect count(*) from DBAMON.BACKUP_AGE' | svrmgrl | grep 'does not exist'") == 0)
  {
    &msg("Table does not exist - creating");
    system('svrmgrl <<!
      connect DBAMON/NOPASS;
      create table "BACKUP_AGE" 
        ( 
          sid      char(8),
          bcklevel char(8),
          tsnum    number,
          tstext   char(20),
          dbsize   number,
          elapsed  number
        ) 
        storage (initial 4k next 4k pctincrease 0);
      exit;
    !');
    if (system("orastat -ta | grep 'DBAMON.BACKUP_AGE'") != 0)
    {
      &error("Table DBAMON.BACKUP_AGE still does not exist - Something is wrong"); 
    }
  }

  ### Delete rows of this type for this sid >= 50 days old - don't care if it fails
  &msg("Deleting old $backup_type row from DBAMON.BACKUP_AGE");
  $days50 = time() - 60*60*24*50;
  system("svrmgrl <<! > /tmp/ora_backup_ba_d_$os.out 2>&1
    connect internal;
    delete from DBAMON.BACKUP_AGE 
      where 
        bcklevel = '$backup_type' and
        sid      = '$os' and
        tsnum    < $days50;
    exit;
  !");

  # Insert new row
  chop($date = `date +%Y/%m/%d-%H:%M:%S`);
  $nowtime = time();

  &msg("Inserting new row into DBAMON.BACKUP_AGE");
  system("svrmgrl <<! > /tmp/ora_backup_ba_i_$os.out 2>&1
    connect internal;
    spool /tmp/ora_backup_ba_i_$os.out;
    set echo on;
    insert into DBAMON.BACKUP_AGE 
    (sid,bcklevel,tsnum,tstext,dbsize,elapsed) 
    values 
    ('$os','$backup_type',$nowtime,'$date',$dbsize,$elapsed);
    exit;
  !");

  system("chmod 777 /tmp/ora_backup_ba*.out");
  
  ##&msg("Contents of DBAMON.BACKUP_AGE:");
  ##system("svrmgrl <<!
  ##  connect internal;
  ##  select * from DBAMON.BACKUP_AGE;
  ##  exit;
  ##!");
}
