sub remsh
#--------------------------------------------------------------------------------
#
# remsh.pl
#
#--------------------------------------------------------------------------------
# Changes:
#  bb 01Jun04: Change to logic for retrying "AccountLocked" errors. We now want
#   to retry these because they after 5 minutes will automatically unlock. Also
#   changed remsh_sleep from 30 to 90.
#  bb 04Jun04: Moved the setting of remsh_sleep to dbamonrc.
#--------------------------------------------------------------------------------
{
  my ($c, $file, $h) = @_;

  # If we are inhibited, then don't bother
  if (&check_inhib($h) eq "Y")
  {
    $up[$h]  = "I";
    $rc_save = -2;
    return;
  }

  # Build command including prefix and suffix
  $cmdall = $cmdpfx[$h] . $c . $cmdsfx[$h];

  # Debug
  ##print "cmdpfx[$h]=$cmdpfx[$h]\n";
  ##print "c=$c\n";
  ##print "cmdsfx[$h]=$cmdsfx[$h]\n";

  # Clear, delete and init a logfile
  open(TMPFL7,">$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
  print TMPFL7 "Command Input:\n";
  print TMPFL7 "$cmdall \n";

  # Clear and delete remshlog
  open(REMSHLOG,">$remshlog") ||  &error_msg("Can't open output $remshlog - $!");

  $remsh_try = 0;
  $remsh_stop = 0;

  while ($remsh_try++ <= $remsh_retries)
  {
    # Issue command

    if ($probe_timeout > 0)
    {
      $start_remsh_time = time;

      ##$SIG{CHLD} = \&REAPER;   # TEST

      eval {
	alarm ($probe_timeout);
        $rc = system("$cmdall > $file 2> $remshlog");
	alarm (0);
      };

      ##$SIG{CHLD} = 'DEFAULT';   # TEST

      alarm(0);    # Just in case... turn off alarm
      $elapsed_remsh_time = time - $start_remsh_time;

      print "DEBUG - $titleraw[$h] - probe_timeout=$probe_timeout - elapsed_remsh_time=$elapsed_remsh_time\n" if $debug_293 eq "Y";

      if ($@)      # Did eval fail?
      {
	chop($eval_msg = $@);
	print "DBAmon Internal - remsh Exception: ($eval_msg)\n";
        if ($eval_msg =~ /TIMEOUT/)
        {
	  $up[$h] = "N";
	  $short = "ProbeTimeout";
	  open(TMPFL7,">>$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
	  print  TMPFL7 "\nThe remsh/ssh connect to $hostname[$h] ran for $elapsed_remsh_time seconds which is >= than the\n";
	  print  TMPFL7 "(dbamonrc) Probe_Timeout: value of $probe_timeout, and the command was cancelled. \n";
	  print  TMPFL7 "This may be an indication that this DB instance or server is hung.\n";
	  print  TMPFL7 "Log File(s): \n";
	  print  TMPFL7 `cat $remshlog`;
	  print  TMPFL7 `cat $file`;
	  close(TMPFL7);
          &event("C",$thishost,"Probe Connect TIMEOUT (dbamonrc:Probe_Timeout=$probe_timeout)","ProbeTimeout","DBA293",$tmpfl7);
          system("cat $tmpfl7 | dbamon_mailer -m -s 'DBAmon Remsh TIMEOUT: $titleraw[$h]' bill_border\@agilent.com") if $debug_293 eq "Y";
        }
      }
    }
    else
    {
      $rc = system("$cmdall > $file 2> $remshlog");
    }

    if (
        $comm_method[$thishost] eq "ssh" &&
        $dbms[$thishost] eq "MSSQL" &&
        $rc == 1280
       )
    {
      &logit("Changing bogus ssh rc=5 to rc=0",$h);
      $rc = 0;
    }

    #---- If remsh output is null, then set a bad RC
    $file_contents   = `cat $file`;
    $file_contents_l = length($file_contents);

    ##&logit("DEBUG: titleraw[$h]=${titleraw[$h]} file=$file c=$c file_contents_l=$file_contents_l rc=$rc");

    #---- BB 15Apr04 Test
    if ($file_contents_l < 10 && 
        $c !~ /^erase / &&
        $c !~ /^rm / &&
        $c !~ /^cksum / &&
        $rc == 0)
    {
      ##system("cat $file | dbamon_mailer -m -s 'DBAmon Remsh - $titleraw[$h] - FILE_CONTENTS_SMALL_LT_10 - l=$file_contents_l' bill_border\@agilent.com");
      &logit("DEBUG: Changing rc $rc/-2 - titleraw[$h]=${titleraw[$h]} c=$c file_contents_l=$file_contents_l");
      &logit("DEBUG:   file_contents=$file_contents");
      $rc = -2;
    }

    if ($rcdebug)  
    {
      &logit("DEBUG: h=$h cmdall=$cmdall rc=$rc ");
      &logit("DEBUG: cmdpfx=$cmdpfx[$h] c=$c cmdsfx=$cmdsfx[$h] ");
      &logit("DEBUG: remshlog contents=" . `cat $remshlog`);
      &logit("DEBUG: file $file contents=" . `cat $file`);
    }
   
    $remsherr = "";
    $pingflag = "";
    $pingable = "";

    # Capture possible error in remsherr var
    chop($remsherr = `cat $remshlog | grep -v 'Warning: Remote host denied X11 forwarding' | grep -v 'otrace/admin'`); 

    # Look for possible NT error
    if (system("grep  'AUTO-INHIBIT' $file") == 0) 
    {
      &event("I",$thishost,"DBAmon checking inhibited by AUTO-INHIBIT","InhibAuto","DBA304I","$file");
      $up[$h] = "I";
      return;
    }

    # Look for truncated probe output - will retry below
    if (
        system("grep 'Starting' $file > /dev/null ") == 0 && 
        system("grep 'dbamon' $file > /dev/null ") == 0 && 
        system("grep 'END_OF_PROBE' $file > /dev/null ") != 0
       ) 
    {
      $rc = -1;
      print REMSHLOG "END_OF_PROBE Not Found in probe output \n";
    }

    # Look for possible NT error
    if (system("grep  'Access Denied' $file") == 0) 
    {
      $rc = -1;
      print REMSHLOG "Access Denied \n";
    }

    # remsh worked - look for downloaded pgm's disappearing
    ## OLD TEST if ($remsherr =~ /not found/ || $remsherr =~ /No such file/)
    if (
        ($remsherr =~ /not found/ && $remsherr =~ /ksh: \/tmp\/dbamon_/) ||
        $remsherr =~ /open perl script/          
       )
    {
      print "dlfoff-1 $titleraw[$h] remsherr=$remsherr \n rc=$rc\n";
      print "dlfoff-1 rc=$rc\n";
      &downloadFlagOff($h);
      open(TMPFL7,">>$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
      print  TMPFL7 "Log File(s): \n";
      print  TMPFL7 `cat $remshlog`;
      print  TMPFL7 `cat $file`;
      close(TMPFL7);
      &event("W",$thishost,"Downloaded DBAmon Probe Software Not Found - Will retry download on next iteration","ProbeGone","DBA291",$tmpfl7);
      $up = "D";
      return;
    }

    # Look for compilation error
    if ($remsherr =~ /compilation error/)
    {
      &event("C",$thishost,"Connect resulted in compilation error","Compile","DBA292C",$remshlog);
    }

    if ($rc == 0)
    {
      ##if ($remsh_try > 1) 
      ##{
      ##  system("echo ' ' | dbamon_mailer -m -s 'DBAmon Remsh Retry Worked For: $title[$h] $remsh_try' bill_border\@agilent.com");
      ##}
      $must_complete = 0;

      if ($remsherr ne "")
      {
	# Test - append remshlog to output file
	open(APPEND,">>$file") ||  &error_msg("Can't open output $file - $!");
        print APPEND "\n---------- remsh Diagnostic Output: ----------\n";
	printf APPEND "Command: $cmdall\n";
	printf APPEND $remsherr;
	close(APPEND);
      }

      return;
    }

    ##print "remshlog="
    ##system("cat " remshlog)


    $conshort = "Undiagnosed";

    #---- Diagnose possible errors

    if ($remsherr =~ /Connection refused/) 
    {
      $conshort = "ConnRefused";
    }
    if ($remsherr =~ /account is currently locked/)
    {
      $conshort = "AccountLocked";
      system("cat $file     | mailx -m -s 'DBAmon Remsh AccountLocked: $titleraw[$h] - file    ' bill_border\@agilent.com");
      system("cat $remshlog | mailx -m -s 'DBAmon Remsh AccountLocked: $titleraw[$h] - remshlog' bill_border\@agilent.com");
      #---- BB 01Jun04 - Comment this line so that it retries
      #$remsh_stop = 1;
    }
    if ($remsherr =~ /Account is disabled/) 
    {
      $conshort = "AccountDisabled";
      $remsh_stop = 1;
    }
    if ($remsherr =~ /Password for this account expired/) 
    {
      $conshort = "PWExpired";
      $remsh_stop = 1;
    }
    if ($remsherr =~ /Password Expired/) 
    {
      $conshort = "PWExpired";
      $remsh_stop = 1;
    }
    if ($remsherr =~ /Access Denied/) 
    {
      $conshort = "AccessDenied";
    }
    if ($remsherr =~ /Connection timed out/) 
    {
      $conshort = "NetTimeout";
    }
    if ($remsherr =~ /Login incorrect/) 
    {
      $conshort = "LoginIncorrect";
      $remsh_stop = 1;
      print "dlfoff-2 $titleraw[$h]\n";
      &downloadFlagOff($h);
    }
    
    print "test-3 $titleraw[$h] rc=$rc remsh_stop=$remsh_stop remsherr=$remsherr conshort=$conshort\n";

    if (
        ($remsherr =~ /not found/ && $remsherr =~ /ksh: \/tmp\/dbamon_/) ||
        $remsherr =~ /open perl script/          
       )
    {
      $conshort = "SWDisappeared";
      $remsh_stop = 1;
      &downloadFlagOff($h);
      $rc = -1;    # Test
      print "dlfoff-3 $titleraw[$h] rc=$rc remsh_stop=$remsh_stop\n";
    }

    # See if END_OF_PROBE is present. If it isn't, then retry
    if (
        system("grep 'Starting' $file > /dev/null ") == 0 && 
        system("grep 'dbamon' $file > /dev/null ") == 0 && 
        system("grep 'END_OF_PROBE' $file > /dev/null ") != 0 && $remsh_stop != 1
       )
    {
      $conshort = "END_OF_PROBE_N/F";
      system("cat $file | mailx -s 'DBAmon remsh() END_OF_PROBE Not Found $titleraw[$h] remsh_try=$remsh_try remsherr=$remsherr [AUTO]' bill_border\@agilent.com") if $remsh_try > 3;
      if ($remsh_try <= $remsh_retries)
      {
	$msg = sprintf("... Remsh Connect Attempt:%s Failed - END_OF_PROBE NF - Err=[%s] RC=%i - Will Retry - Sleeping %s ...",$remsh_try,$remsherr,$rc,$remsh_sleep);
	&logit($msg,$h);
	system("sleep $remsh_sleep");
	next;
      }
    }

    # It error'd on < the last attempt
    if ($rc != 0 && $remsh_try <= $remsh_retries && $remsh_stop != 1)
    {
      # Log message
      $msg = sprintf("... Remsh Connect Attempt:%s Failed Err=[%s] RC=%i - Will Retry - Sleeping %s ...",$remsh_try,$remsherr,$rc,$remsh_sleep);
      &logit($msg,$h);
      system("sleep $remsh_sleep");
      next;
    }

    # It error'd on what will be the last attempt
    if ($rc != 0 && ($remsh_try > $remsh_retries || $remsh_stop))
    {
      # See if server is pingable
      $rc = system("ping $hostname[$h] 64 2 > $pinglog 2>&1");
      if ($rc != 0 || system("grep -i '100% packet loss' $pinglog") == 0)
      {
	$pingable = "Is *NOT* Ping-able";
        $pingflag = "N";
        print "dlfoff-4 $titleraw[$h]\n";
        &downloadFlagOff($h);
      }
      else
      {
	$pingable = "Is Ping-able";
        $pingflag = "Y";
      }

      # Create report
      open(TMPFL7,">>$tmpfl7") ||  &error_msg("Can't open output $tmpfl7 - $!");
      printf TMPFL7 "Remsh Attempt: %s RC=%s - Error Details:\n",$remsh_try,$rc;
      print  TMPFL7 `cat $remshlog`;
      print  TMPFL7 `cat $file`;
      print  TMPFL7 "\nHost: $hostname[$h] \n";
      print  TMPFL7 "remsherr: $remsherr \n";
      close(TMPFL7);

      # Log message
      if ($remsh_sev eq "C" || $must_connect)
      {
        close(TMPFL7);
        &event("C",$thishost,"Connect Failure: ($conshort) After $remsh_try Attempt(s) - Server $pingable - Err=($remsherr) RC=$rc","Connect","DBA290",$tmpfl7);
      }
      else
      {
        close(TMPFL7);
        &event("W",$thishost,"Connect Failure: ($conshort) After $remsh_try Attempt(s) - Server $pingable - Err=($remsherr) RC=$rc","Connect","DBA290",$tmpfl7);
      }

      $up[$h] = "N";
      $short = "Rsh";
      last;
    }
  } # End of retries while loop

  #    
  # Look for errors that have slipped by
  #

  ## # See if END_OF_PROBE is present. If it isn't, create event
  ## if ($up eq "" && system("grep 'END_OF_PROBE' $file") != 0)
  ## {
  ##   $msg = sprintf("... Probe END_OF_PROBE Not Found - Will NOT Retry");
  ##   &logit($msg,$h);
  ##   system("cat $file | mailx -s 'DBAmon remsh() END_OF_PROBE Not Found $titleraw[$h] Will NOT Retry remsherr=$remsherr [AUTO]' bill_border\@agilent.com");
  ## }

  # Set vars
  $rc_save = $rc;
  $must_connect = 0;
}

#-------------------------------------------------------------------------------
1;
